@@ -8,6 +8,7 @@ import keyword
 import re
 import sys
 import urllib.parse
+from collections import Counter
 from pathlib import Path
 from typing import (
     TYPE_CHECKING,
@@ -20,11 +21,7 @@ from typing import (
     MutableMapping,
 )
 
-if TYPE_CHECKING:
-    import pandas as pd
-    import pyarrow as pa
-
-    import ibis.expr.schema as sch
+from bidict import MutableBidirectionalMapping, bidict
 
 import ibis
 import ibis.common.exceptions as exc
@@ -33,6 +30,10 @@ import ibis.expr.operations as ops
 import ibis.expr.types as ir
 from ibis import util
 
+if TYPE_CHECKING:
+    import pandas as pd
+    import pyarrow as pa
+
 __all__ = ('BaseBackend', 'Database', 'connect')
 
 
@@ -206,16 +207,21 @@ class TablesAccessor(collections.abc.Mapping):
         )
         return list(o)
 
+    def __repr__(self) -> str:
+        tables = self._backend.list_tables()
+        rows = ["Tables", "------"]
+        rows.extend(f"- {name}" for name in sorted(tables))
+        return "\n".join(rows)
+
     def _ipython_key_completions_(self) -> list[str]:
         return self._backend.list_tables()
 
 
-# should have a better name
-class ResultHandler:
+class _FileIOHandler:
     @staticmethod
     def _import_pyarrow():
         try:
-            import pyarrow
+            import pyarrow  # noqa: ICN001
         except ImportError:
             raise ModuleNotFoundError(
                 "Exporting to arrow formats requires `pyarrow` but it is not installed"
@@ -223,16 +229,6 @@ class ResultHandler:
         else:
             return pyarrow
 
-    @staticmethod
-    def _table_or_column_schema(expr: ir.Expr) -> sch.Schema:
-        from ibis.backends.pyarrow.datatypes import sch
-
-        if isinstance(expr, ir.Table):
-            return expr.schema()
-        else:
-            # ColumnExpr has no schema method, define single-column schema
-            return sch.schema([(expr.get_name(), expr.type())])
-
     @util.experimental
     def to_pyarrow(
         self,
@@ -275,7 +271,7 @@ class ResultHandler:
         except ValueError:
             # The pyarrow batches iterator is empty so pass in an empty
             # iterator and a pyarrow schema
-            schema = self._table_or_column_schema(expr)
+            schema = expr.as_table().schema()
             table = pa.Table.from_batches([], schema=schema.to_pyarrow())
 
         if isinstance(expr, ir.Table):
@@ -317,7 +313,7 @@ class ResultHandler:
         params
             Mapping of scalar parameter expressions to value.
         chunk_size
-            Number of rows in each returned record batch.
+            Maximum number of rows in each returned record batch.
         kwargs
             Keyword arguments
 
@@ -328,8 +324,128 @@ class ResultHandler:
         """
         raise NotImplementedError
 
+    def read_parquet(
+        self, path: str | Path, table_name: str | None = None, **kwargs: Any
+    ) -> ir.Table:
+        """Register a parquet file as a table in the current backend.
 
-class BaseBackend(abc.ABC, ResultHandler):
+        Parameters
+        ----------
+        path
+            The data source.
+        table_name
+            An optional name to use for the created table. This defaults to
+            a sequentially generated name.
+        **kwargs
+            Additional keyword arguments passed to the backend loading function.
+
+        Returns
+        -------
+        ir.Table
+            The just-registered table
+        """
+        raise NotImplementedError(
+            f"{self.name} does not support direct registration of parquet data."
+        )
+
+    def read_csv(
+        self, path: str | Path, table_name: str | None = None, **kwargs: Any
+    ) -> ir.Table:
+        """Register a CSV file as a table in the current backend.
+
+        Parameters
+        ----------
+        path
+            The data source. A string or Path to the CSV file.
+        table_name
+            An optional name to use for the created table. This defaults to
+            a sequentially generated name.
+        **kwargs
+            Additional keyword arguments passed to the backend loading function.
+
+        Returns
+        -------
+        ir.Table
+            The just-registered table
+        """
+        raise NotImplementedError(
+            f"{self.name} does not support direct registration of CSV data."
+        )
+
+    @util.experimental
+    def to_parquet(
+        self,
+        expr: ir.Table,
+        path: str | Path,
+        *,
+        params: Mapping[ir.Scalar, Any] | None = None,
+        **kwargs: Any,
+    ) -> None:
+        """Write the results of executing the given expression to a parquet file.
+
+        This method is eager and will execute the associated expression
+        immediately.
+
+        Parameters
+        ----------
+        expr
+            The ibis expression to execute and persist to parquet.
+        path
+            The data source. A string or Path to the parquet file.
+        params
+            Mapping of scalar parameter expressions to value.
+        **kwargs
+            Additional keyword arguments passed to pyarrow.parquet.ParquetWriter
+
+        https://arrow.apache.org/docs/python/generated/pyarrow.parquet.ParquetWriter.html
+        """
+        self._import_pyarrow()
+        import pyarrow.parquet as pq
+
+        batch_reader = expr.to_pyarrow_batches(params=params)
+
+        with pq.ParquetWriter(path, batch_reader.schema) as writer:
+            for batch in batch_reader:
+                writer.write_batch(batch)
+
+    @util.experimental
+    def to_csv(
+        self,
+        expr: ir.Table,
+        path: str | Path,
+        *,
+        params: Mapping[ir.Scalar, Any] | None = None,
+        **kwargs: Any,
+    ) -> None:
+        """Write the results of executing the given expression to a CSV file.
+
+        This method is eager and will execute the associated expression
+        immediately.
+
+        Parameters
+        ----------
+        expr
+            The ibis expression to execute and persist to CSV.
+        path
+            The data source. A string or Path to the CSV file.
+        params
+            Mapping of scalar parameter expressions to value.
+        kwargs
+            Additional keyword arguments passed to pyarrow.csv.CSVWriter
+
+        https://arrow.apache.org/docs/python/generated/pyarrow.csv.CSVWriter.html
+        """
+        self._import_pyarrow()
+        import pyarrow.csv as pcsv
+
+        batch_reader = expr.to_pyarrow_batches(params=params)
+
+        with pcsv.CSVWriter(path, batch_reader.schema) as writer:
+            for batch in batch_reader:
+                writer.write_batch(batch)
+
+
+class BaseBackend(abc.ABC, _FileIOHandler):
     """Base backend class.
 
     All Ibis backends must subclass this class and implement all the
@@ -343,6 +459,12 @@ class BaseBackend(abc.ABC, ResultHandler):
     def __init__(self, *args, **kwargs):
         self._con_args: tuple[Any] = args
         self._con_kwargs: dict[str, Any] = kwargs
+        # expression cache
+        self._query_cache: MutableBidirectionalMapping[
+            ops.TableNode, ops.PhysicalTable
+        ] = bidict()
+
+        self._refs = Counter()
 
     def __getstate__(self):
         return dict(
@@ -352,6 +474,9 @@ class BaseBackend(abc.ABC, ResultHandler):
             _con_kwargs=self._con_kwargs,
         )
 
+    def __rich_repr__(self):
+        yield "name", self.name
+
     def __hash__(self):
         return hash(self.db_identity)
 
@@ -383,10 +508,10 @@ class BaseBackend(abc.ABC, ResultHandler):
 
         Parameters
         ----------
-        args
+        *args
             Mandatory connection parameters, see the docstring of `do_connect`
             for details.
-        kwargs
+        **kwargs
             Extra connection parameters, see the docstring of `do_connect` for
             details.
 
@@ -522,6 +647,23 @@ class BaseBackend(abc.ABC, ResultHandler):
             The list of the table names that match the pattern `like`.
         """
 
+    @abc.abstractmethod
+    def table(self, name: str, database: str | None = None) -> ir.Table:
+        """Construct a table expression.
+
+        Parameters
+        ----------
+        name
+            Table name
+        database
+            Database name
+
+        Returns
+        -------
+        Table
+            Table expression
+        """
+
     @functools.cached_property
     def tables(self):
         """An accessor for tables in the database.
@@ -575,6 +717,14 @@ class BaseBackend(abc.ABC, ResultHandler):
         """Compile an expression."""
         return self.compiler.to_sql(expr, params=params)
 
+    def _to_sql(self, expr: ir.Expr, **kwargs) -> str:
+        """Convert an expression to a SQL string.
+
+        Called by `ibis.to_sql`/`ibis.show_sql`, gives the backend an
+        opportunity to generate nicer SQL for human consumption.
+        """
+        raise NotImplementedError(f"Backend '{self.name}' backend doesn't support SQL")
+
     def execute(self, expr: ir.Expr) -> Any:
         """Execute an expression."""
 
@@ -619,17 +769,19 @@ class BaseBackend(abc.ABC, ResultHandler):
             f'Backend "{self.name}" does not implement "create_database"'
         )
 
+    @abc.abstractmethod
     def create_table(
         self,
         name: str,
         obj: pd.DataFrame | ir.Table | None = None,
+        *,
         schema: ibis.Schema | None = None,
         database: str | None = None,
-    ) -> None:
+        temp: bool = False,
+        overwrite: bool = False,
+    ) -> ir.Table:
         """Create a new table.
 
-        Not all backends implement this method.
-
         Parameters
         ----------
         name
@@ -644,14 +796,22 @@ class BaseBackend(abc.ABC, ResultHandler):
         database
             Name of the database where the table will be created, if not the
             default.
-        """
-        raise NotImplementedError(
-            f'Backend "{self.name}" does not implement "create_table"'
-        )
+        temp
+            Whether a table is temporary or not
+        overwrite
+            Whether to clobber existing data
 
+        Returns
+        -------
+        Table
+            The table that was created.
+        """
+
+    @abc.abstractmethod
     def drop_table(
         self,
         name: str,
+        *,
         database: str | None = None,
         force: bool = False,
     ) -> None:
@@ -670,31 +830,38 @@ class BaseBackend(abc.ABC, ResultHandler):
             f'Backend "{self.name}" does not implement "drop_table"'
         )
 
+    @abc.abstractmethod
     def create_view(
         self,
         name: str,
-        expr: ir.Table,
+        obj: ir.Table,
+        *,
         database: str | None = None,
-    ) -> None:
-        """Create a view.
+        overwrite: bool = False,
+    ) -> ir.Table:
+        """Create a new view from an expression.
 
         Parameters
         ----------
         name
-            Name for the new view.
-        expr
-            An Ibis table expression that will be used to extract the query
-            of the view.
+            Name of the new view.
+        obj
+            An Ibis table expression that will be used to create the view.
         database
-            Name of the database where the view will be created, if not the
-            default.
-        """
-        raise NotImplementedError(
-            f'Backend "{self.name}" does not implement "create_view"'
-        )
+            Name of the database where the view will be created, if not
+            provided the database's default is used.
+        overwrite
+            Whether to clobber an existing view with the same name
 
+        Returns
+        -------
+        Table
+            The view that was created.
+        """
+
+    @abc.abstractmethod
     def drop_view(
-        self, name: str, database: str | None = None, force: bool = False
+        self, name: str, *, database: str | None = None, force: bool = False
     ) -> None:
         """Drop a view.
 
@@ -707,9 +874,6 @@ class BaseBackend(abc.ABC, ResultHandler):
         force
             If `False`, an exception is raised if the view does not exist.
         """
-        raise NotImplementedError(
-            f'Backend "{self.name}" does not implement "drop_view"'
-        )
 
     @classmethod
     def has_operation(cls, operation: type[ops.Value]) -> bool:
@@ -738,6 +902,57 @@ class BaseBackend(abc.ABC, ResultHandler):
             f"{cls.name} backend has not implemented `has_operation` API"
         )
 
+    def _cached(self, expr):
+        """Cache the provided expression.
+
+        All subsequent operations on the returned expression will be performed on the cached data.
+
+        Parameters
+        ----------
+        expr
+            Table expression to cache
+
+        Returns
+        -------
+        Expr
+            Cached table
+
+        """
+        op = expr.op()
+        if (result := self._query_cache.get(op)) is None:
+            name = util.generate_unique_table_name("cache")
+            self._load_into_cache(name, expr)
+            self._query_cache[op] = result = self.table(name).op()
+        self._refs[op] += 1
+        return ir.CachedTable(result)
+
+    def _release_cached(self, expr):
+        """Releases the provided cached expression.
+
+        Parameters
+        ----------
+        expr
+            Cached expression to release
+        """
+        op = expr.op()
+        # we need to remove the expression representing the temp table as well
+        # as the expression that was used to create the temp table
+        #
+        # bidict automatically handles this for us; without it we'd have to
+        # do to the bookkeeping ourselves with two dicts
+        if (key := self._query_cache.inverse.get(op)) is None:
+            raise exc.IbisError(
+                "This expression has already been released. Did you call "
+                "`.release()` twice on the same expression?"
+            )
+
+        self._refs[key] -= 1
+
+        if not self._refs[key]:
+            del self._query_cache[key]
+            del self._refs[key]
+            self._clean_up_cached_table(op)
+
 
 @functools.lru_cache(maxsize=None)
 def _get_backend_names() -> frozenset[str]:
