@@ -6,16 +6,23 @@ import operator
 from typing import Any
 
 import sqlalchemy as sa
+from sqlalchemy.sql.functions import FunctionElement, GenericFunction
 
 import ibis.common.exceptions as com
 import ibis.expr.analysis as an
 import ibis.expr.datatypes as dt
 import ibis.expr.operations as ops
 import ibis.expr.types as ir
-import ibis.expr.window as W
 from ibis.backends.base.sql.alchemy.database import AlchemyTable
 
 
+class substr(GenericFunction):
+    """A generic substr function, so dialects can customize compilation."""
+
+    type = sa.types.String()
+    inherit_cache = True
+
+
 def variance_reduction(func_name):
     suffix = {'sample': 'samp', 'pop': 'pop'}
 
@@ -72,33 +79,35 @@ def get_sqla_table(ctx, table):
         while sa_table is None and ctx_level.parent is not ctx_level:
             ctx_level = ctx_level.parent
             sa_table = ctx_level.get_ref(table)
+    elif isinstance(table, AlchemyTable):
+        sa_table = table.sqla_table
     else:
-        if isinstance(table, AlchemyTable):
-            sa_table = table.sqla_table
-        else:
-            sa_table = ctx.get_compiled_expr(table)
+        sa_table = ctx.get_compiled_expr(table)
 
     return sa_table
 
 
-def get_col_or_deferred_col(sa_table, colname):
-    """Get a `Column`, or create a "deferred" column.
+def get_col(sa_table, op: ops.TableColumn) -> sa.sql.ColumnClause:
+    """Extract a column from a table."""
+    cols = sa_table.exported_columns
+    colname = op.name
 
-    This is to handle the case when selecting a column from a join, which
-    happens when a join expression is cached during join traversal
+    if (col := cols.get(colname)) is not None:
+        return col
 
-    We'd like to avoid generating a subquery just for selection but in
-    sqlalchemy the Join object is not selectable. However, at this point
-    know that the column can be referred to unambiguously
-
-    Later the expression is assembled into
-    `sa.select([sa.column(colname)]).select_from(table_set)` (roughly)
-    where `table_set` is `sa_table` above.
-    """
-    try:
-        return sa_table.exported_columns[colname]
-    except KeyError:
-        return sa.column(colname)
+    # `cols` is a SQLAlchemy column collection that contains columns
+    # with names that are secretly prefixed by table that contains them
+    #
+    # for example, in `t0.join(t1).select(t0.a, t1.b)` t0.a will be named `t0_a`
+    # and t1.b will be named `t1_b`
+    #
+    # unfortunately SQLAlchemy doesn't let you select by the *un*prefixed
+    # column name despite the uniqueness of `colname`
+    #
+    # however, in ibis we have already deduplicated column names so we can
+    # refer to the name by position
+    colindex = op.table.schema._name_locs[colname]
+    return cols[colindex]
 
 
 def _table_column(t, op):
@@ -107,7 +116,7 @@ def _table_column(t, op):
 
     sa_table = get_sqla_table(ctx, table)
 
-    out_expr = get_col_or_deferred_col(sa_table, op.name)
+    out_expr = get_col(sa_table, op)
     out_expr.quote = t._always_quote_columns
 
     # If the column does not originate from the table set in the current SELECT
@@ -123,14 +132,21 @@ def _table_column(t, op):
 
 
 def _table_array_view(t, op):
+    # the table that the TableArrayView op contains (op.table) has
+    # one or more input relations that we need to "pin" for sqlalchemy's
+    # auto correlation functionality -- this is what `.correlate_except` does
+    #
+    # every relation that is NOT passed to `correlate_except` is considered an
+    # outer-query table
     ctx = t.context
     table = ctx.get_compiled_expr(op.table)
-    return table
+    # TODO: handle the case of `op.table` being a join
+    first, *_ = an.find_immediate_parent_tables(op.table, keep_input=False)
+    ref = ctx.get_ref(first)
+    return table.correlate_except(ref)
 
 
 def _exists_subquery(t, op):
-    from ibis.backends.base.sql.alchemy.query_builder import AlchemyCompiler
-
     ctx = t.context
 
     # TODO(kszucs): avoid converting the predicates to expressions
@@ -138,11 +154,11 @@ def _exists_subquery(t, op):
     filtered = (
         op.foreign_table.to_expr()
         .filter([pred.to_expr() for pred in op.predicates])
-        .projection([ir.literal(1).name(ir.core.unnamed)])
+        .projection([ir.literal(1).name("")])
     )
 
     sub_ctx = ctx.subcontext()
-    clause = AlchemyCompiler.to_sql(filtered, sub_ctx, exists=True)
+    clause = ctx.compiler.to_sql(filtered, sub_ctx, exists=True)
 
     if isinstance(op, ops.NotExistsSubquery):
         clause = sa.not_(clause)
@@ -153,18 +169,15 @@ def _exists_subquery(t, op):
 def _cast(t, op):
     arg = op.arg
     typ = op.to
+    arg_dtype = arg.output_dtype
 
     sa_arg = t.translate(arg)
-    sa_type = t.get_sqla_type(typ)
-
-    if isinstance(arg, ir.CategoryValue) and typ == dt.int32:
-        return sa_arg
 
     # specialize going from an integer type to a timestamp
-    if arg.output_dtype.is_integer() and isinstance(sa_type, sa.DateTime):
+    if arg_dtype.is_integer() and typ.is_timestamp():
         return t.integer_to_timestamp(sa_arg)
 
-    if arg.output_dtype.is_binary() and typ.is_string():
+    if arg_dtype.is_binary() and typ.is_string():
         return sa.func.encode(sa_arg, 'escape')
 
     if typ.is_binary():
@@ -174,21 +187,23 @@ def _cast(t, op):
 
     if typ.is_json() and not t.native_json_type:
         return sa_arg
-    return sa.cast(sa_arg, sa_type)
+
+    return sa.cast(sa_arg, t.get_sqla_type(typ))
 
 
 def _contains(func):
     def translate(t, op):
         left = t.translate(op.value)
 
-        if isinstance(op.options, tuple):
+        options = op.options
+        if isinstance(options, tuple):
             right = [t.translate(x) for x in op.options]
-        elif op.options.output_shape.is_columnar():
-            right = t.translate(op.options)
+        elif options.output_shape.is_columnar():
+            right = t.translate(ops.TableArrayView(options.to_expr().as_table()))
             if not isinstance(right, sa.sql.Selectable):
                 right = sa.select(right)
         else:
-            right = t.translate(op.options)
+            right = t.translate(options)
 
         return func(left, right)
 
@@ -205,6 +220,9 @@ def _literal(_, op):
     dtype = op.output_dtype
     value = op.value
 
+    if value is None:
+        return sa.null()
+
     if dtype.is_set():
         return list(map(sa.literal, value))
     elif dtype.is_array():
@@ -220,7 +238,7 @@ def _is_null(t, op):
 
 def _not_null(t, op):
     arg = t.translate(op.arg)
-    return arg.isnot(sa.null())
+    return arg.is_not(sa.null())
 
 
 def _round(t, op):
@@ -242,22 +260,19 @@ def _floor_divide(t, op):
 
 
 def _simple_case(t, op):
-    cases = [ops.Equals(op.base, case) for case in op.cases]
-    return _translate_case(t, cases, op.results, op.default)
+    return _translate_case(t, op, value=t.translate(op.base))
 
 
 def _searched_case(t, op):
-    return _translate_case(t, op.cases, op.results, op.default)
+    return _translate_case(t, op, value=None)
 
 
-def _translate_case(t, cases, results, default):
-    case_args = [t.translate(arg) for arg in cases]
-    result_args = [t.translate(arg) for arg in results]
-
-    whens = zip(case_args, result_args)
-    default = t.translate(default)
-
-    return sa.case(list(whens), else_=default)
+def _translate_case(t, op, *, value):
+    return sa.case(
+        *zip(map(t.translate, op.cases), map(t.translate, op.results)),
+        value=value,
+        else_=t.translate(op.default),
+    )
 
 
 def _negate(t, op):
@@ -292,69 +307,73 @@ _cumulative_to_reduction = {
 }
 
 
-def _cumulative_to_window(translator, op, window):
-    win = W.cumulative_window()
-    win = win.group_by(window._group_by).order_by(window._order_by)
-
+def _cumulative_to_window(translator, op, frame):
     klass = _cumulative_to_reduction[type(op)]
     new_op = klass(*op.args)
     new_expr = new_op.to_expr().name(op.name)
+    new_frame = frame.copy(start=None, end=0)
 
     if type(new_op) in translator._rewrites:
         new_expr = translator._rewrites[type(new_op)](new_expr)
 
     # TODO(kszucs): rewrite to receive and return an ops.Node
-    return an.windowize_function(new_expr, win)
+    return an.windowize_function(new_expr, frame=new_frame)
 
 
-def _window(t, op):
-    arg, window = op.args
-    reduction = t.translate(arg)
+def _translate_window_boundary(boundary):
+    if boundary is None:
+        return None
 
-    window_op = arg
+    if isinstance(boundary.value, ops.Literal):
+        if boundary.preceding:
+            return -boundary.value.value
+        else:
+            return boundary.value.value
 
-    if isinstance(window_op, ops.CumulativeOp):
-        arg = _cumulative_to_window(t, arg, window).op()
-        return t.translate(arg)
+    raise com.TranslationError("Window boundaries must be literal values")
 
-    if window.max_lookback is not None:
-        raise NotImplementedError(
-            'Rows with max lookback is not implemented '
-            'for SQLAlchemy-based backends.'
-        )
 
-    # Checks for invalid user input e.g. passing in tuple for preceding and
-    # non-None value for following are caught and raised in expr/window.py
-    # if we're here, then the input is valid, we just need to interpret it
-    # correctly
-    if isinstance(window.preceding, tuple):
-        start, end = (-1 * x if x is not None else None for x in window.preceding)
-    elif isinstance(window.following, tuple):
-        start, end = window.following
-    else:
-        start = -window.preceding if window.preceding is not None else window.preceding
-        end = window.following
+def _window_function(t, window):
+    if isinstance(window.func, ops.CumulativeOp):
+        func = _cumulative_to_window(t, window.func, window.frame).op()
+        return t.translate(func)
+
+    reduction = t.translate(window.func)
 
     # Some analytic functions need to have the expression of interest in
     # the ORDER BY part of the window clause
-    if isinstance(window_op, t._require_order_by) and not window._order_by:
-        order_by = t.translate(window_op.args[0])
+    if isinstance(window.func, t._require_order_by) and not window.frame.order_by:
+        order_by = t.translate(window.func.arg)  # .args[0])
     else:
-        order_by = [t.translate(arg) for arg in window._order_by]
+        order_by = [t.translate(arg) for arg in window.frame.order_by]
 
-    partition_by = [t.translate(arg) for arg in window._group_by]
+    partition_by = [t.translate(arg) for arg in window.frame.group_by]
+
+    if isinstance(window.frame, ops.RowsWindowFrame):
+        if window.frame.max_lookback is not None:
+            raise NotImplementedError(
+                'Rows with max lookback is not implemented for SQLAlchemy-based '
+                'backends.'
+            )
+        how = 'rows'
+    elif isinstance(window.frame, ops.RangeWindowFrame):
+        how = 'range_'
+    else:
+        raise NotImplementedError(type(window.frame))
+
+    if t._forbids_frame_clause and isinstance(window.func, t._forbids_frame_clause):
+        # some functions on some backends don't support frame clauses
+        additional_params = {}
+    else:
+        start = _translate_window_boundary(window.frame.start)
+        end = _translate_window_boundary(window.frame.end)
+        additional_params = {how: (start, end)}
 
-    how = {'range': 'range_'}.get(window.how, window.how)
-    additional_params = (
-        {}
-        if t._forbids_frame_clause and isinstance(window_op, t._forbids_frame_clause)
-        else {how: (start, end)}
-    )
     result = reduction.over(
         partition_by=partition_by, order_by=order_by, **additional_params
     )
 
-    if isinstance(window_op, (ops.RowNumber, ops.DenseRank, ops.MinRank, ops.NTile)):
+    if isinstance(window.func, (ops.RowNumber, ops.DenseRank, ops.MinRank, ops.NTile)):
         return result - 1
     else:
         return result
@@ -400,18 +419,27 @@ def reduction(sa_func):
 def _zero_if_null(t, op):
     sa_arg = t.translate(op.arg)
     return sa.case(
-        [(sa_arg.is_(None), sa.cast(0, t.get_sqla_type(op.output_dtype)))],
+        (sa_arg.is_(None), sa.cast(0, t.get_sqla_type(op.output_dtype))),
         else_=sa_arg,
     )
 
 
 def _substring(t, op):
-    args = t.translate(op.arg), t.translate(op.start) + 1
-
-    if (length := op.length) is not None:
-        args += (t.translate(length),)
-
-    return sa.func.substr(*args)
+    sa_arg = t.translate(op.arg)
+    sa_start = t.translate(op.start) + 1
+    # Start is an expression, need a runtime branch
+    sa_arg_length = t.translate(ops.StringLength(op.arg))
+    if op.length is None:
+        return sa.case(
+            ((sa_start >= 1), sa.func.substr(sa_arg, sa_start)),
+            else_=sa.func.substr(sa_arg, sa_start + sa_arg_length),
+        )
+    else:
+        sa_length = t.translate(op.length)
+        return sa.case(
+            ((sa_start >= 1), sa.func.substr(sa_arg, sa_start, sa_length)),
+            else_=sa.func.substr(sa_arg, sa_start + sa_arg_length, sa_length),
+        )
 
 
 def _gen_string_find(func):
@@ -431,21 +459,6 @@ def _nth_value(t, op):
     return sa.func.nth_value(t.translate(op.arg), t.translate(op.nth) + 1)
 
 
-def _clip(*, min_func, max_func):
-    def translate(t, op):
-        arg = t.translate(op.arg)
-
-        if (upper := op.upper) is not None:
-            arg = min_func(t.translate(upper), arg)
-
-        if (lower := op.lower) is not None:
-            arg = max_func(t.translate(lower), arg)
-
-        return arg
-
-    return translate
-
-
 def _bitwise_op(operator):
     def translate(t, op):
         left = t.translate(op.left)
@@ -480,6 +493,14 @@ def _extract(fmt: str):
     return translator
 
 
+class array_map(FunctionElement):
+    pass
+
+
+class array_filter(FunctionElement):
+    pass
+
+
 sqlalchemy_operation_registry: dict[Any, Any] = {
     ops.Alias: _alias,
     ops.And: fixed_arity(operator.and_, 2),
@@ -511,7 +532,6 @@ sqlalchemy_operation_registry: dict[Any, Any] = {
     ops.NotNull: _not_null,
     ops.Negate: _negate,
     ops.Round: _round,
-    ops.TypeOf: unary(sa.func.typeof),
     ops.Literal: _literal,
     ops.NullLiteral: lambda *_: sa.null(),
     ops.SimpleCase: _simple_case,
@@ -524,6 +544,11 @@ sqlalchemy_operation_registry: dict[Any, Any] = {
     ops.Least: varargs(sa.func.least),
     ops.Greatest: varargs(sa.func.greatest),
     # string
+    ops.Capitalize: unary(
+        lambda arg: sa.func.concat(
+            sa.func.upper(sa.func.substr(arg, 1, 1)), sa.func.substr(arg, 2)
+        )
+    ),
     ops.LPad: fixed_arity(sa.func.lpad, 3),
     ops.RPad: fixed_arity(sa.func.rpad, 3),
     ops.Strip: unary(sa.func.trim),
@@ -594,10 +619,9 @@ sqlalchemy_operation_registry: dict[Any, Any] = {
     ops.IdenticalTo: fixed_arity(
         sa.sql.expression.ColumnElement.is_not_distinct_from, 2
     ),
-    ops.Clip: _clip(min_func=sa.func.least, max_func=sa.func.greatest),
     ops.Where: fixed_arity(
         lambda predicate, value_if_true, value_if_false: sa.case(
-            [(predicate, value_if_true)],
+            (predicate, value_if_true),
             else_=value_if_false,
         ),
         3,
@@ -616,6 +640,7 @@ sqlalchemy_operation_registry: dict[Any, Any] = {
     ops.ExtractHour: _extract('hour'),
     ops.ExtractMinute: _extract('minute'),
     ops.ExtractSecond: _extract('second'),
+    ops.Time: fixed_arity(lambda arg: sa.cast(arg, sa.TIME), 1),
 }
 
 
@@ -631,12 +656,13 @@ sqlalchemy_window_functions_registry = {
     ops.PercentRank: unary(lambda _: sa.func.percent_rank()),
     ops.CumeDist: unary(lambda _: sa.func.cume_dist()),
     ops.NthValue: _nth_value,
-    ops.Window: _window,
-    ops.CumulativeOp: _window,
+    ops.WindowFunction: _window_function,
     ops.CumulativeMax: unary(sa.func.max),
     ops.CumulativeMin: unary(sa.func.min),
     ops.CumulativeSum: unary(sa.func.sum),
     ops.CumulativeMean: unary(sa.func.avg),
+    ops.CumulativeAny: unary(sa.func.bool_or),
+    ops.CumulativeAll: unary(sa.func.bool_and),
 }
 
 geospatial_functions = {
