@@ -15,7 +15,7 @@ from ibis.backends.base.sql.registry.main import (
 )
 from ibis.backends.base.sql.registry.window import (
     cumulative_to_window,
-    format_window,
+    format_window_frame,
     time_range_to_range_window,
 )
 
@@ -31,6 +31,6 @@ __all__ = (
     'reduction',
     'unary',
     'cumulative_to_window',
-    'format_window',
+    'format_window_frame',
     'time_range_to_range_window',
 )
