@@ -50,6 +50,11 @@ def bigquery_cast_integer_to_timestamp(compiled_arg, from_, to):
     return f"TIMESTAMP_SECONDS({compiled_arg})"
 
 
+@bigquery_cast.register(str, dt.Interval, dt.Integer)
+def bigquery_cast_interval_to_integer(compiled_arg, from_, to):
+    return f"EXTRACT({from_.resolution.upper()} from {compiled_arg})"
+
+
 @bigquery_cast.register(str, dt.DataType, dt.DataType)
 def bigquery_cast_generate(compiled_arg, from_, to):
     """Cast to desired type."""
@@ -191,12 +196,22 @@ def _string_right(translator, op):
 
 
 def _string_substring(translator, op):
-    _, _, length = op.args
+    length = op.length
     if (length := getattr(length, "value", None)) is not None and length < 0:
         raise ValueError("Length parameter must be a non-negative value.")
 
-    base_substring = operation_registry[ops.Substring]
-    return base_substring(translator, op)
+    arg = translator.translate(op.arg)
+    start = translator.translate(op.start)
+
+    arg_length = f"LENGTH({arg})"
+    if op.length is not None:
+        suffix = f", {translator.translate(op.length)}"
+    else:
+        suffix = ""
+
+    if_pos = f"SUBSTR({arg}, {start} + 1{suffix})"
+    if_neg = f"SUBSTR({arg}, {arg_length} + {start} + 1{suffix})"
+    return f"IF({start} >= 0, {if_pos}, {if_neg})"
 
 
 def _array_literal_format(op):
@@ -216,14 +231,28 @@ def _log(translator, op):
 
 def _literal(translator, op):
     dtype = op.output_dtype
+    value = op.value
+
+    if value is None:
+        return "NULL"
+
+    if dtype.is_decimal():
+        if value.is_nan():
+            return "CAST('NaN' AS FLOAT64)"
+        if value.is_infinite():
+            prefix = "-" * value.is_signed()
+            return f"CAST('{prefix}inf' AS FLOAT64)"
+        else:
+            return f"{ibis_type_to_bigquery_type(dtype)} '{value}'"
+    elif dtype.is_uuid():
+        return translator.translate(ops.Literal(str(value), dtype=dt.str))
+
     if isinstance(dtype, dt.Numeric):
-        value = op.value
         if not np.isfinite(value):
             return f"CAST({str(value)!r} AS FLOAT64)"
 
     # special case literal timestamp, date, and time scalars
     if isinstance(op, ops.Literal):
-        value = op.value
         if isinstance(dtype, dt.Date):
             if isinstance(value, datetime.datetime):
                 raw_value = value.date()
@@ -239,13 +268,19 @@ def _literal(translator, op):
             return "FROM_BASE64('{}')".format(
                 base64.b64encode(value).decode(encoding="utf-8")
             )
+        elif dtype.is_struct():
+            cols = (
+                f'{translator.translate(ops.Literal(value[name], dtype=type_))} AS {name}'
+                for name, type_ in zip(dtype.names, dtype.types)
+            )
+            return "STRUCT({})".format(", ".join(cols))
 
     try:
         return literal(translator, op)
     except NotImplementedError:
         if isinstance(dtype, dt.Array):
             return _array_literal_format(op)
-        raise NotImplementedError(type(op).__name__)
+        raise NotImplementedError(f'Unsupported type: {dtype!r}')
 
 
 def _arbitrary(translator, op):
@@ -478,6 +513,11 @@ def _nullifzero(t, op):
     return f"NULLIF({t.translate(op.arg)}, {casted})"
 
 
+def _zeroifnull(t, op):
+    casted = bigquery_cast('0', op.output_dtype)
+    return f"COALESCE({t.translate(op.arg)}, {casted})"
+
+
 def _array_agg(t, op):
     arg = op.arg
     if (where := op.where) is not None:
@@ -515,8 +555,8 @@ def _neg_idx_to_pos(array, idx):
 def _array_slice(t, op):
     arg = t.translate(op.arg)
     cond = [f"index >= {_neg_idx_to_pos(arg, t.translate(op.start))}"]
-    if op.stop:
-        cond.append(f"index < {_neg_idx_to_pos(arg, t.translate(op.stop))}")
+    if stop := op.stop:
+        cond.append(f"index < {_neg_idx_to_pos(arg, t.translate(stop))}")
     return (
         f"ARRAY("
         f"SELECT el "
@@ -527,19 +567,8 @@ def _array_slice(t, op):
 
 
 def _capitalize(t, op):
-    return f"CONCAT(UPPER(SUBSTR({t.translate(op.arg)}, 1, 1)), SUBSTR({t.translate(op.arg)}, 2))"
-
-
-def _clip(t, op):
     arg = t.translate(op.arg)
-
-    if (upper := op.upper) is not None:
-        arg = f"LEAST({t.translate(upper)}, {arg})"
-
-    if (lower := op.lower) is not None:
-        arg = f"GREATEST({t.translate(lower)}, {arg})"
-
-    return arg
+    return f"CONCAT(UPPER(SUBSTR({arg}, 1, 1)), SUBSTR({arg}, 2))"
 
 
 def _nth_value(t, op):
@@ -551,6 +580,17 @@ def _nth_value(t, op):
     return f'NTH_VALUE({arg}, {nth_op.value + 1})'
 
 
+def _interval_multiply(t, op):
+    if isinstance(op.left, ops.Literal) and isinstance(op.right, ops.Literal):
+        value = op.left.value * op.right.value
+        literal = ops.Literal(value, op.left.output_dtype)
+        return t.translate(literal)
+
+    left, right = t.translate(op.left), t.translate(op.right)
+    unit = op.left.output_dtype.resolution.upper()
+    return f"INTERVAL EXTRACT({unit} from {left}) * {right} {unit}"
+
+
 OPERATION_REGISTRY = {
     **operation_registry,
     # Literal
@@ -561,6 +601,7 @@ OPERATION_REGISTRY = {
     ops.IfNull: fixed_arity("IFNULL", 2),
     ops.NullIf: fixed_arity("NULLIF", 2),
     ops.NullIfZero: _nullifzero,
+    ops.ZeroIfNull: _zeroifnull,
     ops.NotAny: bigquery_compile_notany,
     ops.NotAll: bigquery_compile_notall,
     # Reductions
@@ -572,9 +613,12 @@ OPERATION_REGISTRY = {
     ops.Floor: compiles_floor,
     ops.Modulus: fixed_arity("MOD", 2),
     ops.Sign: unary("SIGN"),
-    ops.Clip: _clip,
-    ops.Degrees: lambda t, op: f"(180 * {t.translate(op.arg)} / ACOS(-1))",
-    ops.Radians: lambda t, op: f"(ACOS(-1) * {t.translate(op.arg)} / 180)",
+    ops.BitwiseNot: lambda t, op: f"~ {t.translate(op.arg)}",
+    ops.BitwiseXor: lambda t, op: f"{t.translate(op.left)} ^ {t.translate(op.right)}",
+    ops.BitwiseOr: lambda t, op: f"{t.translate(op.left)} | {t.translate(op.right)}",
+    ops.BitwiseAnd: lambda t, op: f"{t.translate(op.left)} & {t.translate(op.right)}",
+    ops.BitwiseLeftShift: lambda t, op: f"{t.translate(op.left)} << {t.translate(op.right)}",
+    ops.BitwiseRightShift: lambda t, op: f"{t.translate(op.left)} >> {t.translate(op.right)}",
     # Temporal functions
     ops.Date: unary("DATE"),
     ops.DateFromYMD: fixed_arity("DATE", 3),
@@ -605,6 +649,7 @@ OPERATION_REGISTRY = {
     ops.TimestampNow: fixed_arity("CURRENT_TIMESTAMP", 0),
     ops.TimestampSub: _timestamp_op("TIMESTAMP_SUB", {"h", "m", "s", "ms", "us"}),
     ops.TimestampTruncate: _truncate("TIMESTAMP", _timestamp_units),
+    ops.IntervalMultiply: _interval_multiply,
     ops.Hash: _hash,
     ops.StringReplace: fixed_arity("REPLACE", 3),
     ops.StringSplit: fixed_arity("SPLIT", 2),
@@ -688,12 +733,23 @@ OPERATION_REGISTRY = {
     ops.RandomScalar: fixed_arity("RAND", 0),
     ops.NthValue: _nth_value,
     ops.JSONGetItem: lambda t, op: f"{t.translate(op.arg)}[{t.translate(op.index)}]",
+    ops.ArrayStringJoin: lambda t, op: f"ARRAY_TO_STRING({t.translate(op.arg)}, {t.translate(op.sep)})",
+    ops.StartsWith: fixed_arity("STARTS_WITH", 2),
+    ops.EndsWith: fixed_arity("ENDS_WITH", 2),
 }
 
 _invalid_operations = {
     ops.FindInSet,
     ops.DateDiff,
     ops.TimestampDiff,
+    ops.ExtractAuthority,
+    ops.ExtractFile,
+    ops.ExtractFragment,
+    ops.ExtractHost,
+    ops.ExtractPath,
+    ops.ExtractProtocol,
+    ops.ExtractQuery,
+    ops.ExtractUserInfo,
 }
 
 OPERATION_REGISTRY = {
