@@ -253,7 +253,7 @@ def test_large_compile():
         pass
 
     names = [f"col_{i}" for i in range(num_columns)]
-    schema = ibis.Schema(names, ["string"] * num_columns)
+    schema = ibis.Schema(dict.fromkeys(names, "string"))
     ibis_client = MockBackend()
     table = ops.SQLQueryResult("select * from t", schema, ibis_client).to_expr()
     for _ in range(num_joins):  # noqa: F402
@@ -384,8 +384,8 @@ def test_timestamp_accepts_date_literals(alltypes):
     params = {p: date_string}
     result = to_sql(expr, params=params)
     expected = """\
-SELECT \\*, @param_\\d+ AS `param`
-FROM functional_alltypes"""
+SELECT t\\d+\\.\\*, @param_\\d+ AS `param`
+FROM functional_alltypes t\\d+"""
     assert re.match(expected, result) is not None
 
 
@@ -470,7 +470,7 @@ def test_range_window_function(alltypes, window, snapshot):
     "preceding",
     [
         param(5, id="five"),
-        param(ibis.interval(nanoseconds=1), id="nanos"),
+        param(ibis.interval(nanoseconds=1), id="nanos", marks=pytest.mark.xfail),
         param(ibis.interval(microseconds=1), id="micros"),
         param(ibis.interval(seconds=1), id="seconds"),
         param(ibis.interval(minutes=1), id="minutes"),
@@ -486,15 +486,6 @@ def test_trailing_range_window(alltypes, preceding, snapshot):
     snapshot.assert_match(to_sql(expr), "out.sql")
 
 
-def test_trailing_range_window_unsupported(alltypes):
-    t = alltypes
-    preceding = ibis.interval(years=1)
-    w = ibis.trailing_range_window(preceding=preceding, order_by=t.timestamp_col)
-    expr = t.mutate(win_avg=t.float_col.mean().over(w))
-    with pytest.raises(ValueError):
-        to_sql(expr)
-
-
 @pytest.mark.parametrize("distinct1", [True, False])
 @pytest.mark.parametrize("distinct2", [True, False])
 def test_union_cte(alltypes, distinct1, distinct2, snapshot):
@@ -581,6 +572,6 @@ def test_scalar_param_scope(alltypes):
     param = ibis.param("timestamp")
     result = to_sql(t.mutate(param=param), params={param: "2017-01-01"})
     expected = """\
-SELECT \\*, @param_\\d+ AS `param`
-FROM functional_alltypes"""
+SELECT t\\d+\\.\\*, @param_\\d+ AS `param`
+FROM functional_alltypes t\\d+"""
     assert re.match(expected, result) is not None
