@@ -14,7 +14,6 @@ import ibis.expr.analysis as an
 import ibis.expr.datatypes as dt
 import ibis.expr.operations as ops
 import ibis.expr.rules as rlz
-import ibis.expr.types as ir
 from ibis.backends.base.sql.registry import helpers
 from ibis.backends.clickhouse.datatypes import serialize
 
@@ -25,7 +24,7 @@ from ibis.backends.clickhouse.datatypes import serialize
 @functools.singledispatch
 def translate_val(op, **_):
     """Translate a value expression into sqlglot."""
-    raise NotImplementedError(type(op))
+    raise com.OperationNotDefinedError(f'No translation rule for {type(op)}')
 
 
 @translate_val.register(dt.DataType)
@@ -230,6 +229,8 @@ def _aggregate(op, func, *, where=None, **kw):
     if where is not None:
         func += "If"
         args.append(translate_val(where, **kw))
+    elif func == "any":
+        func = '"any"'
 
     joined_args = ", ".join(map(_sql, args))
     return f"{func}({joined_args})"
@@ -259,11 +260,16 @@ def _substring(op, **kw):
     # Clickhouse is 1-indexed
     arg = translate_val(op.arg, **kw)
     start = translate_val(op.start, **kw)
-    if (length := op.length) is None:
-        return f"substring({arg}, {start} + 1)"
+    arg_length = f"length({arg})"
+    if op.length is not None:
+        length = translate_val(op.length, **kw)
+        suffix = f", {length}"
+    else:
+        suffix = ""
 
-    length = translate_val(length, **kw)
-    return f"substring({arg}, {start} + 1, {length})"
+    if_pos = f"substring({arg}, {start} + 1{suffix})"
+    if_neg = f"substring({arg}, {arg_length} + {start} + 1{suffix})"
+    return f"if({start} >= 0, {if_pos}, {if_neg})"
 
 
 @translate_val.register(ops.StringFind)
@@ -277,7 +283,7 @@ def _string_find(op, **kw):
 
     arg = translate_val(op.arg, **kw)
     substr = translate_val(op.substr, **kw)
-    return f"position({substr} IN {arg}) - 1"
+    return f"locate({arg}, {substr}) - 1"
 
 
 @translate_val.register(ops.RegexExtract)
@@ -412,6 +418,22 @@ def _literal(op, **kw):
     elif dtype.is_string():
         quoted = value.replace("'", "''").replace("\\", "\\\\")
         return f"'{quoted}'"
+    elif dtype.is_decimal():
+        precision = dtype.precision
+        if precision is None or not 1 <= precision <= 76:
+            raise NotImplementedError(
+                f'Unsupported precision. Supported values: [1 : 76]. Current value: {precision!r}'
+            )
+
+        if 1 <= precision <= 9:
+            type_name = 'Decimal32'
+        elif 10 <= precision <= 18:
+            type_name = 'Decimal64'
+        elif 19 <= precision <= 38:
+            type_name = 'Decimal128'
+        else:
+            type_name = 'Decimal256'
+        return f"to{type_name}({str(value)}, {dtype.scale})"
     elif dtype.is_numeric():
         return repr(value)
     elif dtype.is_interval():
@@ -454,7 +476,7 @@ def _literal(op, **kw):
         fields = ", ".join(f"{value} as `{key}`" for key, value in op.value.items())
         return f"tuple({fields})"
     else:
-        raise NotImplementedError(type(op))
+        raise NotImplementedError(f'Unsupported type: {dtype!r}')
 
 
 def _array_literal_values(op):
@@ -516,7 +538,7 @@ def _table_array_view(op, *, cache, **kw):
 def _timestamp_from_unix(op, **kw):
     arg = translate_val(op.arg, **kw)
     if (unit := op.unit) in {"ms", "us", "ns"}:
-        raise ValueError(f"`{unit}` unit is not supported!")
+        raise com.UnsupportedOperationError(f"{unit!r} unit is not supported!")
 
     return f"toDateTime({arg})"
 
@@ -545,14 +567,61 @@ def _truncate(op, **kw):
     return f"{converter}({arg})"
 
 
+@translate_val.register(ops.DateFromYMD)
+def _date_from_ymd(op, **kw):
+    y = translate_val(op.year, **kw)
+    m = translate_val(op.month, **kw)
+    d = translate_val(op.day, **kw)
+    return (
+        f"toDate(concat("
+        f"toString({y}), '-', "
+        f"leftPad(toString({m}), 2, '0'), '-', "
+        f"leftPad(toString({d}), 2, '0')"
+        f"))"
+    )
+
+
+@translate_val.register(ops.TimestampFromYMDHMS)
+def _timestamp_from_ymdhms(op, **kw):
+    y = translate_val(op.year, **kw)
+    m = translate_val(op.month, **kw)
+    d = translate_val(op.day, **kw)
+    h = translate_val(op.hours, **kw)
+    min = translate_val(op.minutes, **kw)
+    s = translate_val(op.seconds, **kw)
+    timezone_arg = ''
+    if timezone := op.output_dtype.timezone:
+        timezone_arg = f', {timezone}'
+
+    return (
+        f"toDateTime("
+        f"concat(toString({y}), '-', "
+        f"leftPad(toString({m}), 2, '0'), '-', "
+        f"leftPad(toString({d}), 2, '0'), ' ', "
+        f"leftPad(toString({h}), 2, '0'), ':', "
+        f"leftPad(toString({min}), 2, '0'), ':', "
+        f"leftPad(toString({s}), 2, '0')"
+        f"), {timezone_arg})"
+    )
+
+
 @translate_val.register(ops.ExistsSubquery)
 @translate_val.register(ops.NotExistsSubquery)
 def _exists_subquery(op, **kw):
-    foreign_table = translate_val(op.foreign_table, **kw)
+    # https://github.com/ClickHouse/ClickHouse/issues/6697
+    #
+    # this would work, if clickhouse supported correlated subqueries
+    from ibis.backends.clickhouse.compiler.relations import translate_rel
+
+    foreign_table = translate_rel(op.foreign_table, **kw)
     predicates = translate_val(op.predicates, **kw)
-    subq = sg.subquery(foreign_table.where(predicates, dialect="clickhouse").select(1))
+    subq = (
+        sg.select(1)
+        .from_(foreign_table, dialect="clickhouse")
+        .where(sg.condition(predicates), dialect="clickhouse")
+    )
     prefix = "NOT " * isinstance(op, ops.NotExistsSubquery)
-    return f"{prefix}EXISTS {subq}"
+    return f"{prefix}EXISTS ({subq})"
 
 
 @translate_val.register(ops.StringSplit)
@@ -591,6 +660,12 @@ def _string_ilike(op, **kw):
     return f"lower({arg}) LIKE lower({pattern})"
 
 
+@translate_val.register(ops.Capitalize)
+def _string_capitalize(op, **kw):
+    arg = translate_val(op.arg, **kw)
+    return f"CONCAT(UPPER(SUBSTR({arg}, 1, 1)), SUBSTR({arg}, 2))"
+
+
 @translate_val.register(ops.GroupConcat)
 def _group_concat(op, **kw):
     arg = translate_val(op.arg, **kw)
@@ -627,7 +702,7 @@ def _bit_agg(func):
     def _translate(op, **kw):
         arg = translate_val(op.arg, **kw)
         if not isinstance((type := op.arg.output_dtype), dt.UnsignedInteger):
-            nbits = type._nbytes * 8
+            nbits = type.nbytes * 8
             arg = f"reinterpretAsUInt{nbits}({arg})"
 
         if (where := op.where) is not None:
@@ -738,12 +813,12 @@ def _scalar_param(op, params: Mapping[ops.Node, Any], **kw):
 def _string_contains(op, **kw):
     haystack = translate_val(op.haystack, **kw)
     needle = translate_val(op.needle, **kw)
-    return f"position({needle} IN {haystack}) > 0"
+    return f"locate({haystack}, {needle}) > 0"
 
 
 def contains(op_string: Literal["IN", "NOT IN"]) -> str:
-    def translate(op, *, cache, **kw):
-        import ibis.expr.analysis as an
+    def tr(op, *, cache, **kw):
+        from ibis.backends.clickhouse.compiler import translate
 
         value = op.value
         options = op.options
@@ -759,28 +834,11 @@ def contains(op_string: Literal["IN", "NOT IN"]) -> str:
             not isinstance(options, tuple)
             and options.output_shape is rlz.Shape.COLUMNAR
         ):
-            leaves = list(an.find_immediate_parent_tables(options))
-            nleaves = len(leaves)
-            if nleaves > 1:
-                raise NotImplementedError(
-                    "more than one leaf table in a NOT IN/IN query unsupported"
-                )
-            (leaf,) = leaves
-
-            shared_roots_count = sum(
-                an.shares_all_roots(value, child)
-                for child in an.find_immediate_parent_tables(options)
-            )
-            if shared_roots_count == nleaves:
-                from ibis.backends.clickhouse.compiler.relations import translate_rel
-
-                op = options.to_expr().as_table().op()
-                subquery = translate_rel(op, table=cache[leaf], **kw)
-                right_arg = f"({subquery})"
-            else:
-                raise NotImplementedError(
-                    "ClickHouse doesn't support correlated subqueries"
-                )
+            # this will fail to execute if there's a correlation, but it's too
+            # annoying to detect so we let it through to enable the
+            # uncorrelated use case (pandas-style `.isin`)
+            subquery = translate(options.to_expr().as_table().op(), {})
+            right_arg = f"({subquery})"
         else:
             right_arg = translate_val(options, cache=cache, **kw)
 
@@ -788,7 +846,7 @@ def contains(op_string: Literal["IN", "NOT IN"]) -> str:
         # make sense to do so for Sequence operations
         return f"{left_arg} {op_string} {right_arg}"
 
-    return translate
+    return tr
 
 
 translate_val.register(ops.Contains)(contains("IN"))
@@ -936,6 +994,7 @@ _simple_ops = {
     ops.Tan: "tan",
     ops.Pi: "pi",
     ops.E: "e",
+    ops.RandomScalar: "randCanonical",
     # Unary aggregates
     ops.ApproxMedian: "median",
     # TODO: there is also a `uniq` function which is the
@@ -967,6 +1026,7 @@ _simple_ops = {
     ops.Strip: "trimBoth",
     ops.RegexSearch: "match",
     ops.RegexReplace: "replaceRegexpAll",
+    ops.StringAscii: "ascii",
     # Temporal operations
     ops.Date: "toDate",
     ops.TimestampNow: "now",
@@ -995,6 +1055,7 @@ _simple_ops = {
     ops.IfNull: "ifNull",
     ops.NullIf: "nullIf",
     ops.MapContains: "mapContains",
+    ops.MapLength: "length",
     ops.MapKeys: "mapKeys",
     ops.MapValues: "mapValues",
     ops.MapMerge: "mapUpdate",
@@ -1022,6 +1083,13 @@ for _op, _name in _simple_ops.items():
 del _fmt, _name, _op
 
 
+@translate_val.register(ops.ExtractMillisecond)
+def _extract_millisecond(op, **kw):
+    arg = translate_val(op.arg, **kw)
+    dtype = serialize(op.output_dtype)
+    return f"CAST(substring(formatDateTime({arg}, '%f'), 1, 3) AS {dtype})"
+
+
 @translate_val.register
 def _sort_key(op: ops.SortKey, **kw):
     arg = translate_val(op.expr, **kw)
@@ -1039,59 +1107,42 @@ _cumulative_to_reduction = {
 }
 
 
-def cumulative_to_window(op, window):
-    klass = _cumulative_to_reduction[type(op)]
-    new_op = klass(*op.args)
-    win = ibis.cumulative_window().group_by(window._group_by).order_by(window._order_by)
-    new_expr = an.windowize_function(new_op.to_expr(), win)
+def cumulative_to_window(func, frame):
+    klass = _cumulative_to_reduction[type(func)]
+    new_op = klass(*func.args)
+    new_frame = frame.copy(start=None, end=0)
+    new_expr = an.windowize_function(new_op.to_expr(), frame=new_frame)
     return new_expr.op()
 
 
-def format_window(op, window, **kw):
+def format_window_boundary(boundary, **kw):
+    value = translate_val(boundary.value, **kw)
+    if boundary.preceding:
+        return f'{value} PRECEDING'
+    else:
+        return f'{value} FOLLOWING'
+
+
+def format_window_frame(func, frame, **kw):
     components = []
 
-    if window.max_lookback is not None:
+    if frame.how == "rows" and frame.max_lookback is not None:
         raise NotImplementedError(
-            'Rows with max lookback is not implemented for string-based backends.'
+            'Rows with max lookback is not implemented for the ClickHouse backend.'
         )
 
-    if window._group_by:
+    if frame.group_by:
         partition_args = ', '.join(
-            map(_sql, map(partial(translate_val, **kw), window._group_by))
+            map(_sql, map(partial(translate_val, **kw), frame.group_by))
         )
         components.append(f'PARTITION BY {partition_args}')
 
-    if window._order_by:
+    if frame.order_by:
         order_args = ', '.join(
-            map(_sql, map(partial(translate_val, **kw), window._order_by))
+            map(_sql, map(partial(translate_val, **kw), frame.order_by))
         )
         components.append(f'ORDER BY {order_args}')
 
-    p, f = window.preceding, window.following
-
-    def _prec(p: int | None) -> str:
-        assert p is None or p >= 0
-
-        if p is None:
-            prefix = 'UNBOUNDED'
-        else:
-            if not p:
-                return 'CURRENT ROW'
-            prefix = str(p)
-        return f'{prefix} PRECEDING'
-
-    def _foll(f: int | None) -> str:
-        assert f is None or f >= 0
-
-        if f is None:
-            prefix = 'UNBOUNDED'
-        else:
-            if not f:
-                return 'CURRENT ROW'
-            prefix = str(f)
-
-        return f'{prefix} FOLLOWING'
-
     frame_clause_not_allowed = (
         ops.Lag,
         ops.Lead,
@@ -1103,32 +1154,21 @@ def format_window(op, window, **kw):
         ops.RowNumber,
     )
 
-    if isinstance(op.expr, frame_clause_not_allowed):
-        frame = None
-    elif p is not None and f is not None:
-        start = _prec(p)
-        end = _foll(f)
-        frame = f'{window.how.upper()} BETWEEN {start} AND {end}'
-
-    elif p is not None:
-        if isinstance(p, tuple):
-            start, end = map(_prec, p)
-            frame = f'{window.how.upper()} BETWEEN {start} AND {end}'
-        else:
-            kind = 'ROWS' if p > 0 else 'RANGE'
-            frame = f'{kind} BETWEEN {_prec(p)} AND UNBOUNDED FOLLOWING'
-    elif f is not None:
-        if isinstance(f, tuple):
-            start, end = map(_foll, f)
-            frame = f'{window.how.upper()} BETWEEN {start} AND {end}'
-        else:
-            kind = 'ROWS' if f > 0 else 'RANGE'
-            frame = f'{kind} BETWEEN UNBOUNDED PRECEDING AND {_foll(f)}'
-    else:
+    if frame.start is None and frame.end is None:
         # no-op, default is full sample
-        frame = None
+        pass
+    elif not isinstance(func, frame_clause_not_allowed):
+        if frame.start is None:
+            start = 'UNBOUNDED PRECEDING'
+        else:
+            start = format_window_boundary(frame.start, **kw)
 
-    if frame is not None:
+        if frame.end is None:
+            end = 'UNBOUNDED FOLLOWING'
+        else:
+            end = format_window_boundary(frame.end, **kw)
+
+        frame = f'{frame.how.upper()} BETWEEN {start} AND {end}'
         components.append(frame)
 
     return f"OVER ({' '.join(components)})"
@@ -1156,83 +1196,6 @@ _map_interval_op_to_op = {
 }
 
 
-def _replace_interval_with_scalar(expr: ir.Expr | dt.Interval | float):
-    if isinstance(expr, ir.Expr):
-        expr_op = expr.op()
-    else:
-        expr_op = None
-
-    if not isinstance(expr, (dt.Interval, ir.IntervalValue)):
-        # Literal expressions have op method but native types do not.
-        if isinstance(expr_op, ops.Literal):
-            return expr_op.value
-        else:
-            return expr
-    elif isinstance(expr, dt.Interval):
-        try:
-            microseconds = _map_interval_to_microseconds[expr.unit]
-            return microseconds
-        except KeyError:
-            raise ValueError(
-                "Expected preceding values of week(), "
-                "day(), hour(), minute(), second(), millisecond(), "
-                f"microseconds(), nanoseconds(); got {expr}"
-            )
-    elif expr_op.args and isinstance(expr, ir.IntervalValue):
-        if len(expr_op.args) > 2:
-            raise NotImplementedError("'preceding' argument cannot be parsed.")
-        left_arg = _replace_interval_with_scalar(expr_op.args[0])
-        right_arg = _replace_interval_with_scalar(expr_op.args[1])
-        method = _map_interval_op_to_op[type(expr_op)]
-        return method(left_arg, right_arg)
-    else:
-        raise TypeError(f'expr has unknown type {type(expr).__name__}')
-
-
-def time_range_to_range_window(window):
-    # Check that ORDER BY column is a single time column:
-    order_by_vars = [x.op().args[0] for x in window._order_by]
-    if len(order_by_vars) > 1:
-        raise com.IbisInputError(
-            f"Expected 1 order-by variable, got {len(order_by_vars)}"
-        )
-
-    order_var = window._order_by[0].op().args[0]
-    timestamp_order_var = order_var.cast('int64')
-    window = window._replace(order_by=timestamp_order_var, how='range')
-
-    # Need to change preceding interval expression to scalars
-    preceding = window.preceding
-    if isinstance(preceding, ir.IntervalScalar):
-        new_preceding = _replace_interval_with_scalar(preceding)
-        window = window._replace(preceding=new_preceding)
-
-    return window
-
-
-@functools.singledispatch
-def transform_result(_, expr) -> str:
-    return expr
-
-
-@transform_result.register(ops.RowNumber)
-@transform_result.register(ops.DenseRank)
-@transform_result.register(ops.MinRank)
-@transform_result.register(ops.NTile)
-def _(_, expr) -> str:
-    return f"({expr} - 1)"
-
-
-REQUIRE_ORDER_BY = (
-    ops.DenseRank,
-    ops.MinRank,
-    ops.FirstValue,
-    ops.LastValue,
-    ops.PercentRank,
-    ops.CumeDist,
-    ops.NTile,
-)
-
 UNSUPPORTED_REDUCTIONS = (
     ops.ApproxMedian,
     ops.GroupConcat,
@@ -1240,39 +1203,25 @@ UNSUPPORTED_REDUCTIONS = (
 )
 
 
-@translate_val.register(ops.Window)
-def _window(op: ops.Window, **kw: Any):
-    arg = op.expr
-    window = op.window
-
-    if isinstance(arg, UNSUPPORTED_REDUCTIONS):
+@translate_val.register(ops.WindowFunction)
+def _window(op: ops.WindowFunction, **kw: Any):
+    if isinstance(op.func, UNSUPPORTED_REDUCTIONS):
         raise com.UnsupportedOperationError(
-            f'{type(arg)} is not supported in window functions'
+            f'{type(op.func)} is not supported in window functions'
         )
 
-    if isinstance(arg, ops.CumulativeOp):
-        arg = cumulative_to_window(arg, window)
+    if isinstance(op.func, ops.CumulativeOp):
+        arg = cumulative_to_window(op.func, op.frame)
         return translate_val(arg, **kw)
 
-    # Some analytic functions need to have the expression of interest in
-    # the ORDER BY part of the window clause
-    if isinstance(arg, REQUIRE_ORDER_BY) and not window._order_by:
-        window = window.order_by(arg.args[0])
+    window_formatted = format_window_frame(op, op.frame, **kw)
+    func_formatted = translate_val(op.func, **kw)
+    result = f'{func_formatted} {window_formatted}'
 
-    # Time ranges need to be converted to microseconds.
-    # FIXME(kszucs): avoid the expression roundtrip
-    if window.how == 'range':
-        order_by_types = [type(x.op().args[0]) for x in window._order_by]
-        time_range_types = (ir.TimeColumn, ir.DateColumn, ir.TimestampColumn)
-        if any(col_type in time_range_types for col_type in order_by_types):
-            window = time_range_to_range_window(window)
+    if isinstance(op.func, ops.RankBase):
+        return f"({result} - 1)"
 
-    window_formatted = format_window(op, window, **kw)
-
-    arg_formatted = translate_val(arg, **kw)
-    result = f'{arg_formatted} {window_formatted}'
-
-    return transform_result(arg, result)
+    return result
 
 
 def shift_like(op_class, name):
@@ -1382,3 +1331,29 @@ def _extract_query(op, **kw):
 def _extract_fragment(op, **kw):
     arg = translate_val(op.arg, **kw)
     return f"nullIf(fragment({arg}), '')"
+
+
+@translate_val.register(ops.ArrayStringJoin)
+def _array_string_join(op, **kw):
+    arg = translate_val(op.arg, **kw)
+    sep = translate_val(op.sep, **kw)
+    return f"arrayStringConcat({arg}, {sep})"
+
+
+@translate_val.register(ops.Argument)
+def _argument(op, **_):
+    return op.name
+
+
+@translate_val.register(ops.ArrayMap)
+def _array_map(op, **kw):
+    arg = translate_val(op.arg, **kw)
+    result = translate_val(op.result, **kw)
+    return f"arrayMap(({op.parameter}) -> {result}, {arg})"
+
+
+@translate_val.register(ops.ArrayFilter)
+def _array_filter(op, **kw):
+    arg = translate_val(op.arg, **kw)
+    result = translate_val(op.result, **kw)
+    return f"arrayFilter(({op.parameter}) -> {result}, {arg})"
