@@ -3,22 +3,20 @@ from __future__ import annotations
 import contextlib
 import importlib
 import importlib.metadata
+import itertools
 import platform
+import sys
 from functools import lru_cache
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, Iterable, Iterator, TextIO
+from typing import Any, TextIO
 
 import _pytest
 import pandas as pd
+import pytest
 import sqlalchemy as sa
 from packaging.requirements import Requirement
 from packaging.version import parse as vparse
 
-if TYPE_CHECKING:
-    import pyarrow as pa
-
-import pytest
-
 import ibis
 from ibis import util
 from ibis.backends.base import _get_backend_names
@@ -95,6 +93,23 @@ TEST_TABLES = {
     ),
 }
 
+# We want to check for exceptions in xfail tests for two reasons:
+# * xfail tests without exception checking may hide problems.
+#   For example, the implementation may work, but we may have an error in the test that needs to be fixed.
+# * facilitates code reviews
+# For now, many of our tests don't do this, and we're working to change this situation
+# by improving all tests file by file. All files that have already been improved are
+# added to this list to prevent regression.
+FIlES_WITH_STRICT_EXCEPTION_CHECK = [
+    'ibis/backends/tests/test_aggregation.py',
+    'ibis/backends/tests/test_binary.py',
+    'ibis/backends/tests/test_numeric.py',
+    'ibis/backends/tests/test_column.py',
+    'ibis/backends/tests/test_string.py',
+    'ibis/backends/tests/test_temporal.py',
+    'ibis/backends/tests/test_uuid.py',
+]
+
 
 @pytest.fixture(scope='session')
 def script_directory() -> Path:
@@ -141,9 +156,9 @@ def recreate_database(
     engine = sa.create_engine(url.set(database=""), **kwargs)
 
     if url.database is not None:
-        with engine.connect() as conn:
-            conn.execute(f'DROP DATABASE IF EXISTS {database}')
-            conn.execute(f'CREATE DATABASE {database}')
+        with engine.begin() as con:
+            con.exec_driver_sql(f"DROP DATABASE IF EXISTS {database}")
+            con.exec_driver_sql(f"CREATE DATABASE {database}")
 
 
 def init_database(
@@ -151,6 +166,7 @@ def init_database(
     database: str,
     schema: TextIO | None = None,
     recreate: bool = True,
+    isolation_level: str | None = "AUTOCOMMIT",
     **kwargs: Any,
 ) -> sa.engine.Engine:
     """Initialise `database` at `url` with `schema`.
@@ -167,11 +183,17 @@ def init_database(
         File object containing schema to use
     recreate : bool
         If true, drop the database if it exists
+    isolation_level : str
+        Transaction isolation_level
 
     Returns
     -------
-    sa.engine.Engine for the database created
+    sa.engine.Engine
+        SQLAlchemy engine object
     """
+    if isolation_level is not None:
+        kwargs["isolation_level"] = isolation_level
+
     if recreate:
         recreate_database(url, database, **kwargs)
 
@@ -183,36 +205,13 @@ def init_database(
     engine = sa.create_engine(url, **kwargs)
 
     if schema:
-        with engine.connect() as conn:
+        with engine.begin() as conn:
             for stmt in filter(None, map(str.strip, schema.read().split(';'))):
-                conn.execute(stmt)
+                conn.exec_driver_sql(stmt)
 
     return engine
 
 
-def read_tables(
-    names: Iterable[str],
-    data_dir: Path,
-) -> Iterator[tuple[str, pa.Table]]:
-    """For each csv {names} in {data_dir} return a pyarrow.Table."""
-
-    import pyarrow.csv as pac
-
-    import ibis.backends.pyarrow.datatypes as pa_dt
-
-    for name in names:
-        schema = TEST_TABLES[name]
-        convert_options = pac.ConvertOptions(
-            column_types={
-                name: pa_dt.to_pyarrow_type(type) for name, type in schema.items()
-            }
-        )
-        yield name, pac.read_csv(
-            data_dir / f'{name}.csv',
-            convert_options=convert_options,
-        )
-
-
 def _random_identifier(suffix: str) -> str:
     return f"__ibis_test_{suffix}_{util.guid()}"
 
@@ -272,7 +271,7 @@ def pytest_ignore_collect(path, config):
 def pytest_collection_modifyitems(session, config, items):
     # add the backend marker to any tests are inside "ibis/backends"
     all_backends = _get_backend_names()
-    xdist_group_markers = []
+    additional_markers = []
 
     for item in items:
         parts = item.path.parts
@@ -280,7 +279,12 @@ def pytest_collection_modifyitems(session, config, items):
         if backend is not None and backend in all_backends:
             item.add_marker(getattr(pytest.mark, backend))
             item.add_marker(pytest.mark.backend)
-        elif "backends" not in parts:
+        elif "backends" not in parts and not tuple(
+            itertools.chain(
+                *(item.iter_markers(name=name) for name in all_backends),
+                item.iter_markers(name="backend"),
+            )
+        ):
             # anything else is a "core" test and is run by default
             if not any(item.iter_markers(name="benchmark")):
                 item.add_marker(pytest.mark.core)
@@ -289,9 +293,23 @@ def pytest_collection_modifyitems(session, config, items):
             # build a list of markers so we're don't invalidate the item's
             # marker iterator
             for _ in item.iter_markers(name=name):
-                xdist_group_markers.append((item, pytest.mark.xdist_group(name=name)))
+                additional_markers.append((item, pytest.mark.xdist_group(name=name)))
 
-    for item, marker in xdist_group_markers:
+        for _ in item.iter_markers(name="pyspark"):
+            additional_markers.append(
+                (
+                    item,
+                    pytest.mark.xfail(
+                        (
+                            sys.version_info >= (3, 11)
+                            and not isinstance(item, pytest.DoctestItem)
+                        ),
+                        reason="PySpark doesn't support Python 3.11",
+                    ),
+                )
+            )
+
+    for item, marker in additional_markers:
         item.add_marker(marker)
 
 
@@ -309,9 +327,6 @@ def _get_backends_to_test(
     if keep:
         backends = backends.intersection(keep)
 
-    # spark is an alias for pyspark
-    backends = backends.difference(("spark",))
-
     return [
         pytest.param(
             backend,
@@ -394,6 +409,11 @@ def pytest_runtest_call(item):
     # This xfails so that you know when it starts to pass
     for marker in item.iter_markers(name="notimpl"):
         if backend in marker.args[0]:
+            if (
+                item.location[0] in FIlES_WITH_STRICT_EXCEPTION_CHECK
+                and "raises" not in marker.kwargs.keys()
+            ):
+                raise ValueError("notimpl requires a raises")
             reason = marker.kwargs.get("reason")
             item.add_marker(
                 pytest.mark.xfail(
@@ -406,6 +426,11 @@ def pytest_runtest_call(item):
     # This xfails so that you know when it starts to pass
     for marker in item.iter_markers(name="notyet"):
         if backend in marker.args[0]:
+            if (
+                item.location[0] in FIlES_WITH_STRICT_EXCEPTION_CHECK
+                and "raises" not in marker.kwargs.keys()
+            ):
+                raise ValueError("notyet requires a raises")
             reason = marker.kwargs.get("reason")
             item.add_marker(
                 pytest.mark.xfail(
@@ -429,6 +454,11 @@ def pytest_runtest_call(item):
     # bring it to attention  -- USE SPARINGLY
     for marker in item.iter_markers(name="broken"):
         if backend in marker.args[0]:
+            if (
+                item.location[0] in FIlES_WITH_STRICT_EXCEPTION_CHECK
+                and "raises" not in marker.kwargs.keys()
+            ):
+                raise ValueError("broken requires a raises")
             reason = marker.kwargs.get("reason")
             item.add_marker(
                 pytest.mark.xfail(
@@ -443,15 +473,16 @@ def pytest_runtest_call(item):
             continue
 
         provided_reason = kwargs.pop("reason", None)
-        spec = kwargs.pop(backend)
-        module = importlib.import_module(backend)
-        version = getattr(module, "__version__", None)
-        assert version is not None, f"{backend} module has no __version__ attribute"
-        condition = Requirement(f"{backend}{spec}").specifier.contains(version)
-        reason = f"{backend} backend test fails with {backend}{spec}"
+        specs = kwargs.pop(backend)
+        failing_specs = []
+        for spec in specs:
+            req = Requirement(spec)
+            if req.specifier.contains(importlib.import_module(req.name).__version__):
+                failing_specs.append(spec)
+        reason = f"{backend} backend test fails with {backend}{specs}"
         if provided_reason is not None:
             reason += f"; {provided_reason}"
-        if condition:
+        if failing_specs:
             item.add_marker(pytest.mark.xfail(reason=reason, **kwargs))
 
 
@@ -490,11 +521,7 @@ def _setup_backend(
     scope='session',
 )
 def ddl_backend(request, data_directory, script_directory, tmp_path_factory, worker_id):
-    """Set up the backends that are SQL-based.
-
-    (sqlite, postgres, mysql, duckdb, datafusion, clickhouse, pyspark,
-    impala)
-    """
+    """Set up the backends that are SQL-based."""
     return _setup_backend(
         request, data_directory, script_directory, tmp_path_factory, worker_id
     )
@@ -508,17 +535,14 @@ def ddl_con(ddl_backend):
 
 @pytest.fixture(
     params=_get_backends_to_test(
-        keep=("sqlite", "postgres", "mysql", "duckdb", "snowflake")
+        keep=("duckdb", "mssql", "mysql", "postgres", "snowflake", "sqlite", "trino")
     ),
     scope='session',
 )
 def alchemy_backend(
     request, data_directory, script_directory, tmp_path_factory, worker_id
 ):
-    """Set up the SQLAlchemy-based backends.
-
-    (sqlite, mysql, postgres, duckdb)
-    """
+    """Set up the SQLAlchemy-based backends."""
     return _setup_backend(
         request, data_directory, script_directory, tmp_path_factory, worker_id
     )
@@ -693,7 +717,7 @@ def current_data_db(ddl_con) -> str:
 
 
 @pytest.fixture
-def alternate_current_database(ddl_con, ddl_backend, current_data_db: str) -> str:
+def alternate_current_database(ddl_con, ddl_backend) -> str:
     """Create a temporary database and yield its name. Drops the created
     database upon completion.
 
@@ -713,7 +737,6 @@ def alternate_current_database(ddl_con, ddl_backend, current_data_db: str) -> st
     try:
         yield name
     finally:
-        ddl_con.set_database(current_data_db)
         ddl_con.drop_database(name, force=True)
 
 
