@@ -9,6 +9,7 @@ from pytest import param
 
 import ibis
 import ibis.expr.datatypes as dt
+from ibis.common.exceptions import OperationNotDefinedError
 
 da = pytest.importorskip("dask.array")
 dd = pytest.importorskip("dask.dataframe")
@@ -548,7 +549,7 @@ def test_value_counts(t, df):
         df.compute()
         .dup_strings.value_counts()
         .reset_index()
-        .rename(columns={'dup_strings': 'count'})
+        .rename(columns={'dup_strings': 'dup_strings_count'})
         .rename(columns={'index': 'dup_strings'})
         .sort_values(['dup_strings'])
         .reset_index(drop=True)
@@ -818,7 +819,7 @@ def test_round(t, df):
 
 
 @pytest.mark.xfail(
-    raises=NotImplementedError,
+    raises=OperationNotDefinedError,
     reason="MultiQuantile is not implemented for the dask backend",
 )
 def test_quantile_group_by(batting, batting_df):
@@ -842,7 +843,8 @@ def test_quantile_group_by(batting, batting_df):
 
 
 def test_summary_numeric(batting, batting_df):
-    expr = batting.aggregate(batting.G.summary())
+    with pytest.warns(FutureWarning, match="is deprecated"):
+        expr = batting.aggregate(batting.G.summary())
     result = expr.execute()
     assert len(result) == 1
 
@@ -860,7 +862,8 @@ def test_summary_numeric(batting, batting_df):
 
 
 def test_summary_numeric_group_by(batting, batting_df):
-    expr = batting.group_by('teamID').G.summary()
+    with pytest.warns(FutureWarning, match="is deprecated"):
+        expr = batting.group_by('teamID').G.summary()
     result = expr.execute()
 
     def agg(s):
@@ -884,7 +887,8 @@ def test_summary_numeric_group_by(batting, batting_df):
 
 
 def test_summary_non_numeric(batting, batting_df):
-    expr = batting.aggregate(batting.teamID.summary())
+    with pytest.warns(FutureWarning, match="is deprecated"):
+        expr = batting.aggregate(batting.teamID.summary())
     result = expr.execute()
     assert len(result) == 1
     assert len(result.columns) == 3
@@ -897,7 +901,8 @@ def test_summary_non_numeric(batting, batting_df):
 
 
 def test_summary_non_numeric_group_by(batting, batting_df):
-    expr = batting.group_by('teamID').playerID.summary()
+    with pytest.warns(FutureWarning, match="is deprecated"):
+        expr = batting.group_by('teamID').playerID.summary()
     result = expr.execute()
     expected = (
         batting_df.groupby('teamID')
