@@ -9,32 +9,31 @@ import warnings
 from pathlib import Path
 from typing import TYPE_CHECKING, Any, Iterable, Iterator, Mapping, MutableMapping
 
-import pyarrow as pa
-import pyarrow.types as pat
 import sqlalchemy as sa
 import toolz
 
+import ibis.common.exceptions as exc
 import ibis.expr.datatypes as dt
-from ibis import util
-from ibis.backends.base.sql.alchemy.datatypes import to_sqla_type
-
-if TYPE_CHECKING:
-    import duckdb
-    import pandas as pd
-
 import ibis.expr.schema as sch
 import ibis.expr.types as ir
+from ibis import util
 from ibis.backends.base.sql.alchemy import BaseAlchemyBackend
 from ibis.backends.duckdb.compiler import DuckDBSQLCompiler
 from ibis.backends.duckdb.datatypes import parse
 
-_dialect = sa.dialects.postgresql.dialect()
+if TYPE_CHECKING:
+    import duckdb
+    import pandas as pd
+    import pyarrow as pa
 
-# counters for in-memory, parquet, and csv reads
+    import ibis.expr.operations as ops
+
+# counters for in-memory, parquet, csv, and json reads
 # used if no table name is specified
 pd_n = itertools.count(0)
 pa_n = itertools.count(0)
 csv_n = itertools.count(0)
+json_n = itertools.count(0)
 
 
 def normalize_filenames(source_list):
@@ -44,14 +43,12 @@ def normalize_filenames(source_list):
     return list(map(util.normalize_filename, source_list))
 
 
-def _quote(name: str):
-    return _dialect.identifier_preparer.quote(name)
-
-
-def _format_kwargs(kwargs):
-    return (
-        f"{k}='{v}'" if isinstance(v, str) else f"{k}={v!r}" for k, v in kwargs.items()
-    )
+def _format_kwargs(kwargs: Mapping[str, Any]):
+    bindparams, pieces = [], []
+    for name, value in kwargs.items():
+        bindparams.append(sa.bindparam(name, value))
+        pieces.append(f"{name} = :{name}")
+    return sa.text(", ".join(pieces)).bindparams(*bindparams)
 
 
 class Backend(BaseAlchemyBackend):
@@ -108,6 +105,7 @@ class Backend(BaseAlchemyBackend):
         --------
         >>> import ibis
         >>> ibis.duckdb.connect("database.ddb", threads=4, memory_limit="1GB")
+        <ibis.backends.duckdb.Backend object at ...>
         """
         if path is not None:
             warnings.warn(
@@ -115,34 +113,43 @@ class Backend(BaseAlchemyBackend):
                 "instead."
             )
             database = path
-        if not (in_memory := database == ":memory:"):
+        if database != ":memory:":
             database = Path(database).absolute()
-        else:
-            if temp_directory is None:
-                temp_directory = (
-                    Path(os.environ.get("XDG_CACHE_HOME", Path.home() / ".cache"))
-                    / "duckdb"
-                )
+        elif temp_directory is None:
+            temp_directory = (
+                Path(os.environ.get("XDG_CACHE_HOME", Path.home() / ".cache"))
+                / "ibis-duckdb"
+                / str(os.getpid())
+            )
 
         if temp_directory is not None:
+            Path(temp_directory).mkdir(parents=True, exist_ok=True)
             config["temp_directory"] = str(temp_directory)
 
-        super().do_connect(
-            sa.create_engine(
-                f"duckdb:///{database}",
-                connect_args=dict(read_only=read_only, config=config),
-                poolclass=sa.pool.SingletonThreadPool if in_memory else None,
-            )
+        config.setdefault("experimental_parallel_csv", 1)
+
+        engine = sa.create_engine(
+            f"duckdb:///{database}",
+            connect_args=dict(read_only=read_only, config=config),
+            poolclass=sa.pool.StaticPool,
         )
-        self._meta = sa.MetaData(bind=self.con)
+
+        @sa.event.listens_for(engine, "connect")
+        def set_time_zone(dbapi_connection, connection_record):
+            dbapi_connection.execute("SET TimeZone = 'UTC'")
+
+        super().do_connect(engine)
+
+        self._meta = sa.MetaData()
         self._extensions = set()
 
     def _load_extensions(self, extensions):
         for extension in extensions:
             if extension not in self._extensions:
-                with self.con.connect() as con:
-                    con.execute(f"INSTALL '{extension}'")
-                    con.execute(f"LOAD '{extension}'")
+                with self.begin() as con:
+                    c = con.connection
+                    c.install_extension(extension)
+                    c.load_extension(extension)
                 self._extensions.add(extension)
 
     def register(
@@ -194,21 +201,74 @@ class Backend(BaseAlchemyBackend):
             return self.read_csv(source, table_name=table_name, **kwargs)
         elif first.startswith(("postgres://", "postgresql://")):
             return self.read_postgres(source, table_name=table_name, **kwargs)
+        elif first.startswith("sqlite://"):
+            return self.read_sqlite(
+                first[len("sqlite://") :], table_name=table_name, **kwargs
+            )
         else:
-            self._register_failure()
-            return None
+            self._register_failure()  # noqa: RET503
 
     def _register_failure(self):
         import inspect
 
         msg = ", ".join(
-            m[0] for m in inspect.getmembers(self) if m[0].startswith("read_")
+            name for name, _ in inspect.getmembers(self) if name.startswith("read_")
         )
         raise ValueError(
             f"Cannot infer appropriate read function for input, "
             f"please call one of {msg} directly"
         )
 
+    @util.experimental
+    def read_json(
+        self,
+        source_list: str | list[str] | tuple[str],
+        table_name: str | None = None,
+        **kwargs,
+    ) -> ir.Table:
+        """Read newline-delimited JSON into an ibis table.
+
+        !!! note "This feature requires duckdb>=0.7.0"
+
+        Parameters
+        ----------
+        source_list
+            File or list of files
+        table_name
+            Optional table name
+        kwargs
+            Additional keyword arguments passed to DuckDB's `read_json_auto` function
+
+        Returns
+        -------
+        Table
+            An ibis table expression
+        """
+        import sqlalchemy_views as sav
+        from packaging.version import parse as vparse
+
+        if (version := vparse(self.version)) < vparse("0.7.0"):
+            raise exc.IbisError(
+                f"`read_json` requires duckdb >= 0.7.0, duckdb {version} is installed"
+            )
+        if not table_name:
+            table_name = f"ibis_read_json_{next(json_n)}"
+
+        view = sav.CreateView(
+            sa.table(table_name),
+            sa.select(sa.literal_column("*")).select_from(
+                sa.func.read_json_auto(
+                    sa.func.list_value(*normalize_filenames(source_list)),
+                    _format_kwargs(kwargs),
+                )
+            ),
+            or_replace=True,
+        )
+        with self.begin() as con:
+            con.execute(view)
+
+        return self.table(table_name)
+
     def read_csv(
         self,
         source_list: str | list[str] | tuple[str],
@@ -225,7 +285,7 @@ class Backend(BaseAlchemyBackend):
         table_name
             An optional name to use for the created table. This defaults to
             a sequentially generated name.
-        **kwargs
+        kwargs
             Additional keyword arguments passed to DuckDB loading function.
             See https://duckdb.org/docs/data/csv for more information.
 
@@ -234,25 +294,26 @@ class Backend(BaseAlchemyBackend):
         ir.Table
             The just-registered table
         """
+        import sqlalchemy_views as sav
+
         source_list = normalize_filenames(source_list)
 
         if not table_name:
             table_name = f"ibis_read_csv_{next(csv_n)}"
 
-        quoted_table_name = _quote(table_name)
-
         # auto_detect and columns collide, so we set auto_detect=True
         # unless COLUMNS has been specified
-        args = [
-            str(source_list),
-            f"auto_detect={kwargs.pop('auto_detect', 'columns' not in kwargs)}",
-            *_format_kwargs(kwargs),
-        ]
-        sql = f"""CREATE OR REPLACE VIEW {quoted_table_name} AS
-SELECT * FROM read_csv({', '.join(args)})"""
+        if any(source.startswith(("http://", "https://")) for source in source_list):
+            self._load_extensions(["httpfs"])
 
-        self.con.execute(sql)
-        return self._read(table_name)
+        kwargs["auto_detect"] = kwargs.pop("auto_detect", "columns" not in kwargs)
+        source = sa.select(sa.literal_column("*")).select_from(
+            sa.func.read_csv(sa.func.list_value(*source_list), _format_kwargs(kwargs))
+        )
+        view = sav.CreateView(sa.table(table_name), source, or_replace=True)
+        with self.begin() as con:
+            con.execute(view)
+        return self.table(table_name)
 
     def read_parquet(
         self,
@@ -270,7 +331,7 @@ SELECT * FROM read_csv({', '.join(args)})"""
         table_name
             An optional name to use for the created table. This defaults to
             a sequentially generated name.
-        **kwargs
+        kwargs
             Additional keyword arguments passed to DuckDB loading function.
             See https://duckdb.org/docs/data/parquet for more information.
 
@@ -279,6 +340,8 @@ SELECT * FROM read_csv({', '.join(args)})"""
         ir.Table
             The just-registered table
         """
+        import sqlalchemy_views as sav
+
         source_list = normalize_filenames(source_list)
 
         if any(source.startswith("s3://") for source in source_list):
@@ -293,7 +356,7 @@ SELECT * FROM read_csv({', '.join(args)})"""
             # We don't create a view since DuckDB special cases Arrow Datasets
             # so if we also create a view we end up with both a "lazy table"
             # and a view with the same name
-            with self.con.begin() as con:
+            with self.begin() as con:
                 # DuckDB normally auto-detects Arrow Datasets that are defined
                 # in local variables but the `dataset` variable won't be local
                 # by the time we execute against this so we register it
@@ -304,16 +367,20 @@ SELECT * FROM read_csv({', '.join(args)})"""
                 source.startswith(("http://", "https://")) for source in source_list
             ):
                 self._load_extensions(["httpfs"])
-            dataset = str(source_list)
-            table_name = table_name or f"ibis_read_parquet_{next(pa_n)}"
 
-            quoted_table_name = _quote(table_name)
-            sql = f"""CREATE OR REPLACE VIEW {quoted_table_name} AS
-            SELECT * FROM read_parquet({dataset})"""
+            if table_name is None:
+                table_name = f"ibis_read_parquet_{next(pa_n)}"
 
-            self.con.execute(sql)
+            source = sa.select(sa.literal_column("*")).select_from(
+                sa.func.read_parquet(
+                    sa.func.list_value(*source_list), _format_kwargs(kwargs)
+                )
+            )
+            view = sav.CreateView(sa.table(table_name), source, or_replace=True)
+            with self.begin() as con:
+                con.execute(view)
 
-        return self._read(table_name)
+        return self.table(table_name)
 
     def read_in_memory(
         self, dataframe: pd.DataFrame | pa.Table, table_name: str | None = None
@@ -334,11 +401,21 @@ SELECT * FROM read_csv({', '.join(args)})"""
             The just-registered table
         """
         table_name = table_name or f"ibis_read_in_memory_{next(pd_n)}"
-        self.con.execute("register", (table_name, dataframe))
+        with self.begin() as con:
+            con.connection.register(table_name, dataframe)
 
-        return self._read(table_name)
+        return self.table(table_name)
 
-    def read_postgres(self, uri, table_name=None):
+    def list_tables(self, like=None, database=None):
+        tables = self.inspector.get_table_names(schema=database)
+        views = self.inspector.get_view_names(schema=database)
+        # workaround for GH5503
+        temp_views = self.inspector.get_view_names(
+            schema="temp" if database is None else database
+        )
+        return self._filter_with_like(tables + views + temp_views, like)
+
+    def read_postgres(self, uri, table_name: str | None = None, schema: str = "public"):
         """Register a table from a postgres instance into a DuckDB table.
 
         Parameters
@@ -347,43 +424,70 @@ SELECT * FROM read_csv({', '.join(args)})"""
             The postgres URI in form 'postgres://user:password@host:port'
         table_name
             The table to read
+        schema
+            PostgreSQL schema where `table_name` resides
 
         Returns
         -------
         ir.Table
             The just-registered table.
         """
+        import sqlalchemy_views as sav
+
         if table_name is None:
             raise ValueError(
                 "`table_name` is required when registering a postgres table"
             )
         self._load_extensions(["postgres_scanner"])
-        quoted_table_name = _quote(table_name)
-        sql = (
-            f"CREATE OR REPLACE VIEW {quoted_table_name} AS "
-            f"SELECT * FROM postgres_scan_pushdown('{uri}', 'public', '{table_name}')"
+        source = sa.select(sa.literal_column("*")).select_from(
+            sa.func.postgres_scan_pushdown(uri, schema, table_name)
         )
-        self.con.execute(sql)
+        view = sav.CreateView(sa.table(table_name), source, or_replace=True)
+        with self.begin() as con:
+            con.execute(view)
 
-        return self._read(table_name)
+        return self.table(table_name)
 
-    def _read(self, table_name):
+    def read_sqlite(self, path: str | Path, table_name: str | None = None) -> ir.Table:
+        """Register a table from a SQLite database into a DuckDB table.
+
+        Parameters
+        ----------
+        path
+            The path to the SQLite database
+        table_name
+            The table to read
+
+        Returns
+        -------
+        ir.Table
+            The just-registered table.
+
+        Examples
+        --------
+        >>> import ibis
+        >>> con = ibis.connect("duckdb://")
+        >>> t = con.read_sqlite("ci/ibis-testing-data/ibis_testing.db", table_name="diamonds")
+        >>> t.head().execute()
+                carat      cut color clarity  depth  table  price     x     y     z
+            0   0.23    Ideal     E     SI2   61.5   55.0    326  3.95  3.98  2.43
+            1   0.21  Premium     E     SI1   59.8   61.0    326  3.89  3.84  2.31
+            2   0.23     Good     E     VS1   56.9   65.0    327  4.05  4.07  2.31
+            3   0.29  Premium     I     VS2   62.4   58.0    334  4.20  4.23  2.63
+            4   0.31     Good     J     SI2   63.3   58.0    335  4.34  4.35  2.75
+        """
+        import sqlalchemy_views as sav
+
+        if table_name is None:
+            raise ValueError("`table_name` is required when registering a sqlite table")
+        self._load_extensions(["sqlite"])
+        source = sa.select(sa.literal_column("*")).select_from(
+            sa.func.sqlite_scan(str(path), table_name)
+        )
+        view = sav.CreateView(sa.table(table_name), source, or_replace=True)
+        with self.begin() as con:
+            con.execute(view)
 
-        _table = self.table(table_name)
-        with warnings.catch_warnings():
-            # don't fail or warn if duckdb-engine fails to discover types
-            # mostly (tinyint)
-            warnings.filterwarnings(
-                "ignore",
-                message="Did not recognize type",
-                category=sa.exc.SAWarning,
-            )
-            # We don't rely on index reflection, ignore this warning
-            warnings.filterwarnings(
-                "ignore",
-                message="duckdb-engine doesn't yet support reflection on indices",
-            )
-            self.inspector.reflect_table(_table.op().sqla_table, _table.columns)
         return self.table(table_name)
 
     def to_pyarrow_batches(
@@ -393,21 +497,38 @@ SELECT * FROM read_csv({', '.join(args)})"""
         params: Mapping[ir.Scalar, Any] | None = None,
         limit: int | str | None = None,
         chunk_size: int = 1_000_000,
-        **kwargs: Any,
-    ) -> IbisRecordBatchReader:
-        _ = self._import_pyarrow()
+        **_: Any,
+    ) -> pa.ipc.RecordBatchReader:
+        """Return a stream of record batches.
+
+        The returned `RecordBatchReader` contains a cursor with an unbounded lifetime.
+
+        For analytics use cases this is usually nothing to fret about. In some cases you
+        may need to explicit release the cursor.
+
+        Parameters
+        ----------
+        expr
+            Ibis expression
+        params
+            Bound parameters
+        limit
+            Limit the result to this number of rows
+        chunk_size
+            !!! warning "DuckDB returns 1024 size batches regardless of what argument is passed."
+        """
+        self._import_pyarrow()
+
+        from ibis.backends.duckdb.pyarrow import IbisRecordBatchReader
+
+        self._register_in_memory_tables(expr)
         query_ast = self.compiler.to_ast_ensure_limit(expr, limit, params=params)
         sql = query_ast.compile()
 
-        cursor = self.raw_sql(sql)
+        cursor = self.con.connect().execute(sql)
 
-        _reader = cursor.cursor.fetch_record_batch(chunk_size=chunk_size)
-        # Horrible hack to make sure cursor isn't garbage collected
-        # before batches are streamed out of the RecordBatchReader
-        batches = IbisRecordBatchReader(_reader, cursor)
-        return batches
-        # TODO: duckdb seems to not care about the `chunk_size` argument
-        # and returns batches in 1024 row chunks
+        reader = cursor.cursor.fetch_record_batch(chunk_size=chunk_size)
+        return IbisRecordBatchReader(reader, cursor)
 
     def to_pyarrow(
         self,
@@ -415,14 +536,17 @@ SELECT * FROM read_csv({', '.join(args)})"""
         *,
         params: Mapping[ir.Scalar, Any] | None = None,
         limit: int | str | None = None,
-        **kwargs: Any,
+        **_: Any,
     ) -> pa.Table:
-        _ = self._import_pyarrow()
+        pa = self._import_pyarrow()
+
+        self._register_in_memory_tables(expr)
         query_ast = self.compiler.to_ast_ensure_limit(expr, limit, params=params)
         sql = query_ast.compile()
 
-        cursor = self.raw_sql(sql)
-        table = cursor.cursor.fetch_arrow_table()
+        with self.begin() as con:
+            cursor = con.execute(sql)
+            table = cursor.cursor.fetch_arrow_table()
 
         if isinstance(expr, ir.Table):
             return table
@@ -438,12 +562,100 @@ SELECT * FROM read_csv({', '.join(args)})"""
         else:
             raise ValueError
 
-    def fetch_from_cursor(
+    @util.experimental
+    def to_parquet(
         self,
-        cursor: duckdb.DuckDBPyConnection,
-        schema: sch.Schema,
-    ):
+        expr: ir.Table,
+        path: str | Path,
+        *,
+        params: Mapping[ir.Scalar, Any] | None = None,
+        **kwargs: Any,
+    ) -> None:
+        """Write the results of executing the given expression to a parquet file.
+
+        This method is eager and will execute the associated expression
+        immediately.
+
+        Parameters
+        ----------
+        expr
+            The ibis expression to execute and persist to parquet.
+        path
+            The data source. A string or Path to the parquet file.
+        params
+            Mapping of scalar parameter expressions to value.
+        kwargs
+            DuckDB Parquet writer arguments. See
+            https://duckdb.org/docs/data/parquet#writing-to-parquet-files for
+            details
+
+        Examples
+        --------
+        Write out an expression to a single parquet file.
+
+        >>> import ibis
+        >>> penguins = ibis.examples.penguins.fetch()
+        >>> con = ibis.get_backend(penguins)
+        >>> con.to_parquet(penguins, "penguins.parquet")
+
+        Write out an expression to a hive-partitioned parquet file.
+
+        >>> import ibis
+        >>> penguins = ibis.examples.penguins.fetch()
+        >>> con = ibis.get_backend(penguins)
+        >>> con.to_parquet(penguins, "penguins_hive_dir", partition_by="year")  # doctest: +SKIP
+        >>> # partition on multiple columns
+        >>> con.to_parquet(penguins, "penguins_hive_dir", partition_by=("year", "island"))  # doctest: +SKIP
+        """
+        query = self._to_sql(expr, params=params)
+        args = ["FORMAT 'parquet'", *(f"{k.upper()} {v!r}" for k, v in kwargs.items())]
+        copy_cmd = f"COPY ({query}) TO {str(path)!r} ({', '.join(args)})"
+        with self.begin() as con:
+            con.exec_driver_sql(copy_cmd)
+
+    @util.experimental
+    def to_csv(
+        self,
+        expr: ir.Table,
+        path: str | Path,
+        *,
+        params: Mapping[ir.Scalar, Any] | None = None,
+        header: bool = True,
+        **kwargs: Any,
+    ) -> None:
+        """Write the results of executing the given expression to a CSV file.
+
+        This method is eager and will execute the associated expression
+        immediately.
+
+        Parameters
+        ----------
+        expr
+            The ibis expression to execute and persist to CSV.
+        path
+            The data source. A string or Path to the CSV file.
+        params
+            Mapping of scalar parameter expressions to value.
+        header
+            Whether to write the column names as the first line of the CSV file.
+        kwargs
+            DuckDB CSV writer arguments. https://duckdb.org/docs/data/csv.html#parameters
+        """
+        query = self._to_sql(expr, params=params)
+        args = [
+            "FORMAT 'csv'",
+            f"HEADER {int(header)}",
+            *(f"{k.upper()} {v!r}" for k, v in kwargs.items()),
+        ]
+        copy_cmd = f"COPY ({query}) TO {str(path)!r} ({', '.join(args)})"
+        with self.begin() as con:
+            con.exec_driver_sql(copy_cmd)
+
+    def fetch_from_cursor(
+        self, cursor: duckdb.DuckDBPyConnection, schema: sch.Schema
+    ) -> pd.DataFrame:
         import pandas as pd
+        import pyarrow.types as pat
 
         table = cursor.cursor.fetch_arrow_table()
 
@@ -460,90 +672,67 @@ SELECT * FROM read_csv({', '.join(args)})"""
         return schema.apply_to(df)
 
     def _metadata(self, query: str) -> Iterator[tuple[str, dt.DataType]]:
-        for name, type, null in toolz.pluck(
-            ["column_name", "column_type", "null"],
-            self.con.execute(f"DESCRIBE {query}"),
-        ):
-            ibis_type = parse(type)
-            yield name, ibis_type(nullable=null.lower() == "yes")
+        with self.begin() as con:
+            rows = con.exec_driver_sql(f"DESCRIBE {query}")
 
-    def _get_schema_using_query(self, query: str) -> sch.Schema:
-        """Return an ibis Schema from a DuckDB SQL string."""
-        return sch.Schema.from_tuples(self._metadata(query))
+            for name, type, null in toolz.pluck(
+                ["column_name", "column_type", "null"], rows.mappings()
+            ):
+                ibis_type = parse(type)
+                yield name, ibis_type.copy(nullable=null.lower() == "yes")
 
-    def _register_in_memory_table(self, table_op):
-        df = table_op.data.to_frame()
-        self.con.execute("register", (table_op.name, df))
+    def _register_in_memory_table(self, op: ops.InMemoryTable) -> None:
+        # in theory we could use pandas dataframes, but when using dataframes
+        # with pyarrow datatypes later reads of this data segfault
+        schema = op.schema
+        if null_columns := [col for col, dtype in schema.items() if dtype.is_null()]:
+            raise exc.IbisTypeError(
+                "DuckDB cannot yet reliably handle `null` typed columns; "
+                f"got null typed columns: {null_columns}"
+            )
+
+        name = op.name
+        table = op.data.to_pyarrow(schema)
+        with self.begin() as con:
+            con.connection.register(name, table)
 
     def _get_sqla_table(
-        self,
-        name: str,
-        schema: str | None = None,
-        **kwargs: Any,
+        self, name: str, schema: str | None = None, **kwargs: Any
     ) -> sa.Table:
         with warnings.catch_warnings():
-            # don't fail or warn if duckdb-engine fails to discover types
-            warnings.filterwarnings(
-                "ignore",
-                message="Did not recognize type",
-                category=sa.exc.SAWarning,
-            )
             # We don't rely on index reflection, ignore this warning
             warnings.filterwarnings(
                 "ignore",
                 message="duckdb-engine doesn't yet support reflection on indices",
             )
-
-            table = super()._get_sqla_table(name, schema, **kwargs)
-
-        nulltype_cols = frozenset(
-            col.name for col in table.c if isinstance(col.type, sa.types.NullType)
-        )
-
-        if not nulltype_cols:
-            return table
-
-        quoted_name = self.con.dialect.identifier_preparer.quote(name)
-
-        for colname, type in self._metadata(quoted_name):
-            if colname in nulltype_cols:
-                # replace null types discovered by sqlalchemy with non null
-                # types
-                table.append_column(
-                    sa.Column(
-                        colname,
-                        to_sqla_type(type),
-                        nullable=type.nullable,
-                    ),
-                    replace_existing=True,
-                )
-        return table
+            return super()._get_sqla_table(name, schema, **kwargs)
 
     def _get_temp_view_definition(
-        self,
-        name: str,
-        definition: sa.sql.compiler.Compiled,
+        self, name: str, definition: sa.sql.compiler.Compiled
     ) -> str:
-        return f"CREATE OR REPLACE TEMPORARY VIEW {name} AS {definition}"
+        yield f"CREATE OR REPLACE TEMPORARY VIEW {name} AS {definition}"
 
+    def _get_compiled_statement(self, view: sa.Table, definition: sa.sql.Selectable):
+        # TODO: remove this once duckdb supports CTAS prepared statements
+        return super()._get_compiled_statement(
+            view, definition, compile_kwargs={"literal_binds": True}
+        )
 
-class IbisRecordBatchReader(pa.ipc.RecordBatchReader):
-    def __init__(self, reader, cursor):
-        self.reader = reader
-        self.cursor = cursor
+    def _insert_dataframe(
+        self, table_name: str, df: pd.DataFrame, overwrite: bool
+    ) -> None:
+        columns = list(df.columns)
+        t = sa.table(table_name, *map(sa.column, columns))
 
-    def close(self):
-        self.reader.close()
-        del self.cursor
+        quote = self.con.dialect.identifier_preparer.quote
+        table_name = quote(table_name)
 
-    def read_all(self):
-        return self.reader.read_all()
+        # the table name df here matters, and *must* match the input variable's
+        # name because duckdb will look up this name in the outer scope of the
+        # insert call and pull in that variable's data to scan
+        source = sa.table("df", *map(sa.column, columns))
 
-    def read_next_batch(self):
-        return self.reader.read_next_batch()
-
-    def read_pandas(self):
-        return self.reader.read_pandas()
-
-    def schema(self):
-        return self.reader.schema
+        with self.begin() as con:
+            if overwrite:
+                con.execute(t.delete())
+            con.execute(t.insert().from_select(columns, sa.select(source)))
