@@ -1,5 +1,5 @@
 CREATE TABLE IF NOT EXISTS `tname`
 STORED AS AVRO
 AS
-SELECT *
-FROM functional_alltypes
\ No newline at end of file
+SELECT t0.*
+FROM functional_alltypes t0
\ No newline at end of file
