@@ -2,5 +2,5 @@ CREATE EXTERNAL TABLE foo.`another_table`
 STORED AS PARQUET
 LOCATION '/path/to/table'
 AS
-SELECT *
-FROM test1
\ No newline at end of file
+SELECT t0.*
+FROM test1 t0
\ No newline at end of file
