@@ -1,4 +1,4 @@
 INSERT OVERWRITE foo.`testing123456`
-SELECT *
-FROM functional_alltypes
+SELECT t0.*
+FROM functional_alltypes t0
 LIMIT 10
\ No newline at end of file
