@@ -110,7 +110,7 @@ from __future__ import annotations
 import datetime
 import functools
 import numbers
-from typing import TYPE_CHECKING, Any, Callable, Iterable, Mapping
+from typing import Any, Callable, Iterable, Mapping
 
 import numpy as np
 import pandas as pd
@@ -119,9 +119,11 @@ from multipledispatch import Dispatcher
 import ibis.common.exceptions as com
 import ibis.expr.datatypes as dt
 import ibis.expr.operations as ops
-import ibis.expr.window as win
+import ibis.expr.types as ir
 import ibis.util
 from ibis.backends.base import BaseBackend
+from ibis.backends.base.df.scope import Scope
+from ibis.backends.base.df.timecontext import TimeContext, canonicalize_context
 from ibis.backends.pandas import aggcontext as agg_ctx
 from ibis.backends.pandas.dispatch import (
     execute_literal,
@@ -130,11 +132,6 @@ from ibis.backends.pandas.dispatch import (
     pre_execute,
 )
 from ibis.backends.pandas.trace import trace
-from ibis.expr.scope import Scope
-from ibis.expr.timecontext import canonicalize_context
-
-if TYPE_CHECKING:
-    from ibis.expr.typing import TimeContext
 
 integer_types = np.integer, int
 floating_types = (numbers.Real,)
@@ -160,7 +157,6 @@ def is_computable_input(arg):
 @is_computable_input.register(ops.Node)
 @is_computable_input.register(dt.DataType)
 @is_computable_input.register(type(None))
-@is_computable_input.register(win.Window)
 @is_computable_input.register(tuple)
 def is_computable_input_arg(arg):
     """Return whether `arg` is a valid computable argument."""
@@ -431,7 +427,7 @@ def main_execute(
 
     # TODO: make expresions hashable so that we can get rid of these .op()
     # calls everywhere
-    params = {k.op() if hasattr(k, 'op') else k: v for k, v in params.items()}
+    params = {k.op() if isinstance(k, ir.Expr) else k: v for k, v in params.items()}
     scope = scope.merge_scope(Scope(params, timecontext))
     return execute_with_scope(
         node,
@@ -505,7 +501,7 @@ def _apply_schema(op: ops.Node, result: pd.DataFrame | pd.Series):
         schema = op.schema
         return schema.apply_to(df.loc[:, list(schema.names)])
     elif isinstance(result, pd.Series):
-        schema = op.to_expr().to_projection().schema()
+        schema = op.to_expr().as_table().schema()
         return schema.apply_to(result.to_frame()).iloc[:, 0].reset_index(drop=True)
     return result
 
@@ -586,9 +582,9 @@ def get_node_arguments_aggregation(node):
     return (node.table,)
 
 
-@get_node_arguments.register(ops.Window)
+@get_node_arguments.register(ops.WindowFunction)
 def get_node_arguments_window(node):
-    return (get_node_arguments(node.expr)[0], node.window)
+    return get_node_arguments(node.func)[:1]
 
 
 @get_node_arguments.register(
