@@ -11,7 +11,6 @@ import math
 import numbers
 import operator
 from collections.abc import Mapping, Sized
-from typing import TYPE_CHECKING
 
 import numpy as np
 import pandas as pd
@@ -24,6 +23,8 @@ import ibis.common.exceptions as com
 import ibis.expr.datatypes as dt
 import ibis.expr.operations as ops
 import ibis.expr.types as ir
+from ibis.backends.base.df.scope import Scope
+from ibis.backends.base.df.timecontext import TimeContext, get_time_col
 from ibis.backends.pandas import Backend as PandasBackend
 from ibis.backends.pandas import aggcontext as agg_ctx
 from ibis.backends.pandas.client import PandasTable
@@ -43,11 +44,6 @@ from ibis.backends.pandas.core import (
 from ibis.backends.pandas.dispatch import execute_literal, execute_node
 from ibis.backends.pandas.execution import constants
 from ibis.backends.pandas.execution.util import coerce_to_output, get_grouping
-from ibis.expr.scope import Scope
-from ibis.expr.timecontext import get_time_col
-
-if TYPE_CHECKING:
-    from ibis.expr.typing import TimeContext
 
 
 # By default return the literal value
@@ -62,21 +58,29 @@ def execute_node_literal_value_datatype(op, value, datatype, **kwargs):
 # bool to an integer.
 @execute_literal.register(ops.Literal, object, dt.Integer)
 def execute_node_literal_any_integer_datatype(op, value, datatype, **kwargs):
+    if value is None:
+        return value
     return int(value)
 
 
 @execute_literal.register(ops.Literal, object, dt.Boolean)
 def execute_node_literal_any_boolean_datatype(op, value, datatype, **kwargs):
+    if value is None:
+        return value
     return bool(value)
 
 
 @execute_literal.register(ops.Literal, object, dt.Floating)
 def execute_node_literal_any_floating_datatype(op, value, datatype, **kwargs):
+    if value is None:
+        return value
     return float(value)
 
 
 @execute_literal.register(ops.Literal, object, dt.Array)
 def execute_node_literal_any_array_datatype(op, value, datatype, **kwargs):
+    if value is None:
+        return value
     return np.array(value)
 
 
@@ -86,9 +90,11 @@ def execute_node_literal_datatype(op, datatype, **kwargs):
 
 
 @execute_literal.register(
-    ops.Literal, timedelta_types + (str,) + integer_types, dt.Interval
+    ops.Literal, (*timedelta_types, str, *integer_types, type(None)), dt.Interval
 )
 def execute_interval_literal(op, value, dtype, **kwargs):
+    if value is None:
+        return pd.NaT
     return pd.Timedelta(value, dtype.unit)
 
 
@@ -123,7 +129,17 @@ def execute_cast_series_array(op, data, type, **kwargs):
             'Array value type must be a primitive type '
             '(e.g., number, string, or timestamp)'
         )
-    return data.map(lambda array, numpy_type=numpy_type: array.astype(numpy_type))
+
+    def cast_to_array(array, numpy_type=numpy_type):
+        elems = [
+            el if el is None else np.array(el, dtype=numpy_type).item() for el in array
+        ]
+        try:
+            return np.array(elems, dtype=numpy_type)
+        except TypeError:
+            return np.array(elems)
+
+    return data.map(cast_to_array)
 
 
 @execute_node.register(ops.Cast, pd.Series, dt.Timestamp)
@@ -200,6 +216,8 @@ def execute_sort_key_series(op, data, _, **kwargs):
 def call_numpy_ufunc(func, op, data, **kwargs):
     if getattr(data, "dtype", None) == np.dtype(np.object_):
         return data.apply(functools.partial(execute_node, op, **kwargs))
+    if func is None:
+        raise com.OperationNotDefinedError(f'{type(op).__name__} not supported')
     return func(data)
 
 
@@ -222,7 +240,11 @@ def execute_series_group_by_negate(op, data, **kwargs):
 
 @execute_node.register(ops.Unary, pd.Series)
 def execute_series_unary_op(op, data, **kwargs):
-    function = getattr(np, type(op).__name__.lower())
+    op_type = type(op)
+    if op_type == ops.BitwiseNot:
+        function = np.bitwise_not
+    else:
+        function = getattr(np, op_type.__name__.lower())
     return call_numpy_ufunc(function, op, data, **kwargs)
 
 
@@ -279,6 +301,11 @@ def execute_series_ceil(op, data, **kwargs):
     return call_numpy_ufunc(func, op, data, **kwargs).astype(return_type)
 
 
+@execute_node.register(ops.BitwiseNot, integer_types)
+def execute_int_bitwise_not(op, data, **kwargs):
+    return np.invert(data)
+
+
 def vectorize_object(op, arg, *args, **kwargs):
     func = np.vectorize(functools.partial(execute_node, op, **kwargs))
     return pd.Series(func(arg, *args), index=arg.index, name=arg.name)
@@ -886,7 +913,7 @@ def _execute_binary_op_impl(op, left, right, **_):
     try:
         operation = constants.BINARY_OPERATIONS[op_type]
     except KeyError:
-        raise NotImplementedError(
+        raise com.OperationNotDefinedError(
             f'Binary operation {op_type.__name__} not implemented'
         )
     else:
@@ -916,6 +943,9 @@ def _execute_binary_op_impl(op, left, right, **_):
 @execute_node.register(ops.Multiply, str, integer_types)
 @execute_node.register(ops.Comparison, pd.Series, timestamp_types)
 @execute_node.register(ops.Comparison, timedelta_types, pd.Series)
+@execute_node.register(ops.BitwiseBinary, integer_types, integer_types)
+@execute_node.register(ops.BitwiseBinary, pd.Series, integer_types)
+@execute_node.register(ops.BitwiseBinary, integer_types, pd.Series)
 def execute_binary_op(op, left, right, **kwargs):
     return _execute_binary_op_impl(op, left, right, **kwargs)
 
@@ -1351,7 +1381,11 @@ def wrap_case_result(raw, expr):
             raw_1d, dtype=constants.IBIS_TYPE_TO_PANDAS_TYPE[expr.type()]
         )
     if result.size == 1 and isinstance(expr, ir.Scalar):
-        return result.iloc[0].item()
+        value = result.iloc[0]
+        try:
+            return value.item()
+        except AttributeError:
+            return value
     return result
 
 
@@ -1397,7 +1431,7 @@ def execute_table_array_view(op, _, **kwargs):
 
 @execute_node.register(ops.ZeroIfNull, pd.Series)
 def execute_zero_if_null_series(op, data, **kwargs):
-    zero = op.arg.output_dtype.to_pandas().type(0)  # noqa: UP003
+    zero = op.arg.output_dtype.to_pandas().type(0)
     return data.replace({np.nan: zero, None: zero, pd.NA: zero})
 
 
@@ -1407,5 +1441,5 @@ def execute_zero_if_null_series(op, data, **kwargs):
 )
 def execute_zero_if_null_scalar(op, data, **kwargs):
     if data is None or pd.isna(data) or math.isnan(data) or np.isnan(data):
-        return op.arg.output_dtype.to_pandas().type(0)  # noqa: UP003
+        return op.arg.output_dtype.to_pandas().type(0)
     return data
