@@ -72,39 +72,72 @@ def _make_duration(value, dtype):
 
 @translate.register(ops.Literal)
 def literal(op):
-    if op.dtype.is_array():
-        value = pl.Series("", op.value)
-        typ = to_polars_type(op.dtype)
+    value = op.value
+    dtype = op.dtype
+
+    if dtype.is_array():
+        value = pl.Series("", value)
+        typ = to_polars_type(dtype)
         return pl.lit(value, dtype=typ).list()
-    elif op.dtype.is_struct():
+    elif dtype.is_struct():
         values = [
-            pl.lit(v, dtype=to_polars_type(op.dtype[k])).alias(k)
-            for k, v in op.value.items()
+            pl.lit(v, dtype=to_polars_type(dtype[k])).alias(k) for k, v in value.items()
         ]
         return pl.struct(values)
-    elif op.dtype.is_interval():
-        return _make_duration(op.value, op.dtype)
-    elif op.dtype.is_null():
-        return pl.lit(op.value)
+    elif dtype.is_interval():
+        return _make_duration(value, dtype)
+    elif dtype.is_null():
+        return pl.lit(value)
+    elif dtype.is_binary():
+        raise NotImplementedError(f"Unsupported type: {dtype!r}")
     else:
-        typ = to_polars_type(op.dtype)
+        typ = to_polars_type(dtype)
         return pl.lit(op.value, dtype=typ)
 
 
+_TIMESTAMP_SCALE_TO_UNITS = {
+    0: "s",
+    1: "ms",
+    2: "ms",
+    3: "ms",
+    4: "us",
+    5: "us",
+    6: "us",
+    7: "ns",
+    8: "ns",
+    9: "ns",
+}
+
+
 @translate.register(ops.Cast)
 def cast(op):
     arg = translate(op.arg)
+    dtype = op.arg.output_dtype
+    to = op.to
 
-    if op.to.is_interval():
-        return _make_duration(arg, op.to)
-    elif op.to.is_date():
-        if op.arg.output_dtype.is_string():
+    if to.is_interval():
+        return _make_duration(arg, to)
+    elif to.is_date():
+        if dtype.is_string():
             return arg.str.strptime(pl.Date, "%Y-%m-%d")
-    elif op.to.is_timestamp():
-        if op.arg.output_dtype.is_integer():
-            return (arg * 1_000_000).cast(pl.Datetime).alias(op.name)
+    elif to.is_timestamp():
+        time_zone = to.timezone
+        time_unit = _TIMESTAMP_SCALE_TO_UNITS.get(to.scale, "us")
 
-    typ = to_polars_type(op.to)
+        if dtype.is_integer():
+            typ = pl.Datetime(time_unit="us", time_zone=time_zone)
+            arg = (arg * 1_000_000).cast(typ)
+            if time_unit != "us":
+                arg = arg.dt.truncate(f"1{time_unit}")
+            return arg.alias(op.name)
+        elif dtype.is_string():
+            typ = pl.Datetime(time_unit=time_unit, time_zone=time_zone)
+            arg = arg.str.strptime(typ)
+            if time_unit == "s":
+                return arg.dt.truncate("1s")
+            return arg
+
+    typ = to_polars_type(to)
     return arg.cast(typ)
 
 
@@ -116,7 +149,11 @@ def column(op):
 @translate.register(ops.SortKey)
 def sort_key(op):
     arg = translate(op.expr)
-    return arg.sort(reverse=op.descending)
+    descending = op.descending
+    try:
+        return arg.sort(descending=descending)
+    except TypeError:  # pragma: no cover
+        return arg.sort(reverse=descending)  # pragma: no cover
 
 
 @translate.register(ops.Selection)
@@ -147,8 +184,11 @@ def selection(op):
 
     if op.sort_keys:
         by = [key.name for key in op.sort_keys]
-        reverse = [key.descending for key in op.sort_keys]
-        lf = lf.sort(by, reverse)
+        descending = [key.descending for key in op.sort_keys]
+        try:
+            lf = lf.sort(by, descending=descending)
+        except TypeError:  # pragma: no cover
+            lf = lf.sort(by, reverse=descending)  # pragma: no cover
 
     return lf
 
@@ -179,7 +219,7 @@ def aggregation(op):
         lf = lf.select
 
     if op.metrics:
-        metrics = [translate(arg) for arg in op.metrics]
+        metrics = [translate(arg).alias(arg.name) for arg in op.metrics]
         lf = lf(metrics)
 
     return lf
@@ -379,7 +419,10 @@ def string_length(op):
 @translate.register(ops.StringUnary)
 def string_unary(op):
     arg = translate(op.arg)
-    func = _string_unary[type(op)]
+    func = _string_unary.get(type(op))
+    if func is None:
+        raise com.OperationNotDefinedError(f'{type(op).__name__} not supported')
+
     method = getattr(arg.str, func)
     return method()
 
@@ -435,7 +478,11 @@ def string_concat(op):
 def string_join(op):
     args = [translate(arg) for arg in op.arg]
     _assert_literal(op.sep)
-    return pl.concat_str(args, sep=op.sep.value)
+    sep = op.sep.value
+    try:
+        return pl.concat_str(args, separator=sep)
+    except TypeError:  # pragma: no cover
+        return pl.concat_str(args, sep=sep)  # pragma: no cover
 
 
 @translate.register(ops.Substring)
@@ -592,15 +639,16 @@ _reductions = {
     ops.Variance: 'var',
 }
 
+for reduction in _reductions.keys():
 
-@translate.register(ops.Reduction)
-def reduction(op):
-    arg = translate(op.arg)
-    agg = _reductions[type(op)]
-    if (where := op.where) is not None:
-        arg = arg.filter(translate(where))
-    method = getattr(arg, agg)
-    return method()
+    @translate.register(reduction)
+    def reduction(op):
+        arg = translate(op.arg)
+        agg = _reductions[type(op)]
+        if (where := op.where) is not None:
+            arg = arg.filter(translate(where))
+        method = getattr(arg, agg)
+        return method()
 
 
 @translate.register(ops.Mode)
@@ -766,7 +814,10 @@ def array_collect(op):
 @translate.register(ops.Unnest)
 def unnest(op):
     arg = translate(op.arg)
-    return arg.explode()
+    try:
+        return arg.arr.explode()
+    except AttributeError:
+        return arg.explode()
 
 
 _date_methods = {
@@ -840,7 +891,9 @@ def day_of_week_name(op):
 @translate.register(ops.Unary)
 def unary(op):
     arg = translate(op.arg)
-    func = _unary[type(op)]
+    func = _unary.get(type(op))
+    if func is None:
+        raise com.OperationNotDefinedError(f'{type(op).__name__} not supported')
     return func(arg)
 
 
@@ -858,15 +911,19 @@ _comparisons = {
 def comparison(op):
     left = translate(op.left)
     right = translate(op.right)
-    func = _comparisons[type(op)]
+    func = _comparisons.get(type(op))
+    if func is None:
+        raise com.OperationNotDefinedError(f'{type(op).__name__} not supported')
     return func(left, right)
 
 
 @translate.register(ops.Between)
 def between(op):
-    arg = translate(op.arg)
-    lower = translate(op.lower_bound)
-    upper = translate(op.upper_bound)
+    op_arg = op.arg
+    arg = translate(op_arg)
+    dtype = op_arg.output_dtype
+    lower = translate(ops.Cast(op.lower_bound, dtype))
+    upper = translate(ops.Cast(op.upper_bound, dtype))
     return arg.is_between(lower, upper, closed="both")
 
 
@@ -881,7 +938,9 @@ _bitwise_binops = {
 
 @translate.register(ops.BitwiseBinary)
 def bitwise_binops(op):
-    ufunc = _bitwise_binops[type(op)]
+    ufunc = _bitwise_binops.get(type(op))
+    if ufunc is None:
+        raise com.OperationNotDefinedError(f'{type(op).__name__} not supported')
     left = translate(op.left)
     right = translate(op.right)
 
@@ -919,7 +978,9 @@ _binops = {
 def binop(op):
     left = translate(op.left)
     right = translate(op.right)
-    func = _binops[type(op)]
+    func = _binops.get(type(op))
+    if func is None:
+        raise com.OperationNotDefinedError(f'{type(op).__name__} not supported')
     return func(left, right)
 
 
@@ -929,3 +990,29 @@ def elementwise_udf(op):
     return_type = to_polars_type(op.return_type)
 
     return pl.map(func_args, lambda args: op.func(*args), return_dtype=return_type)
+
+
+@translate.register(ops.E)
+def execute_e(op, **kwargs):
+    return pl.lit(np.e)
+
+
+@translate.register(ops.Pi)
+def execute_pi(op, **kwargs):
+    return pl.lit(np.pi)
+
+
+@translate.register(ops.Time)
+def execute_time(op, **kwargs):
+    arg = translate(op.arg, **kwargs)
+    if op.arg.output_dtype.is_timestamp():
+        return arg.dt.truncate("1us").cast(pl.Time)
+    return arg
+
+
+@translate.register(ops.Union)
+def execute_union(op, **kwargs):
+    result = pl.concat([translate(op.left, **kwargs), translate(op.right, **kwargs)])
+    if op.distinct:
+        return result.unique()
+    return result
