@@ -5,7 +5,11 @@ from sqlalchemy.dialects import postgresql
 import ibis.expr.datatypes as dt
 import ibis.expr.operations as ops
 import ibis.expr.rules as rlz
-from ibis.backends.base.sql.alchemy import AlchemyCompiler, AlchemyExprTranslator
+from ibis.backends.base.sql.alchemy import (
+    AlchemyCompiler,
+    AlchemyExprTranslator,
+    to_sqla_type,
+)
 from ibis.backends.postgres.registry import operation_registry
 
 
@@ -16,15 +20,8 @@ class PostgresUDFNode(ops.Value):
 class PostgreSQLExprTranslator(AlchemyExprTranslator):
     _registry = operation_registry.copy()
     _rewrites = AlchemyExprTranslator._rewrites.copy()
-    _type_map = AlchemyExprTranslator._type_map.copy()
-    _type_map.update(
-        {
-            dt.Float16: postgresql.REAL,
-            dt.Float32: postgresql.REAL,
-            dt.Float64: postgresql.DOUBLE_PRECISION,
-        }
-    )
     _has_reduction_filter_syntax = True
+    _dialect_name = "postgresql"
 
 
 rewrites = PostgreSQLExprTranslator.rewrites
@@ -40,3 +37,13 @@ def _any_all_no_op(expr):
 
 class PostgreSQLCompiler(AlchemyCompiler):
     translator_class = PostgreSQLExprTranslator
+
+
+@to_sqla_type.register(postgresql.dialect, (dt.Float16, dt.Float32))
+def _float16_float32(_, itype, type_map=None):
+    return postgresql.REAL
+
+
+@to_sqla_type.register(postgresql.dialect, dt.Float64)
+def _float64(_, itype, type_map=None):
+    return postgresql.DOUBLE_PRECISION
