@@ -12,8 +12,8 @@ import warnings
 import sqlalchemy as sa
 from sqlalchemy.dialects import postgresql as pg
 
+import ibis.backends.base.sql.registry.geospatial as geo
 import ibis.common.exceptions as com
-import ibis.common.geospatial as geo
 import ibis.expr.datatypes as dt
 import ibis.expr.operations as ops
 
@@ -26,13 +26,12 @@ from ibis.backends.base.sql.alchemy import (
     sqlalchemy_window_functions_registry,
     unary,
 )
-from ibis.backends.base.sql.alchemy.datatypes import to_sqla_type
 from ibis.backends.base.sql.alchemy.geospatial import geospatial_supported
 from ibis.backends.base.sql.alchemy.registry import (
     _bitwise_op,
     _extract,
     geospatial_functions,
-    get_col_or_deferred_col,
+    get_col,
 )
 
 operation_registry = sqlalchemy_operation_registry.copy()
@@ -72,10 +71,8 @@ def _typeof(t, op):
     # select pg_typeof('thing') returns unknown so we have to check the child's
     # type for nullness
     return sa.case(
-        [
-            ((typ == 'unknown') & (op.arg.output_dtype != dt.null), 'text'),
-            ((typ == 'unknown') & (op.arg.output_dtype == dt.null), 'null'),
-        ],
+        ((typ == 'unknown') & (op.arg.output_dtype != dt.null), 'text'),
+        ((typ == 'unknown') & (op.arg.output_dtype == dt.null), 'null'),
         else_=typ,
     )
 
@@ -264,7 +261,7 @@ def _regex_extract(arg, pattern, index):
     index = index + 1
     does_match = sa.func.textregexeq(arg, pattern)
     matches = sa.func.regexp_match(arg, pattern, type_=pg.ARRAY(sa.TEXT))
-    return sa.case([(does_match, matches[index])], else_=None)
+    return sa.case((does_match, matches[index]), else_=None)
 
 
 def _array_repeat(t, op):
@@ -272,29 +269,20 @@ def _array_repeat(t, op):
     arg = t.translate(op.arg)
     times = t.translate(op.times)
 
+    array_length = sa.func.cardinality(arg)
     array = sa.sql.elements.Grouping(arg) if isinstance(op.arg, ops.Literal) else arg
 
-    # We still need to prefix the table name to the column name in the final
-    # query, so make sure the column knows its origin
-    array.table = arg.table
-
-    array_length = sa.func.cardinality(array)
-
     # sequence from 1 to the total number of elements desired in steps of 1.
-    # the call to greatest isn't necessary, but it provides clearer intent
-    # rather than depending on the implicit postgres generate_series behavior
-    start = step = 1
-    stop = sa.func.greatest(times, 0) * array_length
-    series = sa.func.generate_series(start, stop, step).alias()
-    series_column = sa.column(series.name, type_=sa.INTEGER)
+    series = sa.func.generate_series(1, times * array_length).table_valued()
 
     # if our current index modulo the array's length is a multiple of the
     # array's length, then the index is the array's length
-    index_expression = series_column % array_length
-    index = sa.func.coalesce(sa.func.nullif(index_expression, 0), array_length)
+    index = sa.func.coalesce(
+        sa.func.nullif(series.column % array_length, 0), array_length
+    )
 
     # tie it all together in a scalar subquery and collapse that into an ARRAY
-    return sa.func.array(sa.select(array[index]).select_from(series).scalar_subquery())
+    return sa.func.array(sa.select(array[index]).scalar_subquery())
 
 
 def _table_column(t, op):
@@ -302,7 +290,7 @@ def _table_column(t, op):
     table = op.table
 
     sa_table = get_sqla_table(ctx, table)
-    out_expr = get_col_or_deferred_col(sa_table, op.name)
+    out_expr = get_col(sa_table, op)
 
     if op.output_dtype.is_timestamp():
         timezone = op.output_dtype.timezone
@@ -351,10 +339,10 @@ def _mod(t, op):
 
 
 def _neg_idx_to_pos(array, idx):
-    return sa.case([(idx < 0, sa.func.cardinality(array) + idx)], else_=idx)
+    return sa.case((idx < 0, sa.func.cardinality(array) + idx), else_=idx)
 
 
-def _array_slice(*, index_converter, array_length):
+def _array_slice(*, index_converter, array_length, func):
     def translate(t, op):
         arg = t.translate(op.arg)
 
@@ -371,39 +359,37 @@ def _array_slice(*, index_converter, array_length):
         else:
             stop = index_converter(arg, t.translate(stop))
 
-        return arg[start + 1 : stop]
+        return func(arg, start + 1, stop)
 
     return translate
 
 
-def _array_index(*, index_converter):
+def _array_index(*, index_converter, func):
     def translate(t, op):
         sa_array = t.translate(op.arg)
         sa_index = t.translate(op.index)
         if isinstance(op.arg, ops.Literal):
             sa_array = sa.sql.elements.Grouping(sa_array)
-        return sa_array[index_converter(sa_array, sa_index) + 1]
+        return func(sa_array, index_converter(sa_array, sa_index) + 1)
 
     return translate
 
 
-def _literal(_, op):
+def _literal(t, op):
     dtype = op.output_dtype
     value = op.value
 
     if dtype.is_interval():
-        return sa.text(f"INTERVAL '{value} {dtype.resolution}'")
+        return sa.literal_column(f"INTERVAL '{value} {dtype.resolution}'")
     elif dtype.is_set():
         return list(map(sa.literal, value))
-    # geo spatial data type
     elif dtype.is_geospatial():
         # inline_metadata ex: 'SRID=4326;POINT( ... )'
         return sa.literal_column(geo.translate_literal(op, inline_metadata=True))
-    elif isinstance(value, tuple):
-        return sa.literal_column(
-            str(pg.array(value).compile(compile_kwargs=dict(literal_binds=True))),
-            type_=to_sqla_type(dtype),
-        )
+    elif dtype.is_array():
+        return pg.array(value)
+    elif dtype.is_map():
+        return pg.hstore(list(value.keys()), list(value.values()))
     else:
         return sa.literal(value)
 
@@ -462,16 +448,37 @@ def _binary_variance_reduction(func):
         if (y_type := y.output_dtype).is_boolean():
             y = ops.Cast(y, dt.Int32(nullable=y_type.nullable))
 
-        result = func(t.translate(x), t.translate(y))
+        if t._has_reduction_filter_syntax:
+            result = func(t.translate(x), t.translate(y))
 
-        if (where := op.where) is not None:
-            return result.filter(t.translate(where))
-
-        return result
+            if (where := op.where) is not None:
+                return result.filter(t.translate(where))
+            return result
+        else:
+            if (where := op.where) is not None:
+                x = ops.Where(where, x, None)
+                y = ops.Where(where, y, None)
+            return func(t.translate(x), t.translate(y))
 
     return variance_compiler
 
 
+def _arg_min_max(sort_func):
+    def translate(t, op: ops.ArgMin | ops.ArgMax) -> str:
+        arg = t.translate(op.arg)
+        key = t.translate(op.key)
+
+        conditions = [arg != sa.null(), key != sa.null()]
+
+        agg = sa.func.array_agg(pg.aggregate_order_by(arg, sort_func(key)))
+
+        if (where := op.where) is not None:
+            conditions.append(t.translate(where))
+        return agg.filter(sa.and_(*conditions))[1]
+
+    return translate
+
+
 operation_registry.update(
     {
         ops.Literal: _literal,
@@ -561,8 +568,6 @@ operation_registry.update(
         ops.DayOfWeekName: fixed_arity(
             lambda arg: sa.func.trim(sa.func.to_char(arg, 'Day')), 1
         ),
-        # now is in the timezone of the server, but we want UTC
-        ops.TimestampNow: lambda *_: sa.func.timezone('UTC', sa.func.now()),
         ops.TimeFromHMS: fixed_arity(sa.func.make_time, 3),
         ops.CumulativeAll: unary(sa.func.bool_and),
         ops.CumulativeAny: unary(sa.func.bool_or),
@@ -573,8 +578,11 @@ operation_registry.update(
         ops.ArraySlice: _array_slice(
             index_converter=_neg_idx_to_pos,
             array_length=sa.func.cardinality,
+            func=lambda arg, start, stop: arg[start:stop],
+        ),
+        ops.ArrayIndex: _array_index(
+            index_converter=_neg_idx_to_pos, func=lambda arg, index: arg[index]
         ),
-        ops.ArrayIndex: _array_index(index_converter=_neg_idx_to_pos),
         ops.ArrayConcat: fixed_arity(sa.sql.expression.ColumnElement.concat, 2),
         ops.ArrayRepeat: _array_repeat,
         ops.Unnest: unary(sa.func.unnest),
@@ -584,5 +592,42 @@ operation_registry.update(
         ops.Mode: _mode,
         ops.Quantile: _quantile,
         ops.MultiQuantile: _quantile,
+        ops.TimestampNow: lambda t, op: sa.literal_column(
+            "CURRENT_TIMESTAMP", type_=t.get_sqla_type(op.output_dtype)
+        ),
+        ops.MapGet: fixed_arity(
+            lambda arg, key, default: sa.case(
+                (arg.has_key(key), arg[key]), else_=default
+            ),
+            3,
+        ),
+        ops.MapContains: fixed_arity(pg.HSTORE.Comparator.has_key, 2),
+        ops.MapKeys: unary(pg.HSTORE.Comparator.keys),
+        ops.MapValues: unary(pg.HSTORE.Comparator.vals),
+        ops.MapMerge: fixed_arity(operator.add, 2),
+        ops.MapLength: unary(lambda arg: sa.func.cardinality(arg.keys())),
+        ops.Map: fixed_arity(pg.hstore, 2),
+        ops.ArgMin: _arg_min_max(sa.asc),
+        ops.ArgMax: _arg_min_max(sa.desc),
+        ops.ToJSONArray: unary(
+            lambda arg: sa.case(
+                (
+                    sa.func.json_typeof(arg) == "array",
+                    sa.func.array(
+                        sa.select(
+                            sa.func.json_array_elements(arg).column_valued()
+                        ).scalar_subquery()
+                    ),
+                ),
+                else_=sa.null(),
+            )
+        ),
+        ops.ArrayStringJoin: fixed_arity(
+            lambda sep, arr: sa.func.array_to_string(arr, sep), 2
+        ),
+        ops.Strip: unary(lambda arg: sa.func.trim(arg, string.whitespace)),
+        ops.LStrip: unary(lambda arg: sa.func.ltrim(arg, string.whitespace)),
+        ops.RStrip: unary(lambda arg: sa.func.rtrim(arg, string.whitespace)),
+        ops.StartsWith: fixed_arity(lambda arg, prefix: arg.op("^@")(prefix), 2),
     }
 )
