@@ -15,7 +15,6 @@ import ibis.expr.datatypes as dt
 import ibis.expr.types as ir
 from ibis import config
 from ibis import literal as L
-from ibis.expr.window import rows_with_max_lookback
 
 pytest.importorskip("psycopg2")
 sa = pytest.importorskip("sqlalchemy")
@@ -467,12 +466,16 @@ def test_ifelse(alltypes, df, op, pandas_op):
         # tier and histogram
         param(
             lambda d: d.bucket([0, 10, 25, 50, 100]),
-            lambda s: pd.cut(s, [0, 10, 25, 50, 100], right=False, labels=False),
+            lambda s: pd.cut(s, [0, 10, 25, 50, 100], right=False, labels=False).astype(
+                "int8"
+            ),
             id='include_over_false',
         ),
         param(
             lambda d: d.bucket([0, 10, 25, 50], include_over=True),
-            lambda s: pd.cut(s, [0, 10, 25, 50, np.inf], right=False, labels=False),
+            lambda s: pd.cut(
+                s, [0, 10, 25, 50, np.inf], right=False, labels=False
+            ).astype("int8"),
             id='include_over_true',
         ),
         param(
@@ -493,7 +496,9 @@ def test_ifelse(alltypes, df, op, pandas_op):
         ),
         param(
             lambda d: d.bucket([10, 25, 50, 100], include_under=True),
-            lambda s: pd.cut(s, [0, 10, 25, 50, 100], right=False, labels=False),
+            lambda s: pd.cut(s, [0, 10, 25, 50, 100], right=False, labels=False).astype(
+                "int8"
+            ),
             id='include_under_true',
         ),
     ],
@@ -501,7 +506,7 @@ def test_ifelse(alltypes, df, op, pandas_op):
 def test_bucket(alltypes, df, func, pandas_func):
     expr = func(alltypes.double_col)
     result = expr.execute()
-    expected = pandas_func(df.double_col).astype('category')
+    expected = pandas_func(df.double_col)
     tm.assert_series_equal(result, expected, check_names=False)
 
 
@@ -526,11 +531,8 @@ def test_category_label(alltypes, df):
     tm.assert_series_equal(result, expected)
 
 
-@pytest.mark.parametrize(
-    ('distinct', 'union'),
-    [(True, 'UNION'), (False, 'UNION ALL')],
-)
-def test_union_cte(alltypes, distinct, union):
+@pytest.mark.parametrize("distinct", [True, False])
+def test_union_cte(alltypes, distinct, snapshot):
     t = alltypes
     expr1 = t.group_by(t.string_col).aggregate(metric=t.double_col.sum())
     expr2 = expr1.view()
@@ -542,22 +544,7 @@ def test_union_cte(alltypes, distinct, union):
             expr.compile().compile(compile_kwargs={'literal_binds': True})
         ).splitlines()
     )
-    expected = (
-        "WITH anon_1 AS "
-        "(SELECT t0.string_col AS string_col, sum(t0.double_col) AS metric "
-        "FROM functional_alltypes AS t0 GROUP BY t0.string_col), "
-        "anon_2 AS "
-        "(SELECT t0.string_col AS string_col, sum(t0.double_col) AS metric "
-        "FROM functional_alltypes AS t0 GROUP BY t0.string_col), "
-        "anon_3 AS "
-        "(SELECT t0.string_col AS string_col, sum(t0.double_col) AS metric "
-        "FROM functional_alltypes AS t0 GROUP BY t0.string_col) "
-        "SELECT anon_1.string_col, anon_1.metric "
-        f"FROM anon_1 {union} SELECT anon_2.string_col, anon_2.metric "
-        f"FROM anon_2 {union} SELECT anon_3.string_col, anon_3.metric "
-        "FROM anon_3"
-    )
-    assert str(result) == expected
+    snapshot.assert_match(result, "out.sql")
 
 
 @pytest.mark.parametrize(
@@ -782,7 +769,7 @@ def test_rolling_window(alltypes, func, df):
 def test_rolling_window_with_mlb(alltypes):
     t = alltypes
     window = ibis.trailing_window(
-        preceding=rows_with_max_lookback(3, ibis.interval(days=5)),
+        preceding=ibis.rows_with_max_lookback(3, ibis.interval(days=5)),
         order_by=t.timestamp_col,
     )
     expr = t['double_col'].sum().over(window)
@@ -1040,16 +1027,10 @@ def test_array_concat_mixed_types(array_types):
 
 @pytest.fixture
 def t(con, guid):
-    con.raw_sql(
-        """
-        CREATE TABLE "{}" (
-          id SERIAL PRIMARY KEY,
-          name TEXT
+    with con.begin() as c:
+        c.exec_driver_sql(
+            f"CREATE TABLE {con._quote(guid)} (id SERIAL PRIMARY KEY, name TEXT)"
         )
-        """.format(
-            guid
-        )
-    )
     return con.table(guid)
 
 
@@ -1058,33 +1039,25 @@ def s(con, t, guid, guid2):
     assert t.op().name == guid
     assert t.op().name != guid2
 
-    con.raw_sql(
-        """
-        CREATE TABLE "{}" (
-          id SERIAL PRIMARY KEY,
-          left_t_id INTEGER REFERENCES "{}",
-          cost DOUBLE PRECISION
+    with con.begin() as c:
+        c.exec_driver_sql(
+            f"""
+            CREATE TABLE {con._quote(guid2)} (
+              id SERIAL PRIMARY KEY,
+              left_t_id INTEGER REFERENCES {con._quote(guid)},
+              cost DOUBLE PRECISION
+            )
+            """
         )
-        """.format(
-            guid2, guid
-        )
-    )
     return con.table(guid2)
 
 
 @pytest.fixture
 def trunc(con, guid):
-    con.raw_sql(
-        """
-        CREATE TABLE "{}" (
-          id SERIAL PRIMARY KEY,
-          name TEXT
-        )
-        """.format(
-            guid
-        )
-    )
-    con.raw_sql(f"""INSERT INTO "{guid}" (name) VALUES ('a'), ('b'), ('c')""")
+    quoted = con._quote(guid)
+    with con.begin() as c:
+        c.exec_driver_sql(f"CREATE TABLE {quoted} (id SERIAL PRIMARY KEY, name TEXT)")
+        c.exec_driver_sql(f"INSERT INTO {quoted} (name) VALUES ('a'), ('b'), ('c')")
     return con.table(guid)
 
 
@@ -1093,11 +1066,11 @@ def test_semi_join(t, s):
     expr = t.semi_join(s, t.id == s.id)
     result = expr.compile().compile(compile_kwargs={'literal_binds': True})
     base = (
-        sa.select([t_a.c.id, t_a.c.name])
-        .where(sa.exists(sa.select([1]).where(t_a.c.id == s_a.c.id)))
+        sa.select(t_a.c.id, t_a.c.name)
+        .where(sa.exists(sa.select(1).where(t_a.c.id == s_a.c.id)))
         .subquery()
     )
-    expected = sa.select([base.c.id, base.c.name])
+    expected = sa.select(base.c.id, base.c.name)
     assert str(result) == str(expected)
 
 
@@ -1106,16 +1079,16 @@ def test_anti_join(t, s):
     expr = t.anti_join(s, t.id == s.id)
     result = expr.compile().compile(compile_kwargs={'literal_binds': True})
     base = (
-        sa.select([t_a.c.id, t_a.c.name])
-        .where(~sa.exists(sa.select([1]).where(t_a.c.id == s_a.c.id)))
+        sa.select(t_a.c.id, t_a.c.name)
+        .where(~sa.exists(sa.select(1).where(t_a.c.id == s_a.c.id)))
         .subquery()
     )
-    expected = sa.select([base.c.id, base.c.name])
+    expected = sa.select(base.c.id, base.c.name)
     assert str(result) == str(expected)
 
 
 def test_create_table_from_expr(con, trunc, guid2):
-    con.create_table(guid2, expr=trunc)
+    con.create_table(guid2, obj=trunc)
     t = con.table(guid2)
     assert list(t['name'].execute()) == list('abc')
 
@@ -1230,7 +1203,8 @@ def test_boolean_reduction(alltypes, opname, df):
 
 
 def test_boolean_summary(alltypes):
-    bool_col_summary = alltypes.bool_col.summary()
+    with pytest.warns(FutureWarning, match="is deprecated"):
+        bool_col_summary = alltypes.bool_col.summary()
     expr = alltypes.aggregate(bool_col_summary)
 
     result = expr.execute()
@@ -1325,10 +1299,9 @@ def test_timestamp_with_timezone_select(tzone_compute, tz):
 
 
 def test_timestamp_type_accepts_all_timezones(con):
-    assert all(
-        dt.Timestamp(row.name).timezone == row.name
-        for row in con.con.execute('SELECT name FROM pg_timezone_names')
-    )
+    with con.begin() as c:
+        cur = c.exec_driver_sql("SELECT name FROM pg_timezone_names").fetchall()
+    assert all(dt.Timestamp(row.name).timezone == row.name for row in cur)
 
 
 @pytest.mark.parametrize(
@@ -1426,7 +1399,9 @@ def test_string_to_binary_cast(con):
         f"SELECT decode(string_col, 'escape') AS \"{name}\" "
         "FROM functional_alltypes LIMIT 10"
     )
-    raw_data = [row[0][0] for row in con.raw_sql(sql_string).fetchall()]
+    with con.begin() as c:
+        cur = c.exec_driver_sql(sql_string)
+        raw_data = [row[0][0] for row in cur]
     expected = pd.Series(raw_data, name=name)
     tm.assert_series_equal(result, expected)
 
@@ -1441,8 +1416,7 @@ def test_string_to_binary_round_trip(con):
         f"\"{name}\""
         "FROM functional_alltypes LIMIT 10"
     )
-    expected = pd.Series(
-        [row[0][0] for row in con.raw_sql(sql_string).fetchall()],
-        name=name,
-    )
+    with con.begin() as c:
+        cur = c.exec_driver_sql(sql_string)
+        expected = pd.Series([row[0][0] for row in cur], name=name)
     tm.assert_series_equal(result, expected)
