@@ -13,7 +13,8 @@ import ibis.config
 import ibis.expr.operations as ops
 import ibis.expr.schema as sch
 import ibis.expr.types as ir
-from ibis import util
+from ibis.backends.base.df.scope import Scope
+from ibis.backends.base.df.timecontext import canonicalize_context, localize_context
 from ibis.backends.base.sql import BaseSQLBackend
 from ibis.backends.base.sql.compiler import Compiler, TableSetFormatter
 from ibis.backends.base.sql.ddl import (
@@ -25,9 +26,6 @@ from ibis.backends.base.sql.ddl import (
 from ibis.backends.pyspark import ddl
 from ibis.backends.pyspark.client import PySparkTable, spark_dataframe_schema
 from ibis.backends.pyspark.compiler import PySparkDatabaseTable, PySparkExprTranslator
-from ibis.backends.pyspark.datatypes import spark_dtype
-from ibis.expr.scope import Scope
-from ibis.expr.timecontext import canonicalize_context, localize_context
 
 if TYPE_CHECKING:
     import pandas as pd
@@ -158,13 +156,6 @@ class Backend(BaseSQLBackend):
     def version(self):
         return pyspark.__version__
 
-    @util.deprecated(
-        version='2.0',
-        instead='use a new connection to the database',
-    )
-    def set_database(self, name):
-        self._catalog.setCurrentDatabase(name)
-
     @property
     def current_database(self):
         return self._catalog.currentDatabase()
@@ -203,7 +194,7 @@ class Backend(BaseSQLBackend):
             expr.op(),
             scope=scope,
             timecontext=timecontext,
-            session=self._session,
+            session=getattr(self, "_session", None),
         )
 
     def execute(self, expr: ir.Expr, **kwargs: Any) -> Any:
@@ -340,94 +331,40 @@ class Backend(BaseSQLBackend):
 
         return sch.infer(df)
 
-    def _schema_from_csv(self, path: str, **kwargs: Any) -> sch.Schema:
-        """Return a Schema object for the indicated csv file.
-
-        Spark goes through the file once to determine the schema.
-
-        Parameters
-        ----------
-        path
-            Path to CSV
-        kwargs
-            See documentation for `pyspark.sql.DataFrameReader` for more information.
-
-        Returns
-        -------
-        sch.Schema
-            An ibis schema instance
-        """
-        options = _read_csv_defaults.copy()
-        options.update(kwargs)
-        options['inferSchema'] = True
-
-        df = self._session.read.csv(path, **options)
-        return spark_dataframe_schema(df)
-
-    def _create_table_or_temp_view_from_csv(
-        self,
-        name: str,
-        path: Path,
-        schema: sch.Schema | None = None,
-        database: str | None = None,
-        force: bool = False,
-        temp_view: bool = False,
-        format: str = 'parquet',
-        **kwargs: Any,
-    ):
-        options = _read_csv_defaults.copy()
-        options.update(kwargs)
-
-        if schema:
-            assert ('inferSchema', True) not in options.items()
-            schema = spark_dtype(schema)
-            options['schema'] = schema
-        else:
-            options['inferSchema'] = True
-
-        df = self._session.read.csv(path, **options)
-
-        if temp_view:
-            if force:
-                df.createOrReplaceTempView(name)
-            else:
-                df.createTempView(name)
-        else:
-            qualified_name = self._fully_qualified_name(
-                name, database or self.current_database
-            )
-            mode = 'error'
-            if force:
-                mode = 'overwrite'
-            df.write.saveAsTable(qualified_name, format=format, mode=mode)
-
     def create_table(
         self,
-        table_name: str,
+        name: str,
         obj: ir.Table | pd.DataFrame | None = None,
+        *,
         schema: sch.Schema | None = None,
         database: str | None = None,
-        force: bool = False,
-        # HDFS options
-        format: str = 'parquet',
-    ):
+        temp: bool | None = None,
+        overwrite: bool = False,
+        format: str = "parquet",
+    ) -> ir.Table:
         """Create a new table in Spark.
 
         Parameters
         ----------
-        table_name
-            Table name
+        name
+            Name of the new table.
         obj
-            If passed, creates table from select statement results
+            If passed, creates table from `SELECT` statement results
         schema
-            Mutually exclusive with obj, creates an empty table with a
-            schema
+            Mutually exclusive with `obj`, creates an empty table with a schema
         database
             Database name
-        force
-            If true, create table if table with indicated name already exists
+        temp
+            Whether the new table is temporary
+        overwrite
+            If `True`, overwrite existing data
         format
-            Table format
+            Format of the table on disk
+
+        Returns
+        -------
+        Table
+            The newly created table.
 
         Examples
         --------
@@ -435,13 +372,17 @@ class Backend(BaseSQLBackend):
         """
         import pandas as pd
 
+        if obj is None and schema is None:
+            raise com.IbisError("The schema or obj parameter is required")
+        if temp is True:
+            raise NotImplementedError(
+                "PySpark backend does not yet support temporary tables"
+            )
         if obj is not None:
             if isinstance(obj, pd.DataFrame):
                 spark_df = self._session.createDataFrame(obj)
-                mode = 'error'
-                if force:
-                    mode = 'overwrite'
-                spark_df.write.saveAsTable(table_name, format=format, mode=mode)
+                mode = "overwrite" if overwrite else "error"
+                spark_df.write.saveAsTable(name, format=format, mode=mode)
                 return None
             else:
                 self._register_in_memory_tables(obj)
@@ -450,84 +391,85 @@ class Backend(BaseSQLBackend):
             select = ast.queries[0]
 
             statement = ddl.CTAS(
-                table_name,
+                name,
                 select,
                 database=database,
-                can_exist=force,
+                can_exist=overwrite,
                 format=format,
             )
-        elif schema is not None:
+        else:
             statement = ddl.CreateTableWithSchema(
-                table_name,
+                name,
                 schema,
                 database=database,
                 format=format,
-                can_exist=force,
+                can_exist=overwrite,
             )
-        else:
-            raise com.IbisError('Must pass expr or schema')
 
-        return self.raw_sql(statement.compile())
+        self.raw_sql(statement.compile())
+        return self.table(name, database=database)
 
-    def _register_in_memory_table(self, table_op):
-        spark_df = self.compile(table_op.to_expr())
-        spark_df.createOrReplaceTempView(table_op.name)
+    def _register_in_memory_table(self, op: ops.InMemoryTable) -> None:
+        self.compile(op.to_expr()).createOrReplaceTempView(op.name)
 
     def create_view(
         self,
         name: str,
-        expr: ir.Table,
+        obj: ir.Table,
+        *,
         database: str | None = None,
-        can_exist: bool = False,
-        temporary: bool = False,
-    ):
+        overwrite: bool = False,
+    ) -> ir.Table:
         """Create a Spark view from a table expression.
 
         Parameters
         ----------
         name
             View name
-        expr
+        obj
             Expression to use for the view
         database
             Database name
-        can_exist
+        overwrite
             Replace an existing view of the same name if it exists
-        temporary
-            Whether the table is temporary
+
+        Returns
+        -------
+        Table
+            The created view
         """
-        ast = self.compiler.to_ast(expr)
+        ast = self.compiler.to_ast(obj)
         select = ast.queries[0]
         statement = ddl.CreateView(
-            name,
-            select,
-            database=database,
-            can_exist=can_exist,
-            temporary=temporary,
+            name, select, database=database, can_exist=overwrite, temporary=True
         )
-        return self.raw_sql(statement.compile())
+        self.raw_sql(statement.compile())
+        return self.table(name, database=database)
 
     def drop_table(
         self,
         name: str,
+        *,
         database: str | None = None,
         force: bool = False,
     ) -> None:
         """Drop a table."""
-        self.drop_table_or_view(name, database, force)
+        self.drop_table_or_view(name, database=database, force=force)
 
     def drop_view(
         self,
         name: str,
+        *,
         database: str | None = None,
         force: bool = False,
     ):
         """Drop a view."""
-        self.drop_table_or_view(name, database, force)
+        self.drop_table_or_view(name, database=database, force=force)
 
     def drop_table_or_view(
         self,
         name: str,
+        *,
         database: str | None = None,
         force: bool = False,
     ) -> None:
@@ -551,21 +493,17 @@ class Backend(BaseSQLBackend):
         statement = DropTable(name, database=database, must_exist=not force)
         self.raw_sql(statement.compile())
 
-    def truncate_table(
-        self,
-        table_name: str,
-        database: str | None = None,
-    ) -> None:
+    def truncate_table(self, name: str, database: str | None = None) -> None:
         """Delete all rows from an existing table.
 
         Parameters
         ----------
-        table_name
+        name
             Table name
         database
             Database name
         """
-        statement = TruncateTable(table_name, database=database)
+        statement = TruncateTable(name, database=database)
         self.raw_sql(statement.compile())
 
     def insert(
