@@ -17,6 +17,7 @@ import ibis.expr.datatypes as dt
 import ibis.expr.operations as ops
 import ibis.expr.types as ir
 from ibis import interval
+from ibis.backends.base.df.timecontext import adjust_context
 from ibis.backends.pandas.client import PandasInMemoryTable
 from ibis.backends.pandas.execution import execute
 from ibis.backends.pyspark.datatypes import spark_dtype
@@ -24,9 +25,9 @@ from ibis.backends.pyspark.timecontext import (
     combine_time_context,
     filter_by_time_context,
 )
+from ibis.common.collections import frozendict
 from ibis.config import options
-from ibis.expr.timecontext import adjust_context
-from ibis.util import any_of, frozendict, guid
+from ibis.util import any_of, guid
 
 
 class PySparkDatabaseTable(ops.DatabaseTable):
@@ -331,6 +332,14 @@ def compile_less_equal(t, op, **kwargs):
     return t.translate(op.left, **kwargs) <= t.translate(op.right, **kwargs)
 
 
+@compiles(ops.Between)
+def compile_between(t, op, **kwargs):
+    arg = t.translate(op.arg, **kwargs)
+    lower_bound = t.translate(op.lower_bound, **kwargs)
+    upper_bound = t.translate(op.upper_bound, **kwargs)
+    return arg.between(lower_bound, upper_bound)
+
+
 @compiles(ops.Multiply)
 def compile_multiply(t, op, **kwargs):
     return t.translate(op.left, **kwargs) * t.translate(op.right, **kwargs)
@@ -349,6 +358,9 @@ def compile_literal(t, op, *, raw=False, **kwargs):
     value = op.value
     dtype = op.dtype
 
+    if value is None:
+        return F.lit(value)
+
     if raw:
         return value
 
@@ -445,7 +457,7 @@ def compile_startswith(t, op, **kwargs):
 def compile_endswith(t, op, **kwargs):
     col = t.translate(op.arg, **kwargs)
     end = t.translate(op.end, **kwargs)
-    return col.startswith(end)
+    return col.endswith(end)
 
 
 def _is_table(table):
@@ -860,14 +872,20 @@ def compile_power(t, op, **kwargs):
 
 @compiles(ops.IsNan)
 def compile_isnan(t, op, **kwargs):
-    src_column = t.translate(op.arg, **kwargs)
-    return F.isnan(src_column) | F.isnull(src_column)
+    arg = op.arg
+    if arg.output_dtype.is_floating():
+        src_column = t.translate(arg, **kwargs)
+        return F.isnull(src_column) | F.isnan(src_column)
+    return F.lit(False)
 
 
 @compiles(ops.IsInf)
 def compile_isinf(t, op, **kwargs):
-    src_column = t.translate(op.arg, **kwargs)
-    return (src_column == float('inf')) | (src_column == float('-inf'))
+    arg = op.arg
+    if arg.output_dtype.is_floating():
+        inf = float("inf")
+        return t.translate(arg, **kwargs).isin([inf, -inf])
+    return F.lit(False)
 
 
 @compiles(ops.Uppercase)
@@ -913,8 +931,8 @@ def compile_capitalize(t, op, **kwargs):
 
 
 @compiles(ops.Substring)
-def compile_substring(t, op, **kwargs):
-    src_column = t.translate(op.arg, **kwargs)
+def compile_substring(t, op, raw: bool = False, **kwargs):
+    src_column = t.translate(op.arg, raw=raw, **kwargs)
     start = t.translate(op.start, **kwargs, raw=True) + 1
     length = t.translate(op.length, **kwargs, raw=True)
 
@@ -923,6 +941,9 @@ def compile_substring(t, op, **kwargs):
             "Specifiying `start` or `length` with column expressions is not supported."
         )
 
+    if start < 0:
+        raise NotImplementedError("`start < 0` is not supported.")
+
     return src_column.substr(start, length)
 
 
@@ -1121,9 +1142,9 @@ def _canonicalize_interval(t, interval, **kwargs):
     correspondingly.
     """
     if isinstance(interval, ir.IntervalScalar):
-        value = t.translate(interval.op(), **kwargs)
-        # value is in nanoseconds and spark uses seconds since epoch
-        return int(value / 1e9)
+        t.translate(interval.op(), **kwargs)
+        return None
+
     elif isinstance(interval, int):
         return interval
     else:
@@ -1133,68 +1154,63 @@ def _canonicalize_interval(t, interval, **kwargs):
         )
 
 
-@compiles(ops.Window)
-def compile_window_op(t, op, **kwargs):
-    window = op.window
-    operand = op.expr
+@compiles(ops.WindowBoundary)
+def compile_window_boundary(t, boundary, **kwargs):
+    if boundary.value.output_dtype.is_interval():
+        value = t.translate(boundary.value, **kwargs)
+        # TODO(kszucs): the value can be a literal which is a bug
+        value = value.value if isinstance(value, ops.Literal) else value
+        # value is in nanoseconds and spark uses seconds since epoch
+        value = int(value / 1e9)
+    else:
+        value = boundary.value.value
 
+    return -value if boundary.preceding else value
+
+
+@compiles(ops.WindowFunction)
+def compile_window_function(t, op, **kwargs):
     grouping_keys = [
         key.name if isinstance(key, ops.TableColumn) else t.translate(key, **kwargs)
-        for key in window._group_by
+        for key in op.frame.group_by
     ]
 
     # Timestamp needs to be cast to long for window bounds in spark
     ordering_keys = [
         F.col(sort.name).cast('long') if sort.output_dtype.is_timestamp() else sort.name
-        for sort in window._order_by
+        for sort in op.frame.order_by
     ]
     aggcontext = AggregationContext.WINDOW
     pyspark_window = Window.partitionBy(grouping_keys).orderBy(ordering_keys)
 
-    # Checks for invalid user input e.g. passing in tuple for preceding and
-    # non-None value for following are caught and raised in expr/window.py
-    # if we're here, then the input is valid, we just need to interpret it
-    # correctly
-    if isinstance(window.preceding, tuple):
-        start, end = window.preceding
-    elif isinstance(window.following, tuple):
-        start, end = window.following
-    else:
-        start = window.preceding
-        end = window.following
-
     # If the operand is a shift op (e.g. lead, lag), Spark will set the window
     # bounds. Only set window bounds here if not a shift operation.
-    if not isinstance(operand, ops.ShiftBase):
-        if start is None:
+    if not isinstance(op.func, ops.ShiftBase):
+        if op.frame.start is None:
             win_start = Window.unboundedPreceding
         else:
-            win_start = -_canonicalize_interval(t, start, **kwargs)
-        if end is None:
+            win_start = t.translate(op.frame.start, **kwargs)
+        if op.frame.end is None:
             win_end = Window.unboundedFollowing
         else:
-            win_end = _canonicalize_interval(t, end, **kwargs)
+            win_end = t.translate(op.frame.end, **kwargs)
 
-        if (
-            isinstance(start, ir.IntervalScalar)
-            or isinstance(end, ir.IntervalScalar)
-            or window.how == "range"
-        ):
+        if op.frame.how == 'range':
             pyspark_window = pyspark_window.rangeBetween(win_start, win_end)
         else:
             pyspark_window = pyspark_window.rowsBetween(win_start, win_end)
 
-    res_op = operand
-    if isinstance(res_op, (ops.NotAll, ops.NotAny)):
+    func = op.func
+    if isinstance(func, (ops.NotAll, ops.NotAny)):
         # For NotAll and NotAny, negation must be applied after .over(window)
         # Here we rewrite node to be its negation, and negate it back after
         # translation and window operation
-        operand = res_op.negate()
-    result = t.translate(operand, **kwargs, aggcontext=aggcontext).over(pyspark_window)
+        func = func.negate()
+    result = t.translate(func, **kwargs, aggcontext=aggcontext).over(pyspark_window)
 
-    if isinstance(res_op, (ops.NotAll, ops.NotAny)):
+    if isinstance(op.func, (ops.NotAll, ops.NotAny)):
         return ~result
-    elif isinstance(res_op, (ops.MinRank, ops.DenseRank, ops.RowNumber)):
+    elif isinstance(op.func, ops.RankBase):
         # result must be cast to long type for Rank / RowNumber
         return result.astype('long') - 1
     else:
@@ -1641,9 +1657,13 @@ def compile_null_literal(t, op, **kwargs):
 
 @compiles(ops.IfNull)
 def compile_if_null(t, op, **kwargs):
-    col = t.translate(op.arg, **kwargs)
+    arg = op.arg
+    col = t.translate(arg, **kwargs)
     ifnull_col = t.translate(op.ifnull_expr, **kwargs)
-    return F.when(col.isNull() | F.isnan(col), ifnull_col).otherwise(col)
+    result = F.isnull(col)
+    if arg.output_dtype.is_floating():
+        result |= F.isnan(col)
+    return F.when(result, ifnull_col).otherwise(col)
 
 
 @compiles(ops.NullIf)
@@ -1655,14 +1675,22 @@ def compile_null_if(t, op, **kwargs):
 
 @compiles(ops.IsNull)
 def compile_is_null(t, op, **kwargs):
-    col = t.translate(op.arg, **kwargs)
-    return F.isnull(col) | F.isnan(col)
+    arg = op.arg
+    col = t.translate(arg, **kwargs)
+    result = F.isnull(col)
+    if arg.output_dtype.is_floating():
+        result |= F.isnan(col)
+    return result
 
 
 @compiles(ops.NotNull)
 def compile_not_null(t, op, **kwargs):
-    col = t.translate(op.arg, **kwargs)
-    return ~F.isnull(col) & ~F.isnan(col)
+    arg = op.arg
+    col = t.translate(arg, **kwargs)
+    result = ~F.isnull(col)
+    if arg.output_dtype.is_floating():
+        result &= ~F.isnan(col)
+    return result
 
 
 @compiles(ops.DropNa)
@@ -1813,8 +1841,12 @@ def compile_radians(t, op, **kwargs):
 
 @compiles(ops.ZeroIfNull)
 def compile_zero_if_null(t, op, **kwargs):
-    col = t.translate(op.arg, **kwargs)
-    return F.when(col.isNull() | F.isnan(col), F.lit(0)).otherwise(col)
+    arg = op.arg
+    col = t.translate(arg, **kwargs)
+    result = F.isnull(col)
+    if arg.output_dtype.is_floating():
+        result |= F.isnan(col)
+    return F.when(result, F.lit(0)).otherwise(col)
 
 
 @compiles(ops.Where)
@@ -1912,3 +1944,20 @@ def compile_quantile(t, op, **kwargs):
     return compile_aggregator(
         t, op, fn=F.percentile_approx, where_excludes=("quantile",), **kwargs
     )
+
+
+@compiles(ops.ArgMin)
+def compile_argmin(t, op, **kwargs):
+    return compile_aggregator(t, op, fn=F.min_by, **kwargs)
+
+
+@compiles(ops.ArgMax)
+def compile_argmax(t, op, **kwargs):
+    return compile_aggregator(t, op, fn=F.max_by, **kwargs)
+
+
+@compiles(ops.ArrayStringJoin)
+def compile_array_string_join(t, op, **kwargs):
+    arg = t.translate(op.arg, **kwargs)
+    sep = t.translate(op.sep, raw=True, **kwargs)
+    return F.concat_ws(sep, arg)
