@@ -1,12 +1,15 @@
-import contextlib
 import os
 import platform
 import re
+import string
+import subprocess
+import sys
 
 import numpy as np
 import pandas as pd
 import pandas.testing as tm
 import pytest
+import rich.console
 import sqlalchemy as sa
 from pytest import mark, param
 
@@ -23,21 +26,23 @@ def new_schema():
 
 
 def _create_temp_table_with_schema(con, temp_table_name, schema, data=None):
-    con.drop_table(temp_table_name, force=True)
-    con.create_table(temp_table_name, schema=schema)
-    temporary = con.table(temp_table_name)
-    assert temporary.execute().empty
+    temporary = con.create_table(temp_table_name, schema=schema)
+    assert temporary.to_pandas().empty
 
     if data is not None and isinstance(data, pd.DataFrame):
-        con.load_data(temp_table_name, data, if_exists="append")
-        result = temporary.execute()
+        assert not data.empty
+        tmp = con.create_table(temp_table_name, data, overwrite=True)
+        result = tmp.to_pandas()
         assert len(result) == len(data.index)
-        tm.assert_frame_equal(result, data)
+        tm.assert_frame_equal(
+            result.sort_values(result.columns[0]).reset_index(drop=True),
+            data.sort_values(result.columns[0]).reset_index(drop=True),
+        )
+        return tmp
 
     return temporary
 
 
-@pytest.mark.notimpl(["snowflake"])
 def test_load_data_sqlalchemy(alchemy_backend, alchemy_con, alchemy_temp_table):
     sch = ibis.schema(
         [
@@ -56,9 +61,13 @@ def test_load_data_sqlalchemy(alchemy_backend, alchemy_con, alchemy_temp_table):
             'salary': [100.0, 200.0, 300.0],
         }
     )
-    alchemy_con.create_table(alchemy_temp_table, schema=sch)
-    alchemy_con.load_data(alchemy_temp_table, df, if_exists='append')
-    result = alchemy_con.table(alchemy_temp_table).execute()
+    alchemy_con.create_table(alchemy_temp_table, df, schema=sch, overwrite=True)
+    result = (
+        alchemy_con.table(alchemy_temp_table)
+        .execute()
+        .sort_values("first_name")
+        .reset_index(drop=True)
+    )
 
     alchemy_backend.assert_frame_equal(df, result)
 
@@ -102,14 +111,59 @@ def test_sql(backend, con):
     assert len(result) == 10
 
 
-@mark.notimpl(["bigquery", "clickhouse", "datafusion", "polars"])
+backend_type_mapping = {
+    "bigquery": {
+        # backend only implements int64
+        dt.int32: dt.int64
+    }
+}
+
+
+@mark.notimpl(["clickhouse", "datafusion", "polars", "druid"])
 def test_create_table_from_schema(con, new_schema, temp_table):
-    con.create_table(temp_table, schema=new_schema)
+    new_table = con.create_table(temp_table, schema=new_schema)
+    backend_mapping = backend_type_mapping.get(con.name, dict())
 
-    t = con.table(temp_table)
+    for column_name, column_type in new_table.schema().items():
+        assert (
+            backend_mapping.get(new_schema[column_name], new_schema[column_name])
+            == column_type
+        )
 
-    for k, i_type in t.schema().items():
-        assert new_schema[k] == i_type
+
+@pytest.fixture(scope="session")
+def tmpcon(alchemy_con):
+    """A fixture to scope the connection for temp table testing.
+
+    This prevents resetting the connection for subsequent tests that may depend
+    on connection state persisting across tests.
+    """
+    return alchemy_con._from_url(alchemy_con.con.url)
+
+
+@mark.notimpl(["trino", "druid"])
+@mark.notyet(
+    ["sqlite"], reason="sqlite only support temporary tables in temporary databases"
+)
+@mark.never(
+    ["mssql"],
+    reason="mssql supports support temporary tables through naming conventions",
+)
+def test_create_temporary_table_from_schema(tmpcon, new_schema):
+    temp_table = f"_{guid()}"
+    table = tmpcon.create_table(temp_table, schema=new_schema, temp=True)
+
+    # verify table exist in the current session
+    backend_mapping = backend_type_mapping.get(tmpcon.name, dict())
+    for column_name, column_type in table.schema().items():
+        assert (
+            backend_mapping.get(new_schema[column_name], new_schema[column_name])
+            == column_type
+        )
+
+    tmpcon.reconnect()
+    # verify table no longer exist after reconnect
+    assert temp_table not in tmpcon.tables.keys()
 
 
 @mark.notimpl(
@@ -129,6 +183,7 @@ def test_create_table_from_schema(con, new_schema, temp_table):
         "trino",
     ]
 )
+@mark.broken(["druid"], reason="sqlalchemy dialect is broken")
 def test_rename_table(con, temp_table, new_schema):
     temp_table_original = f'{temp_table}_original'
     con.create_table(temp_table_original, schema=new_schema)
@@ -143,7 +198,7 @@ def test_rename_table(con, temp_table, new_schema):
         con.drop_table(temp_table, force=True)
 
 
-@mark.notimpl(["bigquery", "clickhouse", "datafusion", "polars"])
+@mark.notimpl(["bigquery", "clickhouse", "datafusion", "polars", "druid"])
 @mark.never(["impala", "pyspark"], reason="No non-nullable datatypes")
 @mark.notyet(
     ["trino"], reason="trino doesn't support NOT NULL in its in-memory catalog"
@@ -152,36 +207,24 @@ def test_nullable_input_output(con, temp_table):
     sch = ibis.schema(
         [('foo', 'int64'), ('bar', dt.int64(nullable=False)), ('baz', 'boolean')]
     )
-
-    con.create_table(temp_table, schema=sch)
-
-    t = con.table(temp_table)
+    t = con.create_table(temp_table, schema=sch)
 
     assert t.schema().types[0].nullable
     assert not t.schema().types[1].nullable
     assert t.schema().types[2].nullable
 
 
-@mark.notimpl(
-    [
-        "bigquery",
-        "clickhouse",
-        "datafusion",
-        "duckdb",
-        "mysql",
-        "postgres",
-        "sqlite",
-        "snowflake",
-        "polars",
-        "mssql",
-        "trino",
-    ]
-)
-@mark.notyet(["pyspark"])
+@mark.notimpl(["datafusion", "druid", "polars"])
 def test_create_drop_view(ddl_con, temp_view):
     # setup
     table_name = 'functional_alltypes'
-    expr = ddl_con.table(table_name).limit(1)
+    try:
+        expr = ddl_con.table(table_name)
+    except KeyError:
+        table_name = table_name.upper()
+        expr = ddl_con.table(table_name)
+
+    expr = expr.limit(1)
 
     # create a new view
     ddl_con.create_view(temp_view, expr)
@@ -194,7 +237,7 @@ def test_create_drop_view(ddl_con, temp_view):
     assert set(t_expr.schema().names) == set(v_expr.schema().names)
 
 
-@mark.notimpl(["postgres", "mysql", "clickhouse", "datafusion", "polars"])
+@mark.notimpl(["postgres", "mysql", "datafusion", "polars"])
 def test_separate_database(ddl_con, alternate_current_database, current_data_db):
     # using alternate_current_database switches "con" current
     #  database to a temporary one until a test is over
@@ -205,14 +248,8 @@ def test_separate_database(ddl_con, alternate_current_database, current_data_db)
     assert tmp_db.name == alternate_current_database
 
 
-def _skip_snowflake(con, reason="snowflake can't drop tables"):
-    if con.name == "snowflake":
-        pytest.skip(reason)
-
-
 @pytest.fixture
 def employee_empty_temp_table(alchemy_con, test_employee_schema):
-    _skip_snowflake(alchemy_con)
     temp_table_name = f"temp_to_table_{guid()[:6]}"
     _create_temp_table_with_schema(
         alchemy_con,
@@ -231,7 +268,6 @@ def employee_data_1_temp_table(
     test_employee_schema,
     test_employee_data_1,
 ):
-    _skip_snowflake(alchemy_con)
     temp_table_name = f"temp_to_table_{guid()[:6]}"
     _create_temp_table_with_schema(
         alchemy_con,
@@ -239,6 +275,7 @@ def employee_data_1_temp_table(
         test_employee_schema,
         data=test_employee_data_1,
     )
+    assert temp_table_name in alchemy_con.list_tables()
     try:
         yield temp_table_name
     finally:
@@ -251,7 +288,6 @@ def employee_data_2_temp_table(
     test_employee_schema,
     test_employee_data_2,
 ):
-    _skip_snowflake(alchemy_con)
     temp_table_name = f"temp_to_table_{guid()[:6]}"
     _create_temp_table_with_schema(
         alchemy_con,
@@ -278,9 +314,17 @@ def test_insert_no_overwrite_from_dataframe(
     )
     result = temporary.execute()
     assert len(result) == 3
-    tm.assert_frame_equal(result, test_employee_data_2)
+    tm.assert_frame_equal(
+        result.sort_values("first_name").reset_index(drop=True),
+        test_employee_data_2.sort_values("first_name").reset_index(drop=True),
+    )
 
 
+@pytest.mark.notyet(
+    ["trino"],
+    reason="Connector doesn't support deletion (required for overwrite=True)",
+    raises=sa.exc.ProgrammingError,
+)
 def test_insert_overwrite_from_dataframe(
     alchemy_con,
     employee_data_1_temp_table,
@@ -295,10 +339,13 @@ def test_insert_overwrite_from_dataframe(
     )
     result = temporary.execute()
     assert len(result) == 3
-    tm.assert_frame_equal(result, test_employee_data_2)
+    tm.assert_frame_equal(
+        result.sort_values("first_name").reset_index(drop=True),
+        test_employee_data_2.sort_values("first_name").reset_index(drop=True),
+    )
 
 
-def test_insert_no_overwite_from_expr(
+def test_insert_no_overwrite_from_expr(
     alchemy_con,
     employee_empty_temp_table,
     employee_data_2_temp_table,
@@ -313,7 +360,10 @@ def test_insert_no_overwite_from_expr(
     )
     result = temporary.execute()
     assert len(result) == 3
-    tm.assert_frame_equal(result, from_table.execute())
+    tm.assert_frame_equal(
+        result.sort_values("first_name").reset_index(drop=True),
+        from_table.execute().sort_values("first_name").reset_index(drop=True),
+    )
 
 
 def test_insert_overwrite_from_expr(
@@ -331,9 +381,15 @@ def test_insert_overwrite_from_expr(
     )
     result = temporary.execute()
     assert len(result) == 3
-    tm.assert_frame_equal(result, from_table.execute())
+    tm.assert_frame_equal(
+        result.sort_values("first_name").reset_index(drop=True),
+        from_table.execute().sort_values("first_name").reset_index(drop=True),
+    )
 
 
+@pytest.mark.notyet(
+    ["trino"], reason="memory connector doesn't allow writing to tables"
+)
 def test_insert_overwrite_from_list(
     alchemy_con,
     employee_data_1_temp_table,
@@ -361,59 +417,68 @@ def test_insert_overwrite_from_list(
 def test_insert_from_memtable(alchemy_con):
     df = pd.DataFrame({"x": range(3)})
     table_name = "memtable_test"
-    alchemy_con.insert(table_name, ibis.memtable(df))
-    alchemy_con.insert(table_name, ibis.memtable(df))
+    mt = ibis.memtable(df)
+    alchemy_con.create_table(table_name, schema=mt.schema())
+    alchemy_con.insert(table_name, mt)
+    alchemy_con.insert(table_name, mt)
 
     try:
         table = alchemy_con.tables[table_name]
         assert len(table.execute()) == 6
         assert alchemy_con.tables[table_name].schema() == ibis.schema({"x": "int64"})
     finally:
-        alchemy_con.raw_sql(f"DROP TABLE IF EXISTS {table_name}")
+        with alchemy_con.begin() as con:
+            con.exec_driver_sql(f"DROP TABLE IF EXISTS {table_name}")
         assert table_name not in alchemy_con.list_tables()
 
 
 def test_list_databases(alchemy_con):
     # Every backend has its own databases
-    TEST_DATABASES = {
+    test_databases = {
         "sqlite": {"main"},
         "postgres": {"postgres", "ibis_testing"},
+        "mssql": {"INFORMATION_SCHEMA"},
         "mysql": {"ibis_testing", "information_schema"},
-        "duckdb": {"information_schema", "main", "temp"},
+        "duckdb": {"information_schema", "main"},
         "snowflake": {"IBIS_TESTING"},
+        "trino": {"default"},
     }
-    assert TEST_DATABASES[alchemy_con.name] <= set(alchemy_con.list_databases())
+    assert test_databases[alchemy_con.name] <= set(alchemy_con.list_databases())
 
 
 @pytest.mark.never(
-    ["bigquery", "postgres", "mysql", "snowflake"],
-    reason="postgres and mysql do not support in-memory tables",
+    ["bigquery", "postgres", "mssql", "mysql", "snowflake"],
+    reason="backend does not support client-side in-memory tables",
     raises=(sa.exc.OperationalError, TypeError),
 )
+@pytest.mark.notyet(
+    ["trino"], reason="memory connector doesn't allow writing to tables"
+)
 def test_in_memory(alchemy_backend):
     con = getattr(ibis, alchemy_backend.name()).connect(":memory:")
     table_name = f"t{guid()[:6]}"
-    con.raw_sql(f"CREATE TABLE {table_name} (x int)")
+    with con.begin() as c:
+        c.exec_driver_sql(f"CREATE TABLE {table_name} (x int)")
     try:
         assert table_name in con.list_tables()
     finally:
-        con.raw_sql(f"DROP TABLE IF EXISTS {table_name}")
+        with con.begin() as c:
+            c.exec_driver_sql(f"DROP TABLE IF EXISTS {table_name}")
         assert table_name not in con.list_tables()
 
 
-@pytest.mark.parametrize(
-    "coltype",
-    [dt.uint8, dt.uint16, dt.uint32, dt.uint64],
-    ids=["uint8", "uint16", "uint32", "uint64"],
-)
 @pytest.mark.notyet(
-    ["postgres", "mysql", "sqlite"],
+    ["mssql", "mysql", "postgres", "snowflake", "sqlite", "trino"],
     raises=TypeError,
     reason="postgres, mysql and sqlite do not support unsigned integer types",
 )
-def test_unsigned_integer_type(alchemy_con, coltype):
+def test_unsigned_integer_type(alchemy_con):
     tname = f"t{guid()[:6]}"
-    alchemy_con.create_table(tname, schema=ibis.schema(dict(a=coltype)), force=True)
+    alchemy_con.create_table(
+        tname,
+        schema=ibis.schema(dict(a="uint8", b="uint16", c="uint32", d="uint64")),
+        overwrite=True,
+    )
     try:
         assert tname in alchemy_con.list_tables()
     finally:
@@ -466,17 +531,35 @@ def test_unsigned_integer_type(alchemy_con, coltype):
         ),
         param(
             "pyspark://?spark.app.name=test-pyspark",
-            marks=mark.pyspark,
+            marks=[
+                mark.pyspark,
+                pytest.mark.skipif(
+                    sys.version_info >= (3, 11),
+                    reason="passes on 3.11, but no other pyspark tests do",
+                ),
+            ],
             id="pyspark",
         ),
         param(
             "pyspark://my-warehouse-dir?spark.app.name=test-pyspark",
-            marks=mark.pyspark,
+            marks=[
+                mark.pyspark,
+                pytest.mark.skipif(
+                    sys.version_info >= (3, 11),
+                    reason="passes on 3.11, but no other pyspark tests do",
+                ),
+            ],
             id="pyspark_with_warehouse",
         ),
         param(
             "pyspark://my-warehouse-dir",
-            marks=mark.pyspark,
+            marks=[
+                mark.pyspark,
+                pytest.mark.skipif(
+                    sys.version_info >= (3, 11),
+                    reason="passes on 3.11, but no other pyspark tests do",
+                ),
+            ],
             id="pyspark_with_warehouse_no_params",
         ),
     ],
@@ -649,23 +732,13 @@ def test_deprecated_path_argument(backend_name, tmp_path):
         ),
     ],
 )
-@pytest.mark.notyet(
-    ["bigquery", "mysql", "sqlite"],
-    reason="SQLAlchemy generates incorrect code for `VALUES` projections.",
-    raises=(sa.exc.ProgrammingError, sa.exc.OperationalError),
-)
-@pytest.mark.notimpl(["bigquery", "dask", "datafusion", "pandas"])
+@pytest.mark.notimpl(["dask", "datafusion", "pandas"])
 def test_in_memory_table(backend, con, expr, expected):
     result = con.execute(expr)
     backend.assert_frame_equal(result, expected)
 
 
-@pytest.mark.notyet(
-    ["mysql", "sqlite"],
-    reason="SQLAlchemy generates incorrect code for `VALUES` projections.",
-    raises=(sa.exc.ProgrammingError, sa.exc.OperationalError),
-)
-@pytest.mark.notimpl(["bigquery", "dask", "datafusion", "pandas"])
+@pytest.mark.notimpl(["dask", "datafusion", "pandas"])
 def test_filter_memory_table(backend, con):
     t = ibis.memtable([(1, 2), (3, 4), (5, 6)], columns=["x", "y"])
     expr = t.filter(t.x > 1)
@@ -674,12 +747,7 @@ def test_filter_memory_table(backend, con):
     backend.assert_frame_equal(result, expected)
 
 
-@pytest.mark.notyet(
-    ["mysql", "sqlite"],
-    reason="SQLAlchemy generates incorrect code for `VALUES` projections.",
-    raises=(sa.exc.ProgrammingError, sa.exc.OperationalError),
-)
-@pytest.mark.notimpl(["bigquery", "dask", "datafusion", "pandas"])
+@pytest.mark.notimpl(["dask", "datafusion", "pandas"])
 def test_agg_memory_table(con):
     t = ibis.memtable([(1, 2), (3, 4), (5, 6)], columns=["x", "y"])
     expr = t.x.count()
@@ -690,23 +758,15 @@ def test_agg_memory_table(con):
 @pytest.mark.parametrize(
     "t",
     [
-        param(
-            ibis.memtable([("a", 1.0)], columns=["a", "b"]),
-            id="python",
-        ),
+        param(ibis.memtable([("a", 1.0)], columns=["a", "b"]), id="python"),
         param(
             ibis.memtable(pd.DataFrame([("a", 1.0)], columns=["a", "b"])),
             id="pandas-memtable",
         ),
-        param(
-            pd.DataFrame([("a", 1.0)], columns=["a", "b"]),
-            id="pandas",
-        ),
+        param(pd.DataFrame([("a", 1.0)], columns=["a", "b"]), id="pandas"),
     ],
 )
-@pytest.mark.notimpl(
-    ["bigquery", "clickhouse", "dask", "datafusion", "pandas", "polars"]
-)
+@pytest.mark.notimpl(["clickhouse", "dask", "datafusion", "pandas", "polars", "druid"])
 def test_create_from_in_memory_table(backend, con, t):
     if backend.name() == "snowflake":
         pytest.skip("snowflake is unreliable here")
@@ -719,25 +779,61 @@ def test_create_from_in_memory_table(backend, con, t):
         assert tmp_name not in con.list_tables()
 
 
-def test_default_backend_no_duckdb(backend):
-    # backend is used to ensure that this test runs in CI in the setting
-    # where only the dependencies for a a given backend are installed
+@pytest.mark.usefixtures("backend")
+def test_default_backend_option():
+    orig = ibis.options.default_backend
+    ibis.options.default_backend = ibis.pandas
+    try:
+        backend = ibis.config._default_backend()
+        assert backend.name == "pandas"
+    finally:
+        ibis.options.default_backend = orig
 
-    # if duckdb is available then this test won't fail and so we skip it
-    with contextlib.suppress(ImportError):
-        import duckdb  # noqa: F401
 
-        pytest.skip("duckdb is installed; it will be used as the default backend")
+# backend is used to ensure that this test runs in CI in the setting
+# where only the dependencies for a given backend are installed
+@pytest.mark.usefixtures("backend")
+def test_default_backend_no_duckdb():
+    script = """\
+import sys
+sys.modules["duckdb"] = None
 
-    df = pd.DataFrame({'a': [1, 2, 3]})
-    t = ibis.memtable(df)
-    expr = t.a.sum()
+import ibis
 
-    # run this twice to ensure that we hit the optimizations in
-    # `_default_backend`
-    for _ in range(2):
-        with pytest.raises(com.IbisError, match="Expression depends on no backends"):
-            expr.execute()
+t = ibis.memtable([{'a': 1}, {'a': 2}, {'a': 3}])
+t.execute()"""
+
+    args = [sys.executable, "-c", script]
+    with pytest.raises(subprocess.CalledProcessError) as e:
+        subprocess.check_output(args, stderr=subprocess.STDOUT, universal_newlines=True)
+    assert (
+        re.search(
+            "You have used a function that relies on the default backend",
+            e.value.output,
+        )
+        is not None
+    )
+
+
+@pytest.mark.usefixtures("backend")
+def test_default_backend_no_duckdb_read_parquet():
+    script = """\
+import sys
+sys.modules["duckdb"] = None
+
+import ibis
+ibis.read_parquet("foo.parquet")"""
+
+    args = [sys.executable, "-c", script]
+    with pytest.raises(subprocess.CalledProcessError) as e:
+        subprocess.check_output(args, stderr=subprocess.STDOUT, universal_newlines=True)
+    assert (
+        re.search(
+            "You have used a function that relies on the default backend",
+            e.value.output,
+        )
+        is not None
+    )
 
 
 @pytest.mark.duckdb
@@ -779,64 +875,88 @@ def test_dunder_array_column(alltypes, dtype):
 
 
 @pytest.mark.parametrize("interactive", [True, False])
-def test_repr(alltypes, interactive):
-    expr = alltypes.select("id", "int_col")
+def test_repr(alltypes, interactive, monkeypatch):
+    monkeypatch.setattr(ibis.options, "interactive", interactive)
 
-    val = str(alltypes.limit(5).id.execute().iloc[0])
+    expr = alltypes.select("date_string_col")
 
-    old = ibis.options.interactive
-    ibis.options.interactive = interactive
-    try:
-        s = repr(expr)
-        # no control characters
-        assert all(c.isprintable() or c in "\n\r\t" for c in s)
-        assert "id" in s
-        if interactive:
-            assert val in s
-        else:
-            assert val not in s
-    finally:
-        ibis.options.interactive = old
+    s = repr(expr)
+    # no control characters
+    assert all(c.isprintable() or c in "\n\r\t" for c in s)
+    if interactive:
+        assert "/" in s
+    else:
+        assert "/" not in s
 
 
 @pytest.mark.parametrize("show_types", [True, False])
-def test_interactive_repr_show_types(alltypes, show_types):
+def test_interactive_repr_show_types(alltypes, show_types, monkeypatch):
+    monkeypatch.setattr(ibis.options, "interactive", True)
+    monkeypatch.setattr(ibis.options.repr.interactive, "show_types", show_types)
+
     expr = alltypes.select("id")
-    old = ibis.options.interactive
-    ibis.options.interactive = True
-    ibis.options.repr.interactive.show_types = show_types
-    try:
-        s = repr(expr)
-        if show_types:
-            assert "int" in s
-        else:
-            assert "int" not in s
-    finally:
-        ibis.options.interactive = old
+    s = repr(expr)
+    if show_types:
+        assert "int" in s
+    else:
+        assert "int" not in s
+
+
+@pytest.mark.parametrize("is_jupyter", [True, False])
+def test_interactive_repr_max_columns(alltypes, is_jupyter, monkeypatch):
+    monkeypatch.setattr(ibis.options, "interactive", True)
+
+    cols = {f"c_{i}": ibis._.id + i for i in range(50)}
+    expr = alltypes.mutate(**cols).select(*cols)
+
+    console = rich.console.Console(force_jupyter=is_jupyter)
+    console.options.max_width = 80
+    options = console.options.copy()
+
+    # max_columns = 0
+    text = "".join(s.text for s in console.render(expr, options))
+    assert " c_0 " in text
+    if is_jupyter:
+        # Default of 20 columns are written
+        assert " c_19 " in text
+    else:
+        # width calculations truncates well before 20 columns
+        assert " c_19 " not in text
+
+    # max_columns = 3
+    monkeypatch.setattr(ibis.options.repr.interactive, "max_columns", 3)
+    text = "".join(s.text for s in console.render(expr, options))
+    assert " c_2 " in text
+    assert " c_3 " not in text
+
+    # max_columns = None
+    monkeypatch.setattr(ibis.options.repr.interactive, "max_columns", None)
+    text = "".join(s.text for s in console.render(expr, options))
+    assert " c_0 " in text
+    if is_jupyter:
+        # All columns written
+        assert " c_49 " in text
+    else:
+        # width calculations still truncates
+        assert " c_19 " not in text
 
 
 @pytest.mark.parametrize("expr_type", ["table", "column"])
 @pytest.mark.parametrize("interactive", [True, False])
-def test_repr_mimebundle(alltypes, interactive, expr_type):
+def test_repr_mimebundle(alltypes, interactive, expr_type, monkeypatch):
+    monkeypatch.setattr(ibis.options, "interactive", interactive)
+
     if expr_type == "column":
-        expr = alltypes.id
+        expr = alltypes.date_string_col
     else:
-        expr = alltypes.select("id", "int_col")
+        expr = alltypes.select("date_string_col")
 
-    val = str(alltypes.limit(5).id.execute().iloc[0])
-
-    old = ibis.options.interactive
-    ibis.options.interactive = interactive
-    try:
-        reprs = expr._repr_mimebundle_(include=["text/plain", "text/html"], exclude=[])
-        for format in ["text/plain", "text/html"]:
-            assert "id" in reprs[format]
-            if interactive:
-                assert val in reprs[format]
-            else:
-                assert val not in reprs[format]
-    finally:
-        ibis.options.interactive = old
+    reprs = expr._repr_mimebundle_(include=["text/plain", "text/html"], exclude=[])
+    for format in ["text/plain", "text/html"]:
+        if interactive:
+            assert "r0.date_string_col" not in reprs[format]
+        else:
+            assert "r0.date_string_col" in reprs[format]
 
 
 @pytest.mark.never(
@@ -853,3 +973,323 @@ def test_has_operation_no_geo(con):
     """
     for op in [ops.GeoDistance, ops.GeoAsText, ops.GeoUnaryUnion]:
         assert not con.has_operation(op)
+
+
+def test_get_backend(con, alltypes, monkeypatch):
+    assert ibis.get_backend(alltypes) is con
+    assert ibis.get_backend(alltypes.id.min()) is con
+
+    with pytest.raises(com.IbisError, match="contains unbound tables"):
+        ibis.get_backend(ibis.table({"x": "int"}))
+
+    monkeypatch.setattr(ibis.options, "default_backend", con)
+    assert ibis.get_backend() is con
+    expr = ibis.literal(1) + 2
+    assert ibis.get_backend(expr) is con
+
+
+def test_set_backend(con, monkeypatch):
+    monkeypatch.setattr(ibis.options, "default_backend", None)
+    ibis.set_backend(con)
+    assert ibis.get_backend() is con
+
+
+@pytest.mark.parametrize(
+    "name",
+    [
+        param("duckdb", marks=mark.duckdb, id="duckdb"),
+        param("polars", marks=mark.polars, id="polars"),
+        param("sqlite", marks=mark.sqlite, id="sqlite"),
+    ],
+)
+def test_set_backend_name(name, monkeypatch):
+    # Don't need to test with all backends, only checking that things are
+    # plumbed through correctly.
+    monkeypatch.setattr(ibis.options, "default_backend", None)
+    ibis.set_backend(name)
+    assert ibis.get_backend().name == name
+
+
+@pytest.mark.parametrize(
+    "url",
+    [
+        param(
+            "clickhouse://default@localhost:9000/ibis_testing",
+            marks=mark.clickhouse,
+            id="clickhouse",
+        ),
+        param(
+            "mysql://ibis:ibis@localhost:3306/ibis_testing",
+            marks=mark.mysql,
+            id="mysql",
+        ),
+        param(
+            "postgres://postgres:postgres@localhost:5432/ibis_testing",
+            marks=mark.postgres,
+            id="postgres",
+        ),
+    ],
+)
+def test_set_backend_url(url, monkeypatch):
+    # Don't need to test with all backends, only checking that things are
+    # plumbed through correctly.
+    monkeypatch.setattr(ibis.options, "default_backend", None)
+    name = url.split("://")[0]
+    ibis.set_backend(url)
+    assert ibis.get_backend().name == name
+
+
+@pytest.mark.notyet(
+    [
+        "bigquery",
+        "dask",
+        "datafusion",
+        "duckdb",
+        "impala",
+        "mssql",
+        "mysql",
+        "pandas",
+        "polars",
+        "postgres",
+        "pyspark",
+        "sqlite",
+    ],
+    reason="backend doesn't support timestamp with scale parameter",
+)
+@pytest.mark.notimpl(["clickhouse"], reason="create table isn't implemented")
+@pytest.mark.notimpl(
+    ["snowflake"], reason="scale not implemented in ibis's snowflake backend"
+)
+@pytest.mark.broken(["druid"], reason="sqlalchemy dialect is broken")
+def test_create_table_timestamp(con):
+    schema = ibis.schema(
+        dict(zip(string.ascii_letters, map("timestamp({:d})".format, range(10))))
+    )
+    name = f"timestamp_scale_{guid()}"
+    con.create_table(name, schema=schema, overwrite=True)
+    try:
+        rows = con.raw_sql(f"DESCRIBE {name}").fetchall()
+        result = ibis.schema((name, typ) for name, typ, *_ in rows)
+        assert result == schema
+    finally:
+        con.drop_table(name, force=True)
+
+
+@mark.notimpl(
+    [
+        "clickhouse",
+        "datafusion",
+        "bigquery",
+        "impala",
+        "pyspark",
+        "trino",
+        "druid",
+    ]
+)
+@mark.notyet(
+    ["sqlite"], reason="sqlite only support temporary tables in temporary databases"
+)
+@mark.never(
+    ["mssql"],
+    reason="mssql supports support temporary tables through naming conventions",
+)
+@mark.never(
+    ["pandas"],
+    raises=AssertionError,
+    reason="pandas doesn't cache anything and therefore does no ref counting",
+)
+def test_persist_expression_ref_count(con, alltypes):
+    non_persisted_table = alltypes.mutate(test_column="calculation")
+    persisted_table = non_persisted_table.cache()
+
+    op = non_persisted_table.op()
+
+    # ref count is unaffected without a context manager
+    assert con._refs[op] == 1
+    tm.assert_frame_equal(non_persisted_table.to_pandas(), persisted_table.to_pandas())
+    assert con._refs[op] == 1
+
+
+@mark.notimpl(
+    [
+        "clickhouse",
+        "datafusion",
+        "bigquery",
+        "impala",
+        "pyspark",
+        "trino",
+        "druid",
+    ]
+)
+@mark.notyet(
+    ["sqlite"], reason="sqlite only support temporary tables in temporary databases"
+)
+@mark.never(
+    ["mssql"],
+    reason="mssql supports support temporary tables through naming conventions",
+)
+def test_persist_expression(alltypes):
+    non_persisted_table = alltypes.mutate(test_column="calculation", other_calc="xyz")
+    persisted_table = non_persisted_table.cache()
+    tm.assert_frame_equal(non_persisted_table.to_pandas(), persisted_table.to_pandas())
+
+
+@mark.notimpl(
+    [
+        "clickhouse",
+        "datafusion",
+        "bigquery",
+        "impala",
+        "pyspark",
+        "trino",
+        "druid",
+    ]
+)
+@mark.notyet(
+    ["sqlite"], reason="sqlite only support temporary tables in temporary databases"
+)
+@mark.never(
+    ["mssql"],
+    reason="mssql supports support temporary tables through naming conventions",
+)
+def test_persist_expression_contextmanager(alltypes):
+    non_cached_table = alltypes.mutate(
+        test_column="calculation", other_column="big calc"
+    )
+    with non_cached_table.cache() as cached_table:
+        tm.assert_frame_equal(non_cached_table.to_pandas(), cached_table.to_pandas())
+
+
+@mark.notimpl(
+    [
+        "clickhouse",
+        "datafusion",
+        "bigquery",
+        "impala",
+        "pyspark",
+        "trino",
+        "druid",
+    ]
+)
+@mark.notyet(
+    ["sqlite"], reason="sqlite only support temporary tables in temporary databases"
+)
+@mark.never(
+    ["mssql"],
+    reason="mssql supports support temporary tables through naming conventions",
+)
+@mark.never(
+    ["pandas"],
+    raises=AssertionError,
+    reason="pandas doesn't cache anything and therefore does no ref counting",
+)
+def test_persist_expression_contextmanager_ref_count(con, alltypes):
+    non_cached_table = alltypes.mutate(
+        test_column="calculation", other_column="big calc 2"
+    )
+    op = non_cached_table.op()
+    with non_cached_table.cache() as cached_table:
+        tm.assert_frame_equal(non_cached_table.to_pandas(), cached_table.to_pandas())
+        assert con._refs[op] == 1
+    assert con._refs[op] == 0
+
+
+@mark.notimpl(
+    [
+        "clickhouse",
+        "datafusion",
+        "bigquery",
+        "impala",
+        "pyspark",
+        "trino",
+        "druid",
+    ]
+)
+@mark.notyet(
+    ["sqlite"], reason="sqlite only support temporary tables in temporary databases"
+)
+@mark.never(
+    ["mssql"],
+    reason="mssql supports support temporary tables through naming conventions",
+)
+@mark.never(
+    ["pandas"],
+    raises=AssertionError,
+    reason="pandas doesn't cache anything and therefore does no ref counting",
+)
+def test_persist_expression_multiple_refs(con, alltypes):
+    non_cached_table = alltypes.mutate(
+        test_column="calculation", other_column="big calc 2"
+    )
+    op = non_cached_table.op()
+    with non_cached_table.cache() as cached_table:
+        tm.assert_frame_equal(non_cached_table.to_pandas(), cached_table.to_pandas())
+        with non_cached_table.cache() as nested_cached_table:
+            assert not nested_cached_table.to_pandas().empty
+            assert con._refs[op] == 2
+        assert con._refs[op] == 1
+    assert con._refs[op] == 0
+
+
+@mark.notimpl(
+    [
+        "clickhouse",
+        "datafusion",
+        "bigquery",
+        "impala",
+        "pyspark",
+        "trino",
+        "druid",
+    ]
+)
+@mark.notyet(
+    ["sqlite"], reason="sqlite only support temporary tables in temporary databases"
+)
+@mark.never(
+    ["mssql"],
+    reason="mssql supports support temporary tables through naming conventions",
+)
+def test_persist_expression_repeated_cache(alltypes):
+    non_cached_table = alltypes.mutate(
+        test_column="calculation", other_column="big calc 2"
+    )
+    with non_cached_table.cache() as cached_table:
+        with cached_table.cache() as nested_cached_table:
+            assert not nested_cached_table.to_pandas().empty
+
+
+@mark.notimpl(
+    [
+        "clickhouse",
+        "datafusion",
+        "bigquery",
+        "impala",
+        "pyspark",
+        "trino",
+        "druid",
+    ]
+)
+@mark.notyet(
+    ["sqlite"], reason="sqlite only support temporary tables in temporary databases"
+)
+@mark.never(
+    ["mssql"],
+    reason="mssql supports support temporary tables through naming conventions",
+)
+@mark.never(["pandas"], reason="pandas does not need to release anything")
+def test_persist_expression_release(con, alltypes):
+    non_cached_table = alltypes.mutate(
+        test_column="calculation", other_column="big calc 3"
+    )
+    cached_table = non_cached_table.cache()
+    cached_table.release()
+    assert con._refs[non_cached_table.op()] == 0
+
+    with pytest.raises(
+        com.IbisError,
+        match=r".+Did you call `\.release\(\)` twice on the same expression\?",
+    ):
+        cached_table.release()
+
+    with pytest.raises(Exception, match=cached_table.op().name):
+        cached_table.execute()
