@@ -1,7 +1,7 @@
-import operator
-
 import pytest
 
+import ibis.common.exceptions as com
+
 
 @pytest.mark.notimpl(
     [
@@ -18,10 +18,12 @@ import pytest
         "pyspark",
         "snowflake",
         "trino",
-    ]
+        "druid",
+    ],
+    raises=com.OperationNotDefinedError,
 )
-def test_rowid(con):
-    t = con.table('functional_alltypes')
+def test_rowid(backend):
+    t = backend.diamonds
     result = t.rowid().execute()
     # Only guarantee is that the values are unique integers
     assert result.is_unique
@@ -36,27 +38,9 @@ def test_rowid(con):
     "column",
     ["string_col", "double_col", "date_string_col", "timestamp_col"],
 )
-@pytest.mark.notimpl(["datafusion"])
+@pytest.mark.notimpl(["datafusion"], raises=com.OperationNotDefinedError)
 def test_distinct_column(alltypes, df, column):
     expr = alltypes[[column]].distinct()
     result = expr.execute()
     expected = df[[column]].drop_duplicates()
     assert set(result) == set(expected)
-
-
-@pytest.mark.parametrize(
-    ("opname", "expected"),
-    [
-        ("year", {2009, 2010}),
-        ("month", set(range(1, 13))),
-        ("day", set(range(1, 32))),
-    ],
-)
-@pytest.mark.notimpl(["datafusion"])
-@pytest.mark.notyet(["impala"])
-def test_date_extract_field(con, opname, expected):
-    op = operator.methodcaller(opname)
-    t = con.table("functional_alltypes")
-    expr = t[op(t.timestamp_col.cast("date")).name("date")].distinct()
-    result = expr.execute()["date"].astype(int)
-    assert set(result) == expected
