@@ -1,20 +1,82 @@
+import contextlib
 import decimal
-import io
-from contextlib import redirect_stdout
-from operator import and_, invert, lshift, neg, or_, rshift, xor
+from operator import invert, methodcaller, neg
 
 import numpy as np
 import pandas as pd
 import pytest
+import sqlalchemy as sa
 import toolz
 from pytest import param
 
 import ibis
 import ibis.common.exceptions as com
 import ibis.expr.datatypes as dt
+import ibis.expr.selectors as s
 from ibis import _
 from ibis import literal as L
 
+try:
+    import duckdb
+
+    DuckDBConversionException = duckdb.ConversionException
+except ImportError:
+    DuckDBConversionException = None
+
+try:
+    import clickhouse_driver
+
+    ClickhouseDriverOperationalError = clickhouse_driver.dbapi.errors.OperationalError
+except ImportError:
+    ClickhouseDriverOperationalError = None
+
+
+NULL_BACKEND_TYPES = {
+    'bigquery': "NULL",
+    'clickhouse': 'Nullable(Nothing)',
+    'duckdb': "NULL",
+    'impala': 'BOOLEAN',
+    'snowflake': None,
+    'sqlite': "null",
+    'trino': 'unknown',
+    "postgres": "null",
+}
+
+
+@pytest.mark.broken(["duckdb", "impala", "bigquery"], 'assert nan is None')
+@pytest.mark.notimpl(["datafusion"])
+def test_null_literal(con, backend):
+    expr = ibis.null()
+    result = con.execute(expr)
+    assert result is None
+
+    with contextlib.suppress(com.OperationNotDefinedError):
+        backend_name = backend.name()
+        assert con.execute(expr.typeof()) == NULL_BACKEND_TYPES[backend_name]
+
+
+BOOLEAN_BACKEND_TYPE = {
+    'bigquery': "BOOL",
+    'clickhouse': 'UInt8',
+    'impala': 'BOOLEAN',
+    'snowflake': "BOOLEAN",
+    'sqlite': "integer",
+    'trino': 'boolean',
+    "duckdb": "BOOLEAN",
+    "postgres": "boolean",
+}
+
+
+def test_boolean_literal(con, backend):
+    expr = ibis.literal(False, type=dt.boolean)
+    result = con.execute(expr)
+    assert not result
+    assert type(result) in (np.bool_, bool)
+
+    with contextlib.suppress(com.OperationNotDefinedError):
+        backend_name = backend.name()
+        assert con.execute(expr.typeof()) == BOOLEAN_BACKEND_TYPE[backend_name]
+
 
 @pytest.mark.parametrize(
     ('expr', 'expected'),
@@ -22,13 +84,13 @@ from ibis import literal as L
         param(
             ibis.NA.fillna(5),
             5,
-            marks=pytest.mark.notimpl(["mssql"]),
+            marks=pytest.mark.notimpl(["mssql", "druid"]),
             id="na_fillna",
         ),
         param(
             L(5).fillna(10),
             5,
-            marks=pytest.mark.notimpl(["mssql"]),
+            marks=pytest.mark.notimpl(["mssql", "druid"]),
             id="non_na_fillna",
         ),
         param(L(5).nullif(5), None, id="nullif_null"),
@@ -65,7 +127,7 @@ def test_scalar_fillna_nullif(con, expr, expected):
         ),
     ],
 )
-@pytest.mark.notimpl(["mssql"])
+@pytest.mark.notimpl(["mssql", "druid"])
 def test_isna(backend, alltypes, col, filt):
     table = alltypes.select(
         nan_col=ibis.literal(np.nan), none_col=ibis.NA.cast("float64")
@@ -102,7 +164,7 @@ def test_isna(backend, alltypes, col, filt):
         ),
     ],
 )
-@pytest.mark.notimpl(["datafusion", "mssql"])
+@pytest.mark.notimpl(["datafusion", "mssql", "druid"])
 def test_column_fillna(backend, alltypes, value):
     table = alltypes.mutate(missing=ibis.literal(value).cast("float64"))
     pd_table = table.execute()
@@ -115,27 +177,9 @@ def test_column_fillna(backend, alltypes, value):
 @pytest.mark.parametrize(
     ('expr', 'expected'),
     [
-        param(
-            ibis.coalesce(5, None, 4),
-            5,
-            id="generic",
-            marks=[
-                pytest.mark.broken(
-                    "polars",
-                    reason="implementation error, cannot get ref Int8 from Boolean",
-                ),
-            ],
-        ),
+        param(ibis.coalesce(5, None, 4), 5, id="generic"),
         param(ibis.coalesce(ibis.NA, 4, ibis.NA), 4, id="null_start_end"),
-        param(
-            ibis.coalesce(ibis.NA, ibis.NA, 3.14),
-            3.14,
-            id="non_null_last",
-            marks=pytest.mark.broken(
-                "polars",
-                reason="implementation error, cannot get ref Int8 from Float64",
-            ),
-        ),
+        param(ibis.coalesce(ibis.NA, ibis.NA, 3.14), 3.14, id="non_null_last"),
     ],
 )
 @pytest.mark.notimpl(["datafusion"])
@@ -152,7 +196,9 @@ def test_coalesce(con, expr, expected):
 
 
 # TODO(dask) - identicalTo - #2553
-@pytest.mark.notimpl(["clickhouse", "datafusion", "polars", "dask", "pyspark", "mssql"])
+@pytest.mark.notimpl(
+    ["clickhouse", "datafusion", "polars", "dask", "pyspark", "mssql", "druid"]
+)
 def test_identical_to(backend, alltypes, sorted_df):
     sorted_alltypes = alltypes.order_by('id')
     df = sorted_df
@@ -181,7 +227,7 @@ def test_identical_to(backend, alltypes, sorted_df):
         ('int_col', frozenset({1})),
     ],
 )
-@pytest.mark.notimpl(["mssql"])
+@pytest.mark.notimpl(["mssql", "druid"])
 def test_isin(backend, alltypes, sorted_df, column, elements):
     sorted_alltypes = alltypes.order_by('id')
     expr = sorted_alltypes[
@@ -205,7 +251,7 @@ def test_isin(backend, alltypes, sorted_df, column, elements):
         ('int_col', frozenset({1})),
     ],
 )
-@pytest.mark.notimpl(["mssql"])
+@pytest.mark.notimpl(["mssql", "druid"])
 def test_notin(backend, alltypes, sorted_df, column, elements):
     sorted_alltypes = alltypes.order_by('id')
     expr = sorted_alltypes[
@@ -246,6 +292,7 @@ def test_notin(backend, alltypes, sorted_df, column, elements):
         ),
     ],
 )
+@pytest.mark.notimpl(["druid"])
 def test_filter(backend, alltypes, sorted_df, predicate_fn, expected_fn):
     sorted_alltypes = alltypes.order_by('id')
     table = sorted_alltypes[predicate_fn(sorted_alltypes)].order_by('id')
@@ -269,6 +316,7 @@ def test_filter(backend, alltypes, sorted_df, predicate_fn, expected_fn):
         "polars",
         "mssql",
         "trino",
+        "druid",
     ]
 )
 def test_filter_with_window_op(backend, alltypes, sorted_df):
@@ -313,7 +361,7 @@ def test_case_where(backend, alltypes, df):
 
 
 # TODO: some of these are notimpl (datafusion) others are probably never
-@pytest.mark.notimpl(["datafusion", "mysql", "sqlite", "mssql"])
+@pytest.mark.notimpl(["datafusion", "mysql", "sqlite", "mssql", "druid"])
 @pytest.mark.min_version(duckdb="0.3.3", reason="isnan/isinf unsupported")
 def test_select_filter_mutate(backend, alltypes, df):
     """Test that select, filter and mutate are executed in right order.
@@ -345,7 +393,9 @@ def test_select_filter_mutate(backend, alltypes, df):
 
 
 def test_table_fillna_invalid(alltypes):
-    with pytest.raises(com.IbisTypeError, match=r"'invalid_col' is not a field in.*"):
+    with pytest.raises(
+        com.IbisTypeError, match=r"Column 'invalid_col' is not found in table"
+    ):
         alltypes.fillna({'invalid_col': 0.0})
 
     with pytest.raises(
@@ -367,7 +417,7 @@ def test_table_fillna_invalid(alltypes):
         {"double_col": -1.5, "string_col": "missing"},
     ],
 )
-@pytest.mark.notimpl(["datafusion", "mssql", "clickhouse"])
+@pytest.mark.notimpl(["datafusion", "mssql", "clickhouse", "druid"])
 def test_table_fillna_mapping(backend, alltypes, replacements):
     table = alltypes.mutate(
         int_col=alltypes.int_col.nullif(1),
@@ -382,7 +432,7 @@ def test_table_fillna_mapping(backend, alltypes, replacements):
     backend.assert_frame_equal(result, expected, check_dtype=False)
 
 
-@pytest.mark.notimpl(["datafusion", "mssql", "clickhouse"])
+@pytest.mark.notimpl(["datafusion", "mssql", "clickhouse", "druid"])
 def test_table_fillna_scalar(backend, alltypes):
     table = alltypes.mutate(
         int_col=alltypes.int_col.nullif(1),
@@ -412,7 +462,9 @@ def test_mutate_rename(alltypes):
 
 
 def test_dropna_invalid(alltypes):
-    with pytest.raises(com.IbisTypeError, match=r"'invalid_col' is not a field in.*"):
+    with pytest.raises(
+        com.IbisTypeError, match=r"Column 'invalid_col' is not found in table"
+    ):
         alltypes.dropna(subset=['invalid_col'])
 
     with pytest.raises(ValueError, match=r".*is not in.*"):
@@ -477,17 +529,14 @@ def test_select_sort_sort(alltypes):
         ),
     ],
 )
+@pytest.mark.notimpl(["druid"])
 def test_order_by(backend, alltypes, df, key, df_kwargs):
     result = alltypes.filter(_.id < 100).order_by(key).execute()
     expected = df.loc[df.id < 100].sort_values(**df_kwargs)
     backend.assert_frame_equal(result, expected)
 
 
-@pytest.mark.notimpl(["dask", "datafusion", "impala", "pandas", "polars", "mssql"])
-@pytest.mark.notyet(
-    ["clickhouse"],
-    reason="clickhouse doesn't have a [0.0, 1.0) random implementation",
-)
+@pytest.mark.notimpl(["dask", "pandas", "polars", "mssql", "druid"])
 def test_order_by_random(alltypes):
     expr = alltypes.filter(_.id < 100).order_by(ibis.random()).limit(5)
     r1 = expr.execute()
@@ -498,25 +547,26 @@ def test_order_by_random(alltypes):
     assert not r1.equals(r2)
 
 
-def check_table_info(buf, schema):
-    info_str = buf.getvalue()
-
-    assert "Null" in info_str
-    assert all(type.__class__.__name__ in info_str for type in schema.types)
-    assert all(name in info_str for name in schema.names)
-
-
-def test_table_info_buf(alltypes):
-    buf = io.StringIO()
-    alltypes.info(buf=buf)
-    check_table_info(buf, alltypes.schema())
-
-
-def test_table_info_no_buf(alltypes):
-    buf = io.StringIO()
-    with redirect_stdout(buf):
-        alltypes.info()
-    check_table_info(buf, alltypes.schema())
+@pytest.mark.notyet(
+    ["druid"],
+    raises=sa.exc.ProgrammingError,
+    reason="Druid only supports trivial unions",
+)
+@pytest.mark.notimpl(["datafusion"], raises=com.OperationNotDefinedError)
+def test_table_info(alltypes):
+    expr = alltypes.info()
+    df = expr.execute()
+    assert alltypes.columns == list(df.name)
+    assert expr.columns == [
+        "name",
+        "type",
+        "nullable",
+        "nulls",
+        "non_nulls",
+        "null_frac",
+        "pos",
+    ]
+    assert expr.columns == list(df.columns)
 
 
 @pytest.mark.parametrize(
@@ -556,7 +606,7 @@ def test_isin_notin(backend, alltypes, df, ibis_op, pandas_op):
     reason="dask doesn't support Series as isin/notin argument",
     raises=NotImplementedError,
 )
-@pytest.mark.notimpl(["datafusion"])
+@pytest.mark.notimpl(["datafusion", "druid"])
 @pytest.mark.parametrize(
     ("ibis_op", "pandas_op"),
     [
@@ -624,7 +674,7 @@ def test_logical_negation_column(backend, alltypes, df, op):
     backend.assert_series_equal(result, expected, check_names=False)
 
 
-@pytest.mark.notimpl(["bigquery", "datafusion"])
+@pytest.mark.notimpl(["datafusion"])
 @pytest.mark.parametrize(
     ("dtype", "zero", "expected"),
     [("int64", 0, 1), ("float64", 0.0, 1.0)],
@@ -634,7 +684,7 @@ def test_zeroifnull_literals(con, dtype, zero, expected):
     assert con.execute(ibis.literal(expected, type=dtype).zeroifnull()) == expected
 
 
-@pytest.mark.notimpl(["bigquery", "datafusion"])
+@pytest.mark.notimpl(["datafusion"])
 @pytest.mark.min_version(
     dask="2022.01.1",
     reason="unsupported operation with later versions of pandas",
@@ -699,111 +749,17 @@ def test_select_filter_select(backend, alltypes, df):
     backend.assert_series_equal(result, expected)
 
 
-pyspark_no_bitshift = pytest.mark.notyet(
-    ["pyspark"], reason="pyspark doesn't implement bitshit operators"
-)
+@pytest.mark.notimpl(["datafusion"], raises=com.OperationNotDefinedError)
+@pytest.mark.broken(["mssql"], raises=sa.exc.OperationalError)
+def test_between(backend, alltypes, df):
+    expr = alltypes.double_col.between(5, 10)
+    result = expr.execute().rename("double_col")
 
-
-@pytest.mark.parametrize("op", [and_, or_, xor])
-@pytest.mark.parametrize(
-    ("left_fn", "right_fn"),
-    [
-        param(lambda t: t.int_col, lambda t: t.int_col, id="col_col"),
-        param(lambda _: 3, lambda t: t.int_col, id="scalar_col"),
-        param(lambda t: t.int_col, lambda _: 3, id="col_scalar"),
-    ],
-)
-@pytest.mark.notimpl(["bigquery", "dask", "datafusion", "pandas", "snowflake"])
-def test_bitwise_columns(backend, con, alltypes, df, op, left_fn, right_fn):
-    expr = op(left_fn(alltypes), right_fn(alltypes)).name("tmp")
-    result = con.execute(expr)
-
-    expected = op(left_fn(df), right_fn(df)).rename("tmp")
+    expected = df.double_col.between(5, 10)
     backend.assert_series_equal(result, expected)
 
 
-@pytest.mark.parametrize(
-    ("op", "left_fn", "right_fn"),
-    [
-        param(
-            lshift,
-            lambda t: t.int_col,
-            lambda t: t.int_col,
-            id="lshift_col_col",
-        ),
-        param(
-            lshift,
-            lambda _: 3,
-            lambda t: t.int_col,
-            marks=pytest.mark.broken(
-                ["impala"],
-                reason="impala's behavior differs from every other backend",
-            ),
-            id="lshift_scalar_col",
-        ),
-        param(lshift, lambda t: t.int_col, lambda _: 3, id="lshift_col_scalar"),
-        param(rshift, lambda t: t.int_col, lambda t: t.int_col, id="rshift_col_col"),
-        param(rshift, lambda _: 3, lambda t: t.int_col, id="rshift_scalar_col"),
-        param(rshift, lambda t: t.int_col, lambda _: 3, id="rshift_col_scalar"),
-    ],
-)
-@pytest.mark.notimpl(["bigquery", "dask", "datafusion", "pandas"])
-@pyspark_no_bitshift
-def test_bitwise_shift(backend, alltypes, df, op, left_fn, right_fn):
-    expr = op(left_fn(alltypes), right_fn(alltypes)).name("tmp")
-    result = expr.execute()
-
-    pandas_left = getattr(left := left_fn(df), "values", left)
-    pandas_right = getattr(right := right_fn(df), "values", right)
-    expected = pd.Series(
-        op(pandas_left, pandas_right),
-        name="tmp",
-        dtype="int64",
-    )
-    backend.assert_series_equal(result, expected)
-
-
-@pytest.mark.parametrize(
-    "op",
-    [
-        param(and_, marks=[pytest.mark.notimpl(["snowflake"])]),
-        param(or_, marks=[pytest.mark.notimpl(["snowflake"])]),
-        param(xor, marks=[pytest.mark.notimpl(["snowflake"])]),
-        param(lshift, marks=pyspark_no_bitshift),
-        param(rshift, marks=pyspark_no_bitshift),
-    ],
-)
-@pytest.mark.parametrize(
-    ("left", "right"),
-    [param(4, L(2), id="int_col"), param(L(4), 2, id="col_int")],
-)
-@pytest.mark.notimpl(["bigquery", "dask", "datafusion", "pandas"])
-def test_bitwise_scalars(con, op, left, right):
-    expr = op(left, right)
-    result = con.execute(expr)
-    expected = op(4, 2)
-    assert result == expected
-
-
-@pytest.mark.notimpl(["bigquery", "dask", "datafusion", "pandas", "snowflake"])
-def test_bitwise_not_scalar(con):
-    expr = ~L(2)
-    result = con.execute(expr)
-    expected = -3
-    assert result == expected
-
-
-@pytest.mark.notimpl(["bigquery", "dask", "datafusion", "pandas", "snowflake"])
-def test_bitwise_not_col(backend, alltypes, df):
-    expr = (~alltypes.int_col).name("tmp")
-    result = expr.execute()
-    expected = ~df.int_col
-    backend.assert_series_equal(result, expected.rename("tmp"))
-
-
-@pytest.mark.notimpl(
-    ["snowflake"], reason="snowflake doesn't implement timestamp subtraction"
-)
+@pytest.mark.notimpl(["druid"])
 def test_interactive(alltypes):
     expr = alltypes.mutate(
         str_col=_.string_col.replace("1", "").nullif("2"),
@@ -824,6 +780,16 @@ def test_correlated_subquery(alltypes):
     assert expr.compile() is not None
 
 
+@pytest.mark.notimpl(["dask", "polars", "pyspark", "datafusion"])
+def test_uncorrelated_subquery(backend, batting, batting_df):
+    subset_batting = batting[batting.yearID <= 2000]
+    expr = batting[_.yearID == subset_batting.yearID.max()]['playerID', 'yearID']
+    result = expr.execute()
+
+    expected = batting_df[batting_df.yearID == 2000][['playerID', 'yearID']]
+    backend.assert_frame_equal(result, expected)
+
+
 def test_int_column(alltypes):
     expr = alltypes.mutate(x=1).x
     result = expr.execute()
@@ -831,7 +797,7 @@ def test_int_column(alltypes):
     assert result.dtype == np.int8
 
 
-@pytest.mark.notimpl(["datafusion"])
+@pytest.mark.notimpl(["datafusion", "druid"])
 @pytest.mark.never(
     ["bigquery", "sqlite", "snowflake"], reason="backend only implements int64"
 )
@@ -840,3 +806,169 @@ def test_int_scalar(alltypes):
     result = expr.execute()
     assert expr.type() == dt.int16
     assert result.dtype == np.int16
+
+
+@pytest.mark.notimpl(["dask", "datafusion", "pandas", "polars", "druid"])
+@pytest.mark.notyet(
+    ["clickhouse"], reason="https://github.com/ClickHouse/ClickHouse/issues/6697"
+)
+@pytest.mark.notyet(["pyspark"])
+@pytest.mark.parametrize("method_name", ["any", "notany"])
+def test_exists(batting, awards_players, method_name):
+    method = methodcaller(method_name)
+    expr = batting[method(batting.yearID == awards_players.yearID)]
+    result = expr.execute()
+    assert not result.empty
+
+
+@pytest.mark.notimpl(
+    [
+        "dask",
+        "datafusion",
+        "mssql",
+        "mysql",
+        "pandas",
+        "pyspark",
+        "polars",
+        "druid",
+    ],
+    raises=com.OperationNotDefinedError,
+)
+def test_typeof(backend, con):
+    # Other tests also use the typeof operation, but only this test has this operation required.
+    expr = ibis.literal(1).typeof()
+    result = con.execute(expr)
+
+    assert result is not None
+
+
+@pytest.mark.broken(["polars"], reason="incorrect answer")
+@pytest.mark.notyet(["impala"], reason="can't find table in subquery")
+@pytest.mark.notimpl(["datafusion", "pyspark", "druid"])
+@pytest.mark.notyet(["dask", "mssql"], reason="not supported by the backend")
+def test_isin_uncorrelated(
+    backend, batting, awards_players, batting_df, awards_players_df
+):
+    expr = batting.select(
+        "playerID",
+        "yearID",
+        has_year_id=batting.yearID.isin(awards_players.yearID),
+    ).order_by(["playerID", "yearID"])
+    result = expr.execute().has_year_id
+    expected = (
+        batting_df.sort_values(["playerID", "yearID"])
+        .reset_index(drop=True)
+        .yearID.isin(awards_players_df.yearID)
+        .rename("has_year_id")
+    )
+    backend.assert_series_equal(result, expected)
+
+
+@pytest.mark.broken(["polars"], reason="incorrect answer")
+@pytest.mark.notimpl(["datafusion", "pyspark", "druid"])
+@pytest.mark.notyet(["dask"], reason="not supported by the backend")
+def test_isin_uncorrelated_filter(
+    backend, batting, awards_players, batting_df, awards_players_df
+):
+    expr = (
+        batting.select("playerID", "yearID")
+        .filter(batting.yearID.isin(awards_players.yearID))
+        .order_by(["playerID", "yearID"])
+    )
+    result = expr.execute()
+    expected = (
+        batting_df.loc[
+            batting_df.yearID.isin(awards_players_df.yearID), ["playerID", "yearID"]
+        ]
+        .sort_values(["playerID", "yearID"])
+        .reset_index(drop=True)
+    )
+    backend.assert_frame_equal(result, expected)
+
+
+@pytest.mark.parametrize(
+    "dtype",
+    [
+        "bool",
+        param("bytes", marks=pytest.mark.notyet(["polars"])),
+        "str",
+        "int",
+        "float",
+        "int8",
+        "int16",
+        "int32",
+        "int64",
+        "float32",
+        "float64",
+        "timestamp",
+        "date",
+        param("time", marks=pytest.mark.notyet(["datafusion"])),
+    ],
+)
+def test_literal_na(con, dtype):
+    expr = ibis.literal(None, type=dtype)
+    result = con.execute(expr)
+    assert pd.isna(result)
+
+
+@pytest.mark.notimpl(["datafusion"], raises=com.OperationNotDefinedError)
+@pytest.mark.notimpl(["dask", "pandas"], raises=com.UnboundExpressionError)
+def test_memtable_bool_column(backend, con, monkeypatch):
+    monkeypatch.setattr(ibis.options, "default_backend", con)
+
+    t = ibis.memtable({"a": [True, False, True]})
+    backend.assert_series_equal(t.a.execute(), pd.Series([True, False, True], name="a"))
+
+
+@pytest.mark.notimpl(
+    ["dask", "datafusion", "pandas", "polars"],
+    raises=NotImplementedError,
+    reason="not a SQL backend",
+)
+@pytest.mark.notimpl(["pyspark"], raises=com.OperationNotDefinedError)
+@pytest.mark.notimpl(["druid"], reason="no sqlglot dialect", raises=ValueError)
+def test_many_subqueries(con, snapshot):
+    def query(t, group_cols):
+        t2 = t.mutate(key=ibis.row_number().over(ibis.window(order_by=group_cols)))
+        return t2.inner_join(t2[["key"]], "key")
+
+    t = ibis.table(dict(street="str"), name="data")
+
+    t2 = query(t, group_cols=["street"])
+    t3 = query(t2, group_cols=["street"])
+
+    snapshot.assert_match(str(ibis.to_sql(t3, dialect=con.name)), "out.sql")
+
+
+@pytest.mark.notimpl(["dask", "pandas"], raises=com.OperationNotDefinedError)
+@pytest.mark.notyet(["polars"], reason="polars doesn't expand > 1 explode")
+@pytest.mark.notimpl(["druid"], raises=AssertionError)
+@pytest.mark.notyet(
+    ["pyspark"], reason="pyspark doesn't allow more than one explode in a select"
+)
+@pytest.mark.notyet(
+    ["bigquery"],
+    reason="backend doesn't implement unnest",
+    raises=com.OperationNotDefinedError,
+)
+@pytest.mark.notyet(
+    ["datafusion", "impala", "mssql", "mysql", "sqlite"],
+    reason="backend doesn't support arrays and we don't implement pivot_longer with unions yet",
+    raises=com.OperationNotDefinedError,
+)
+def test_pivot_longer(backend):
+    diamonds = backend.diamonds
+    res = diamonds.pivot_longer(s.c("x", "y", "z"), names_to="pos", values_to="xyz")
+    assert res.schema().names == (
+        "carat",
+        "cut",
+        "color",
+        "clarity",
+        "depth",
+        "table",
+        "price",
+        "pos",
+        "xyz",
+    )
+    df = res.limit(5).execute()
+    assert not df.empty
