@@ -9,6 +9,8 @@ import ibis.expr.datatypes as dt
 from ibis import _
 from ibis.udf.vectorized import analytic, reduction
 
+pytestmark = pytest.mark.notimpl(["druid"])
+
 
 @reduction(input_type=[dt.double], output_type=dt.double)
 def mean_udf(s):
@@ -88,28 +90,19 @@ def calc_zscore(s):
             id='last',
         ),
         param(
-            lambda t, win: t.float_col.nth(3).over(win),
-            lambda t: t.float_col.apply(
+            lambda t, win: t.double_col.nth(3).over(win),
+            lambda t: t.double_col.apply(
                 lambda s: pd.concat(
                     [
-                        pd.Series(np.nan, index=s.index[:3], dtype="float32"),
-                        pd.Series(
-                            s.iloc[3],
-                            index=s.index[3:],
-                            dtype="float32",
-                        ),
+                        pd.Series(np.nan, index=s.index[:3], dtype="float64"),
+                        pd.Series(s.iloc[3], index=s.index[3:], dtype="float64"),
                     ]
                 )
             ),
             id="nth",
             marks=[
-                pytest.mark.notimpl(["pandas", "snowflake"]),
+                pytest.mark.notimpl(["pandas"]),
                 pytest.mark.notyet(["impala", "mssql"]),
-                pytest.mark.broken(
-                    "bigquery",
-                    strict=False,
-                    reason="output types is float64 instead of the expected float32",
-                ),
             ],
         ),
         param(
@@ -382,7 +375,6 @@ def test_grouped_bounded_following_window(backend, alltypes, df, preceding, foll
                 order_by=[t.id],
             ),
             id='preceding-2-following-0-tuple',
-            marks=pytest.mark.notimpl(["pandas"]),
         ),
         param(
             lambda t: ibis.trailing_window(
@@ -395,7 +387,6 @@ def test_grouped_bounded_following_window(backend, alltypes, df, preceding, foll
 @pytest.mark.notimpl(["dask", "datafusion", "polars"])
 def test_grouped_bounded_preceding_window(backend, alltypes, df, window_fn):
     window = window_fn(alltypes)
-
     expr = alltypes.mutate(val=alltypes.double_col.sum().over(window))
 
     result = expr.execute().set_index('id').sort_index()
@@ -449,7 +440,7 @@ def test_grouped_bounded_preceding_window(backend, alltypes, df, window_fn):
 @pytest.mark.parametrize(
     ('ordered'),
     [
-        param(True, id='ordered', marks=pytest.mark.notimpl(["dask", "pandas"])),
+        param(True, id='ordered', marks=pytest.mark.notimpl(["dask"])),
         param(False, id='unordered'),
     ],
 )
@@ -484,6 +475,30 @@ def test_grouped_unbounded_window(
     backend.assert_series_equal(left, right)
 
 
+@pytest.mark.parametrize(
+    ('ibis_fn', 'pandas_fn'),
+    [
+        (lambda col: col.sum(), lambda s: s.cumsum()),
+        (lambda col: col.min(), lambda s: s.cummin()),
+        (lambda col: col.mean(), lambda s: s.expanding().mean()),
+    ],
+)
+@pytest.mark.broken(["mssql", "pandas", "snowflake"])
+@pytest.mark.notimpl(["datafusion", "polars", "dask"])
+def test_simple_ungrouped_unbound_following_window(
+    backend, alltypes, ibis_fn, pandas_fn
+):
+    t = alltypes[alltypes.double_col < 50].order_by('id')
+    df = t.execute()
+
+    w = ibis.window(rows=(0, None), order_by=t.id)
+    expr = ibis_fn(t.double_col).over(w).name('double_col')
+    result = expr.execute()
+    expected = pandas_fn(df.double_col[::-1])[::-1]
+
+    backend.assert_series_equal(result, expected)
+
+
 @pytest.mark.parametrize(
     ("result_fn", "expected_fn", "ordered"),
     [
@@ -806,8 +821,8 @@ def test_mutate_window_filter(backend, alltypes, df):
 
 @pytest.mark.notimpl(["dask", "datafusion", "polars"])
 @pytest.mark.broken(["impala"], reason="the database returns incorrect results")
-def test_first_last(con):
-    t = con.table("win")
+def test_first_last(backend):
+    t = backend.win
     w = ibis.window(group_by=t.g, order_by=[t.x, t.y], preceding=1, following=0)
     expr = t.mutate(
         x_first=t.x.first().over(w),
