@@ -1,6 +1,7 @@
 import copy
 import pickle
 import weakref
+from typing import Mapping, Sequence, Tuple, TypeVar
 
 import pytest
 
@@ -9,10 +10,13 @@ from ibis.common.annotations import (
     Signature,
     argument,
     attribute,
-    mandatory,
     optional,
+    required,
+    varargs,
+    varkwargs,
 )
 from ibis.common.caching import WeakCache
+from ibis.common.collections import frozendict
 from ibis.common.grounds import (
     Annotable,
     Base,
@@ -21,9 +25,8 @@ from ibis.common.grounds import (
     Immutable,
     Singleton,
 )
-from ibis.common.validators import instance_of, validator
+from ibis.common.validators import Coercible, Validator, instance_of, option, validator
 from ibis.tests.util import assert_pickle_roundtrip
-from ibis.util import frozendict
 
 is_any = instance_of(object)
 is_bool = instance_of(bool)
@@ -68,6 +71,117 @@ class BetweenWithCalculated(Concrete):
         return self.value + self.lower
 
 
+class VariadicArgs(Concrete):
+    args = varargs(is_int)
+
+
+class VariadicKeywords(Concrete):
+    kwargs = varkwargs(is_int)
+
+
+class VariadicArgsAndKeywords(Concrete):
+    args = varargs(is_int)
+    kwargs = varkwargs(is_int)
+
+
+T = TypeVar('T')
+K = TypeVar('K')
+V = TypeVar('V')
+
+
+class List(Concrete, Sequence[T], Coercible):
+    @classmethod
+    def __coerce__(self, values):
+        values = tuple(values)
+        if values:
+            head, *rest = values
+            return ConsList(head, rest)
+        else:
+            return EmptyList()
+
+
+class EmptyList(List[T]):
+    def __getitem__(self, key):
+        raise IndexError(key)
+
+    def __len__(self):
+        return 0
+
+
+class ConsList(List[T]):
+    head: T
+    rest: List[T]
+
+    def __getitem__(self, key):
+        if key == 0:
+            return self.head
+        else:
+            return self.rest[key - 1]
+
+    def __len__(self):
+        return len(self.rest) + 1
+
+
+class Map(Concrete, Mapping[K, V], Coercible):
+    @classmethod
+    def __coerce__(self, pairs):
+        pairs = dict(pairs)
+        if pairs:
+            head_key = next(iter(pairs))
+            head_value = pairs.pop(head_key)
+            rest = pairs
+            return ConsMap((head_key, head_value), rest)
+        else:
+            return EmptyMap()
+
+
+class EmptyMap(Map[K, V]):
+    def __getitem__(self, key):
+        raise KeyError(key)
+
+    def __iter__(self):
+        return iter(())
+
+    def __len__(self):
+        return 0
+
+
+class ConsMap(Map[K, V]):
+    head: Tuple[K, V]
+    rest: Map[K, V]
+
+    def __getitem__(self, key):
+        if key == self.head[0]:
+            return self.head[1]
+        else:
+            return self.rest[key]
+
+    def __iter__(self):
+        yield self.head[0]
+        yield from self.rest
+
+    def __len__(self):
+        return len(self.rest) + 1
+
+
+class Integer(int, Coercible):
+    @classmethod
+    def __coerce__(cls, value):
+        return Integer(value)
+
+
+class Float(float, Coercible):
+    @classmethod
+    def __coerce__(cls, value):
+        return Float(value)
+
+
+class MyExpr(Concrete):
+    a: Integer
+    b: List[Float]
+    c: Map[str, Integer]
+
+
 def test_annotable():
     class InBetween(BetweenSimple):
         pass
@@ -146,6 +260,40 @@ def test_annotable_with_type_annotations():
         Op()
 
 
+def test_annotable_with_recursive_generic_type_annotations():
+    # testing cons list
+    validator = Validator.from_typehint(List[Integer])
+    values = ["1", 2.0, 3]
+    result = validator(values)
+    expected = ConsList(1, ConsList(2, ConsList(3, EmptyList())))
+    assert result == expected
+    assert result[0] == 1
+    assert result[1] == 2
+    assert result[2] == 3
+    assert len(result) == 3
+    with pytest.raises(IndexError):
+        result[3]
+
+    # testing cons map
+    validator = Validator.from_typehint(Map[Integer, Float])
+    values = {"1": 2, 3: "4.0", 5: 6.0}
+    result = validator(values)
+    expected = ConsMap((1, 2.0), ConsMap((3, 4.0), ConsMap((5, 6.0), EmptyMap())))
+    assert result == expected
+    assert result[1] == 2.0
+    assert result[3] == 4.0
+    assert result[5] == 6.0
+    assert len(result) == 3
+    with pytest.raises(KeyError):
+        result[7]
+
+    # testing both encapsulated in a class
+    expr = MyExpr(a="1", b=["2.0", 3, True], c={"a": "1", "b": 2, "c": 3.0})
+    assert expr.a == 1
+    assert expr.b == ConsList(2.0, ConsList(3.0, ConsList(1.0, EmptyList())))
+    assert expr.c == ConsMap(("a", 1), ConsMap(("b", 2), ConsMap(("c", 3), EmptyMap())))
+
+
 def test_composition_of_annotable_and_immutable():
     class AnnImm(Annotable, Immutable):
         value = is_int
@@ -223,22 +371,22 @@ def test_signature_inheritance():
 
     assert IntBinop.__signature__ == Signature(
         [
-            Parameter('left', annotation=mandatory(is_int)),
-            Parameter('right', annotation=mandatory(is_int)),
+            Parameter('left', annotation=required(is_int)),
+            Parameter('right', annotation=required(is_int)),
         ]
     )
 
     assert FloatAddRhs.__signature__ == Signature(
         [
-            Parameter('left', annotation=mandatory(is_int)),
-            Parameter('right', annotation=mandatory(is_float)),
+            Parameter('left', annotation=required(is_int)),
+            Parameter('right', annotation=required(is_float)),
         ]
     )
 
     assert FloatAddClip.__signature__ == Signature(
         [
-            Parameter('left', annotation=mandatory(is_float)),
-            Parameter('right', annotation=mandatory(is_float)),
+            Parameter('left', annotation=required(is_float)),
+            Parameter('right', annotation=required(is_float)),
             Parameter('clip_lower', annotation=optional(is_int, default=0)),
             Parameter('clip_upper', annotation=optional(is_int, default=10)),
         ]
@@ -246,8 +394,8 @@ def test_signature_inheritance():
 
     assert IntAddClip.__signature__ == Signature(
         [
-            Parameter('left', annotation=mandatory(is_int)),
-            Parameter('right', annotation=mandatory(is_int)),
+            Parameter('left', annotation=required(is_int)),
+            Parameter('right', annotation=required(is_int)),
             Parameter('clip_lower', annotation=optional(is_int, default=0)),
             Parameter('clip_upper', annotation=optional(is_int, default=10)),
         ]
@@ -303,6 +451,124 @@ def test_keyword_argument_reordering():
     assert obj.e == 4
 
 
+def test_variadic_argument_reordering():
+    class Test(Annotable):
+        a = is_int
+        b = is_int
+        args = varargs(is_int)
+
+    class Test2(Test):
+        c = is_int
+        args = varargs(is_int)
+
+    with pytest.raises(TypeError, match="missing a required argument: 'c'"):
+        Test2(1, 2)
+
+    a = Test2(1, 2, 3)
+    assert a.a == 1
+    assert a.b == 2
+    assert a.c == 3
+    assert a.args == ()
+
+    b = Test2(*range(5))
+    assert b.a == 0
+    assert b.b == 1
+    assert b.c == 2
+    assert b.args == (3, 4)
+
+    msg = "only one variadic \\*args parameter is allowed"
+    with pytest.raises(TypeError, match=msg):
+
+        class Test3(Test):
+            another_args = varargs(is_int)
+
+
+def test_variadic_keyword_argument_reordering():
+    class Test(Annotable):
+        a = is_int
+        b = is_int
+        options = varkwargs(is_int)
+
+    class Test2(Test):
+        c = is_int
+        options = varkwargs(is_int)
+
+    with pytest.raises(TypeError, match="missing a required argument: 'c'"):
+        Test2(1, 2)
+
+    a = Test2(1, 2, c=3)
+    assert a.a == 1
+    assert a.b == 2
+    assert a.c == 3
+    assert a.options == {}
+
+    b = Test2(1, 2, c=3, d=4, e=5)
+    assert b.a == 1
+    assert b.b == 2
+    assert b.c == 3
+    assert b.options == {'d': 4, 'e': 5}
+
+    msg = "only one variadic \\*\\*kwargs parameter is allowed"
+    with pytest.raises(TypeError, match=msg):
+
+        class Test3(Test):
+            another_options = varkwargs(is_int)
+
+
+def test_variadic_argument():
+    class Test(Annotable):
+        a = is_int
+        b = is_int
+        args = varargs(is_int)
+
+    assert Test(1, 2).args == ()
+    assert Test(1, 2, 3).args == (3,)
+    assert Test(1, 2, 3, 4, 5).args == (3, 4, 5)
+
+
+def test_variadic_keyword_argument():
+    class Test(Annotable):
+        first = is_int
+        second = is_int
+        options = varkwargs(is_int)
+
+    assert Test(1, 2).options == {}
+    assert Test(1, 2, a=3).options == {'a': 3}
+    assert Test(1, 2, a=3, b=4, c=5).options == {'a': 3, 'b': 4, 'c': 5}
+
+
+def test_concrete_copy_with_variadic_argument():
+    class Test(Annotable):
+        a = is_int
+        b = is_int
+        args = varargs(is_int)
+
+    t = Test(1, 2, 3, 4, 5)
+    assert t.a == 1
+    assert t.b == 2
+    assert t.args == (3, 4, 5)
+
+    u = t.copy(a=6, args=(8, 9, 10))
+    assert u.a == 6
+    assert u.b == 2
+    assert u.args == (8, 9, 10)
+
+
+def test_concrete_pickling_variadic_arguments():
+    v = VariadicArgs(1, 2, 3, 4, 5)
+    assert v.args == (1, 2, 3, 4, 5)
+    assert_pickle_roundtrip(v)
+
+    v = VariadicKeywords(a=3, b=4, c=5)
+    assert v.kwargs == {'a': 3, 'b': 4, 'c': 5}
+    assert_pickle_roundtrip(v)
+
+    v = VariadicArgsAndKeywords(1, 2, 3, 4, 5, a=3, b=4, c=5)
+    assert v.args == (1, 2, 3, 4, 5)
+    assert v.kwargs == {'a': 3, 'b': 4, 'c': 5}
+    assert_pickle_roundtrip(v)
+
+
 def test_dont_copy_default_argument():
     default = tuple()
 
@@ -426,10 +692,7 @@ def test_multiple_inheritance_argument_order():
     class Sum(VersionedOp, Reduction):
         where = optional(is_bool, default=False)
 
-    assert (
-        str(Sum.__signature__)
-        == "(arg: instance_of(<class 'object'>,), version: instance_of(<class 'int'>,), where: option(instance_of(<class 'bool'>,),default=False) = None)"
-    )
+    assert tuple(Sum.__signature__.parameters.keys()) == ("arg", "version", "where")
 
 
 def test_multiple_inheritance_optional_argument_order():
@@ -444,9 +707,11 @@ def test_multiple_inheritance_optional_argument_order():
         max = is_int
         how = optional(is_str, default="strict")
 
-    assert (
-        str(Between.__signature__)
-        == "(min: instance_of(<class 'int'>,), max: instance_of(<class 'int'>,), how: option(instance_of(<class 'str'>,),default='strict') = None, where: option(instance_of(<class 'bool'>,),default=False) = None)"
+    assert tuple(Between.__signature__.parameters.keys()) == (
+        "min",
+        "max",
+        "how",
+        "where",
     )
 
 
@@ -470,6 +735,14 @@ class Value2(Value):
         return 3
 
 
+class Value3(Value):
+    k = attribute(is_int, default=3)
+
+
+class Value4(Value):
+    k = attribute(option(is_int), default=None)
+
+
 # TODO(kszucs): add a test case with __dict__ added to __slots__
 
 
@@ -498,6 +771,12 @@ def test_annotable_attribute_init():
     assert v.j == 2
     assert v.k == 3
 
+    v = Value3(1)
+    assert v.k == 3
+
+    v = Value4(1)
+    assert v.k is None
+
 
 def test_annotable_mutability_and_serialization():
     v_ = Value(1)
@@ -545,7 +824,7 @@ def test_initialized_attribute_mixed_with_classvar():
     class Reduction(Value):
         output_shape = "scalar"
 
-    class variadic(Value):
+    class Variadic(Value):
         @attribute.default
         def output_shape(self):
             if self.arg > 10:
@@ -557,11 +836,11 @@ def test_initialized_attribute_mixed_with_classvar():
     assert r.output_shape == "scalar"
     assert "output_shape" not in r.__slots__
 
-    v = variadic(1)
+    v = Variadic(1)
     assert v.output_shape == "scalar"
     assert "output_shape" in v.__slots__
 
-    v = variadic(100)
+    v = Variadic(100)
     assert v.output_shape == "columnar"
     assert "output_shape" in v.__slots__
 
