@@ -2,17 +2,17 @@ from __future__ import annotations
 
 import functools
 import operator
-from collections import Counter
-from typing import Iterator, Mapping
+from collections import defaultdict
+from typing import Iterable, Iterator, Mapping
 
 import toolz
 
 import ibis.common.graph as g
 import ibis.expr.operations as ops
+import ibis.expr.operations.relations as rels
 import ibis.expr.types as ir
 from ibis import util
 from ibis.common.exceptions import IbisTypeError, IntegrityError
-from ibis.expr.window import window
 
 # ---------------------------------------------------------------------
 # Some expression metaprogramming / graph transformations to support
@@ -141,16 +141,12 @@ def find_immediate_parent_tables(input_node, keep_input=True):
     >>> import ibis, toolz
     >>> t = ibis.table([('a', 'int64')], name='t')
     >>> expr = t.mutate(foo=t.a + 1)
-    >>> result = find_immediate_parent_tables(expr)
-    >>> len(result)
-    1
-    >>> result[0]
-    r0 := UnboundTable[t]
-      a int64
-    Selection[r0]
-      selections:
-        r0
-        foo: r0.a + 1
+    >>> result, = find_immediate_parent_tables(expr.op())
+    >>> result.equals(expr.op())
+    True
+    >>> result, = find_immediate_parent_tables(expr.op(), keep_input=False)
+    >>> result.equals(t.op())
+    True
     """
     assert all(isinstance(arg, ops.Node) for arg in util.promote_list(input_node))
 
@@ -160,6 +156,12 @@ def find_immediate_parent_tables(input_node, keep_input=True):
                 return g.halt, node
             else:
                 return g.proceed, None
+
+        # HACK: special case ops.Contains to only consider the needle's base
+        # table, since that's the only expression that matters for determining
+        # cardinality
+        elif isinstance(node, ops.Contains):
+            return [node.value], None
         else:
             return g.proceed, None
 
@@ -275,41 +277,20 @@ def apply_filter(op, predicates):
     # This will attempt predicate pushdown in the cases where we can do it
     # easily and safely, to make both cleaner SQL and fewer referential errors
     # for users
-    assert isinstance(op, ops.Node)
-
-    if isinstance(op, ops.Selection):
-        return _filter_selection(op, predicates)
-    elif isinstance(op, ops.Aggregation):
-        # Potential fusion opportunity
-        # GH1344: We can't sub in things with correlated subqueries
-        simplified_predicates = tuple(
-            # Originally this line tried substituting op.table in for expr, but
-            # that is too aggressive in the presence of filters that occur
-            # after aggregations.
-            #
-            # See https://github.com/ibis-project/ibis/pull/3341 for details
-            sub_for(predicate, {op.table: op})
-            if not is_reduction(predicate)
-            else predicate
-            for predicate in predicates
-        )
-
-        if shares_all_roots(simplified_predicates, op.table):
-            return ops.Aggregation(
-                op.table,
-                op.metrics,
-                by=op.by,
-                having=op.having,
-                predicates=op.predicates + simplified_predicates,
-                sort_keys=op.sort_keys,
-            )
-
     if not predicates:
         return op
-    return ops.Selection(op, [], predicates)
+
+    if isinstance(op, ops.Selection):
+        return pushdown_selection_filters(op, predicates)
+    elif isinstance(op, ops.Aggregation):
+        return pushdown_aggregation_filters(op, predicates)
+    else:
+        return ops.Selection(op, [], predicates)
 
 
-def _filter_selection(op, predicates):
+def pushdown_selection_filters(op, predicates):
+    default = ops.Selection(op, selections=[], predicates=predicates)
+
     # We can't push down filters on Unnest or Window because they
     # change the shape and potential values of the data.
     if any(
@@ -319,7 +300,7 @@ def _filter_selection(op, predicates):
         )
         for sel in op.selections
     ):
-        return ops.Selection(op, selections=[], predicates=predicates)
+        return default
 
     # if any of the filter predicates have the parent expression among
     # their roots, then pushdown (at least of that predicate) is not
@@ -345,124 +326,116 @@ def _filter_selection(op, predicates):
             for predicate in predicates
         )
     except IntegrityError:
-        pass
-    else:
-        if shares_all_roots(simplified_predicates, op.table):
-            return ops.Selection(
-                op.table,
-                selections=op.selections,
-                predicates=op.predicates + simplified_predicates,
-                sort_keys=op.sort_keys,
-            )
+        return default
 
-    can_pushdown = _can_pushdown(op, predicates)
+    if not shares_all_roots(simplified_predicates, op.table):
+        return default
 
-    if can_pushdown:
-        simplified_predicates = tuple(substitute_parents(x) for x in predicates)
-        fused_predicates = op.predicates + simplified_predicates
-        return ops.Selection(
+    # find spuriously simplified predicates
+    for predicate in simplified_predicates:
+        # find columns in the predicate
+        depends_on = predicate.find((ops.TableColumn, ops.Literal))
+        for projection in op.selections:
+            if not isinstance(projection, (ops.TableColumn, ops.Literal)):
+                # if the projection's table columns overlap with columns
+                # used in the predicate then we return immediately
+                #
+                # this means that we were too aggressive during simplification
+                # example: t.mutate(a=_.a + 1).filter(_.a > 1)
+                if projection.find((ops.TableColumn, ops.Literal)) & depends_on:
+                    return default
+
+    return ops.Selection(
+        op.table,
+        selections=op.selections,
+        predicates=op.predicates + simplified_predicates,
+        sort_keys=op.sort_keys,
+    )
+
+
+def pushdown_aggregation_filters(op, predicates):
+    # Potential fusion opportunity
+    # GH1344: We can't sub in things with correlated subqueries
+    simplified_predicates = tuple(
+        # Originally this line tried substituting op.table in for expr, but
+        # that is too aggressive in the presence of filters that occur
+        # after aggregations.
+        #
+        # See https://github.com/ibis-project/ibis/pull/3341 for details
+        sub_for(predicate, {op.table: op}) if not is_reduction(predicate) else predicate
+        for predicate in predicates
+    )
+
+    if shares_all_roots(simplified_predicates, op.table):
+        return ops.Aggregation(
             op.table,
-            selections=op.selections,
-            predicates=fused_predicates,
+            op.metrics,
+            by=op.by,
+            having=op.having,
+            predicates=op.predicates + simplified_predicates,
             sort_keys=op.sort_keys,
         )
     else:
-        return ops.Selection(op, selections=[], predicates=predicates)
+        return ops.Selection(op, [], predicates)
 
 
-def _can_pushdown(op, predicates):
-    # Per issues discussed in #173
-    #
-    # The only case in which pushdown is possible is that all table columns
-    # referenced must meet all of the following (not that onerous in practice)
-    # criteria
-    #
-    # 1) Is a table column, not any other kind of expression
-    # 2) Is unaliased. So, if you project t3.foo AS bar, then filter on bar,
-    #    this cannot be pushed down (until we implement alias rewriting if
-    #    necessary)
-    # 3) Appears in the selections in the projection (either is part of one of
-    #    the entire tables or a single column selection)
+# TODO(kszucs): use ibis.expr.analysis.substitute instead
+def propagate_down_window(func: ops.Value, frame: ops.WindowFrame):
+    import ibis.expr.operations as ops
 
-    for pred in predicates:
-        if isinstance(pred, ir.Expr):
-            pred = pred.op()
-        validator = _PushdownValidate(op, pred)
-        predicate_is_valid = validator.get_result()
-        if not predicate_is_valid:
-            return False
-    return True
+    clean_args = []
+    for arg in func.args:
+        if isinstance(arg, ops.Value) and not isinstance(func, ops.WindowFunction):
+            arg = propagate_down_window(arg, frame)
+            if isinstance(arg, ops.Analytic):
+                arg = ops.WindowFunction(arg, frame)
+        clean_args.append(arg)
 
-
-# TODO(kszucs): rewrite to only work with operation objects
-class _PushdownValidate:
-    def __init__(self, parent, predicate):
-        self.parent = parent
-        self.pred = predicate
-
-    def get_result(self):
-        assert isinstance(self.pred, ops.Node), type(self.pred)
-
-        def validate(node):
-            if isinstance(node, ops.TableColumn):
-                return g.proceed, self._validate_projection(node)
-            return g.proceed, None
-
-        return all(g.traverse(validate, self.pred))
-
-    def _validate_projection(self, node):
-        is_valid = False
-
-        for val in self.parent.selections:
-            if isinstance(val, ops.PhysicalTable) and node.name in val.schema:
-                is_valid = True
-            elif isinstance(val, ops.TableColumn) and node.name == val.name:
-                # Aliased table columns are no good
-                is_valid = val.table == node.table
-
-        return is_valid
+    return type(func)(*clean_args)
 
 
 # TODO(kszucs): rewrite to receive and return an ops.Node
-def windowize_function(expr, w=None):
+def windowize_function(expr, frame):
     assert isinstance(expr, ir.Expr), type(expr)
+    assert isinstance(frame, ops.WindowFrame)
 
-    def _windowize(op, w):
-        if isinstance(op, ops.Window):
-            window_arg, window_w = op.args
-            walked_child = _walk(window_arg, w)
-            walked = ops.Window(walked_child, window_w)
+    def _windowize(op, frame):
+        if isinstance(op, ops.WindowFunction):
+            walked_child = _walk(op.func, frame)
+            walked = walked_child.to_expr().over(op.frame).op()
         elif isinstance(op, ops.Value):
-            walked = _walk(op, w)
+            walked = _walk(op, frame)
         else:
             walked = op
 
         if isinstance(walked, (ops.Analytic, ops.Reduction)):
-            if w is None:
-                w = window()
-            return walked.to_expr().over(w).op()
-        elif isinstance(walked, ops.Window):
-            if w is not None:
-                return walked.to_expr().over(w.combine(walked.window)).op()
+            return op.to_expr().over(frame).op()
+        elif isinstance(walked, ops.WindowFunction):
+            if frame is not None:
+                frame = walked.frame.copy(
+                    group_by=frame.group_by + walked.frame.group_by,
+                    order_by=frame.order_by + walked.frame.order_by,
+                )
+                return walked.to_expr().over(frame).op()
             else:
                 return walked
         else:
             return walked
 
-    def _walk(op, w):
+    def _walk(op, frame):
         # TODO(kszucs): rewrite to use the substitute utility
         windowed_args = []
         for arg in op.args:
             if isinstance(arg, ops.Value):
-                arg = _windowize(arg, w)
+                arg = _windowize(arg, frame)
             elif isinstance(arg, tuple):
-                arg = tuple(_windowize(x, w) for x in arg)
+                arg = tuple(_windowize(x, frame) for x in arg)
 
             windowed_args.append(arg)
 
         return type(op)(*windowed_args)
 
-    return _windowize(expr.op(), w).to_expr()
+    return _windowize(expr.op(), frame).to_expr()
 
 
 def simplify_aggregation(agg):
@@ -521,7 +494,12 @@ class Projector:
         self.parent = parent
         self.input_exprs = proj_exprs
         self.resolved_exprs = [parent._ensure_expr(e) for e in proj_exprs]
-        self.clean_exprs = list(map(windowize_function, self.resolved_exprs))
+
+        default_frame = ops.RowsWindowFrame(table=parent)
+        self.clean_exprs = [
+            windowize_function(expr, frame=default_frame)
+            for expr in self.resolved_exprs
+        ]
 
     def get_result(self):
         roots = find_immediate_parent_tables(self.parent.op())
@@ -625,6 +603,8 @@ def _find_projections(node):
         return g.proceed, None
     elif isinstance(node, ops.TableNode):
         return g.halt, node
+    elif isinstance(node, ops.Contains):
+        return [node.value], None
     else:
         return g.proceed, None
 
@@ -651,19 +631,19 @@ def flatten_predicate(node):
     >>> import ibis
     >>> t = ibis.table([('a', 'int64'), ('b', 'string')], name='t')
     >>> filt = (t.a == 1) & (t.b == 'foo')
-    >>> predicates = flatten_predicate(filt)
+    >>> predicates = flatten_predicate(filt.op())
     >>> len(predicates)
     2
-    >>> predicates[0]
-    r0 := UnboundTable[t]
+    >>> predicates[0].to_expr().name("left")
+    r0 := UnboundTable: t
       a int64
       b string
-    r0.a == 1
-    >>> predicates[1]
-    r0 := UnboundTable[t]
+    left: r0.a == 1
+    >>> predicates[1].to_expr().name("right")
+    r0 := UnboundTable: t
       a int64
       b string
-    r0.b == 'foo'
+    right: r0.b == 'foo'
     """
 
     def predicate(node):
@@ -743,32 +723,21 @@ def find_predicates(node, flatten=True):
     return list(g.traverse(predicate, node))
 
 
-def find_subqueries(node: ops.Node) -> Counter:
-    counts = Counter()
+def find_subqueries(node: ops.Node, min_dependents=1) -> tuple[ops.Node, ...]:
+    subquery_dependents = defaultdict(set)
+    for n in filter(None, util.promote_list(node)):
+        dependents = g.Graph.from_dfs(n).invert()
+        for u, vs in dependents.toposort().items():
+            # count the number of table-node dependents on the current node
+            # but only if the current node is a selection or aggregation
+            if isinstance(u, (rels.Projection, rels.Aggregation, rels.Limit)):
+                subquery_dependents[u].update(vs)
 
-    def finder(node: ops.Node):
-        if isinstance(node, ops.Join):
-            return [node.left, node.right], None
-        elif isinstance(node, ops.PhysicalTable):
-            return g.halt, None
-        elif isinstance(node, ops.SelfReference):
-            return g.proceed, None
-        elif isinstance(node, (ops.Selection, ops.Aggregation)):
-            counts[node] += 1
-            return [node.table], None
-        elif isinstance(node, ops.TableNode):
-            counts[node] += 1
-            return g.proceed, None
-        elif isinstance(node, ops.TableColumn):
-            return node.table not in counts, None
-        else:
-            return g.proceed, None
-
-    # keep duplicates so we can determine where an expression is used
-    # more than once
-    list(g.traverse(finder, node, dedup=False))
-
-    return counts
+    return tuple(
+        node
+        for node, dependents in reversed(subquery_dependents.items())
+        if len(dependents) >= min_dependents
+    )
 
 
 # TODO(kszucs): move to types/logical.py
@@ -817,7 +786,7 @@ def _rewrite_filter_reduction(op, name: str | None = None, **kwargs):
 @_rewrite_filter.register(ops.Literal)
 @_rewrite_filter.register(ops.ExistsSubquery)
 @_rewrite_filter.register(ops.NotExistsSubquery)
-@_rewrite_filter.register(ops.Window)
+@_rewrite_filter.register(ops.WindowFunction)
 def _rewrite_filter_subqueries(op, **kwargs):
     """Don't rewrite any of these operations in filters."""
     return op
@@ -867,3 +836,13 @@ def find_memtables(node: ops.Node) -> Iterator[ops.InMemoryTable]:
         return g.proceed, node if isinstance(node, ops.InMemoryTable) else None
 
     return g.traverse(finder, node, filter=ops.Node)
+
+
+def find_toplevel_unnest_children(nodes: Iterable[ops.Node]) -> Iterator[ops.Table]:
+    def finder(node):
+        return (
+            isinstance(node, ops.Value),
+            find_first_base_table(node) if isinstance(node, ops.Unnest) else None,
+        )
+
+    return g.traverse(finder, nodes, filter=ops.Node)
