@@ -13,6 +13,7 @@ import ibis.expr.schema as sch
 import ibis.expr.types as ir
 from ibis import util
 from ibis.common.annotations import attribute
+from ibis.common.collections import frozendict
 from ibis.expr.deferred import Deferred
 from ibis.expr.operations.core import Named, Node, Value
 from ibis.expr.operations.generic import TableColumn
@@ -34,7 +35,7 @@ class TableNode(Node):
     @property
     @abstractmethod
     def schema(self) -> sch.Schema:
-        """Return a schema."""
+        ...
 
     def to_expr(self):
         import ibis.expr.types as ir
@@ -49,7 +50,7 @@ class PhysicalTable(TableNode, Named):
 
 @public
 class UnboundTable(PhysicalTable):
-    schema = rlz.instance_of(sch.Schema)
+    schema = rlz.coerced_to(sch.Schema)
     name = rlz.optional(rlz.instance_of(str), default=genname)
 
 
@@ -59,9 +60,6 @@ class DatabaseTable(PhysicalTable):
     schema = rlz.instance_of(sch.Schema)
     source = rlz.client
 
-    def change_name(self, new_name):
-        return type(self)(new_name, self.args[1], self.source)
-
 
 @public
 class SQLQueryResult(TableNode):
@@ -82,12 +80,6 @@ class InMemoryTable(PhysicalTable):
     def data(self) -> util.ToFrame:
         """Return the data of an in-memory table."""
 
-    def has_resolved_name(self):
-        return True
-
-    def resolve_name(self):
-        return self.name
-
 
 # TODO(kszucs): desperately need to clean this up, the majority of this
 # functionality should be handled by input rules for the Join class
@@ -179,8 +171,13 @@ class Join(TableNode):
 
     @property
     def schema(self):
-        # For joins retaining both table schemas, merge them together here
-        return self.left.schema.append(self.right.schema)
+        # TODO(kszucs): use `return self.lefts.chema | self.right.schema` instead which
+        # eliminates unnecessary projection over the join, but currently breaks the
+        # pandas backend
+        left, right = self.left.schema, self.right.schema
+        if duplicates := left.keys() & right.keys():
+            raise com.IntegrityError(f'Duplicate column name(s): {duplicates}')
+        return sch.Schema({**left, **right})
 
 
 @public
@@ -250,8 +247,12 @@ class SetOp(TableNode):
     distinct = rlz.optional(rlz.instance_of(bool), default=False)
 
     def __init__(self, left, right, **kwargs):
-        if not left.schema == right.schema:
+        if left.schema != right.schema:
             raise com.RelationError('Table schemas must be equal for set operations')
+        elif left.schema.names != right.schema.names:
+            # rewrite so that both sides have the columns in the same order making it
+            # easier for the backends to implement set operations
+            right = ops.Selection(right, left.schema.names)
         super().__init__(left=left, right=right, **kwargs)
 
     @property
@@ -310,13 +311,10 @@ class Projection(TableNode):
     @attribute.default
     def schema(self):
         # Resolve schema and initialize
-
         if not self.selections:
             return self.table.schema
 
-        types = []
-        names = []
-
+        types, names = [], []
         for projection in self.selections:
             if isinstance(projection, Value):
                 names.append(projection.name)
@@ -326,7 +324,7 @@ class Projection(TableNode):
                 names.extend(schema.names)
                 types.extend(schema.types)
 
-        return sch.Schema(names, types)
+        return sch.schema(names, types)
 
 
 @public
@@ -390,7 +388,7 @@ class DummyTable(TableNode):
 
     @property
     def schema(self):
-        return sch.Schema.from_dict({op.name: op.output_dtype for op in self.values})
+        return sch.Schema({op.name: op.output_dtype for op in self.values})
 
 
 @public
@@ -439,7 +437,9 @@ class Aggregation(TableNode):
         default=(),
     )
     predicates = rlz.optional(rlz.tuple_of(rlz.boolean), default=())
-    sort_keys = rlz.optional(rlz.tuple_of(rlz.sort_key_from("table")), default=())
+    sort_keys = rlz.optional(
+        rlz.tuple_of(rlz.sort_key_from(rlz.ref("table"))), default=()
+    )
 
     def __init__(self, table, metrics, by, having, predicates, sort_keys):
         from ibis.expr.analysis import shares_all_roots, shares_some_roots
@@ -469,14 +469,11 @@ class Aggregation(TableNode):
 
     @attribute.default
     def schema(self):
-        names = []
-        types = []
-
-        for e in self.by + self.metrics:
-            names.append(e.name)
-            types.append(e.output_dtype)
-
-        return sch.Schema(names, types)
+        names, types = [], []
+        for value in self.by + self.metrics:
+            names.append(value.name)
+            types.append(value.output_dtype)
+        return sch.schema(names, types)
 
     def order_by(self, sort_exprs):
         from ibis.expr.analysis import shares_all_roots, sub_immediate_parents
@@ -538,7 +535,7 @@ class FillNa(TableNode):
             replacements=(
                 replacements
                 if not isinstance(replacements, collections.abc.Mapping)
-                else util.frozendict(replacements)
+                else frozendict(replacements)
             ),
             **kwargs,
         )
