@@ -3,18 +3,19 @@ from __future__ import annotations
 from typing import TYPE_CHECKING, Any, Iterable, Literal, Sequence
 
 from public import public
-from rich.jupyter import JupyterMixin
 
 import ibis
+from ibis import util
 import ibis.common.exceptions as com
 import ibis.expr.datatypes as dt
 import ibis.expr.operations as ops
 from ibis.common.grounds import Singleton
-from ibis.expr.types.core import Expr, _binop
+from ibis.expr.types.core import Expr, _binop, _FixedTextJupyterMixin
 
 if TYPE_CHECKING:
+    import pandas as pd
+
     import ibis.expr.types as ir
-    import ibis.expr.window as win
 
 
 @public
@@ -37,9 +38,9 @@ class Value(Expr):
         Examples
         --------
         >>> import ibis
-        >>> t = ibis.table(dict(a="int64"))
+        >>> t = ibis.table(dict(a="int64"), name="t")
         >>> t.a.name("b")
-        r0 := UnboundTable[unbound_table_...]
+        r0 := UnboundTable: t
           a int64
         b: r0.a
         """
@@ -58,7 +59,8 @@ class Value(Expr):
         return op.to_expr()
 
     # TODO(kszucs): should rename to dtype
-    def type(self):
+    def type(self) -> dt.DataType:
+        """Return the [DataType] of this expression."""
         return self.op().output_dtype
 
     def hash(self, how: str = "fnv") -> ir.IntegerValue:
@@ -95,7 +97,7 @@ class Value(Expr):
             # noop case if passed type is the same
             return self
 
-        if op.to.is_geography() or op.to.is_geometry():
+        if op.to.is_geospatial():
             from_geotype = self.type().geotype or 'geometry'
             to_geotype = op.to.geotype
             if from_geotype == to_geotype:
@@ -119,8 +121,8 @@ class Value(Expr):
         Examples
         --------
         >>> import ibis
-        >>> ibis.coalesce(None, 4, 5)
-        Coalesce((None, 4, 5))
+        >>> ibis.coalesce(None, 4, 5).name("x")
+        x: Coalesce(...)
         """
         return ops.Coalesce((self, *args)).to_expr()
 
@@ -177,17 +179,44 @@ class Value(Expr):
         Examples
         --------
         >>> import ibis
-        >>> table = ibis.table(dict(col='int64', other_col='int64'))
-        >>> result = table.col.fillna(5)
-        r0 := UnboundTable: unbound_table_0
-          col       int64
-          other_col int64
-        IfNull(r0.col, ifnull_expr=5)
-        >>> table.col.fillna(table.other_col * 3)
-        r0 := UnboundTable: unbound_table_0
-          col       int64
-          other_col int64
-        IfNull(r0.col, ifnull_expr=r0.other_col * 3)
+        >>> ibis.options.interactive = True
+        >>> t = ibis.examples.penguins.fetch()
+        >>> t.sex
+        ┏━━━━━━━━┓
+        ┃ sex    ┃
+        ┡━━━━━━━━┩
+        │ string │
+        ├────────┤
+        │ male   │
+        │ female │
+        │ female │
+        │ ∅      │
+        │ female │
+        │ male   │
+        │ female │
+        │ male   │
+        │ ∅      │
+        │ ∅      │
+        │ …      │
+        └────────┘
+        >>> t.sex.fillna("unrecorded").name("sex")
+        ┏━━━━━━━━━━━━┓
+        ┃ sex        ┃
+        ┡━━━━━━━━━━━━┩
+        │ string     │
+        ├────────────┤
+        │ male       │
+        │ female     │
+        │ female     │
+        │ unrecorded │
+        │ female     │
+        │ male       │
+        │ female     │
+        │ male       │
+        │ unrecorded │
+        │ unrecorded │
+        │ …          │
+        └────────────┘
 
         Returns
         -------
@@ -253,21 +282,21 @@ class Value(Expr):
         Check whether a column's values are contained in a sequence
 
         >>> import ibis
-        >>> table = ibis.table(dict(string_col='string'))
+        >>> table = ibis.table(dict(string_col='string'), name="t")
         >>> table.string_col.isin(['foo', 'bar', 'baz'])
-        r0 := UnboundTable: unbound_table_1
+        r0 := UnboundTable: t
           string_col string
-        Contains(value=r0.string_col, options=('foo', 'bar', 'baz'))
+        Contains(string_col): Contains(...)
 
         Check whether a column's values are contained in another table's column
 
-        >>> table2 = ibis.table(dict(other_string_col='string'))
+        >>> table2 = ibis.table(dict(other_string_col='string'), name="t2")
         >>> table.string_col.isin(table2.other_string_col)
-        r0 := UnboundTable: unbound_table_3
-          other_string_col string
-        r1 := UnboundTable: unbound_table_1
+        r0 := UnboundTable: t
           string_col string
-        Contains(value=r1.string_col, options=r0.other_string_col)
+        r1 := UnboundTable: t2
+          other_string_col string
+        Contains(string_col, other_string_col): Contains(...)
         """
         return ops.Contains(self, values).to_expr()
 
@@ -288,21 +317,24 @@ class Value(Expr):
 
     def substitute(
         self,
-        value: Value,
+        value: Value | dict,
         replacement: Value | None = None,
         else_: Value | None = None,
     ):
-        """Replace one or more values in a value expression.
+        """Replace values given in `values` with `replacement`.
+
+        This is similar to the pandas `replace` method.
 
         Parameters
         ----------
         value
-            Expression or mapping
+            Expression or dict.
         replacement
-            Expression. If an expression is passed to value, this must be
+            If an expression is passed to value, this must be
             passed.
         else_
-            Expression
+            If an original value does not match `value`, then `else_` is used.
+            The default of `None` means leave the original value unchanged.
 
         Returns
         -------
@@ -318,31 +350,63 @@ class Value(Expr):
 
         return expr.else_(else_ if else_ is not None else self).end()
 
-    def over(self, window: win.Window) -> Value:
+    def over(
+        self,
+        window=None,
+        *,
+        rows=None,
+        range=None,
+        group_by=None,
+        order_by=None,
+    ) -> Value:
         """Construct a window expression.
 
         Parameters
         ----------
         window
             Window specification
+        rows
+            Whether to use the `ROWS` window clause
+        range
+            Whether to use the `RANGE` window clause
+        group_by
+            Grouping key
+        order_by
+            Ordering key
 
         Returns
         -------
         Value
             A window function expression
         """
-        prior_op = self.op()
+        import ibis.expr.analysis as an
+        import ibis.expr.builders as bl
+        import ibis.expr.deferred as de
 
-        # TODO(kszucs): fix this ugly hack
-        if isinstance(prior_op, ops.Alias):
-            return prior_op.arg.to_expr().over(window).name(prior_op.name)
+        if window is None:
+            window = ibis.window(
+                rows=rows,
+                range=range,
+                group_by=group_by,
+                order_by=order_by,
+            )
 
-        if isinstance(prior_op, ops.Window):
-            op = prior_op.over(window)
+        def bind(table):
+            frame = window.bind(table)
+            return ops.WindowFunction(self, frame).to_expr()
+
+        op = self.op()
+        if isinstance(op, ops.Alias):
+            return op.arg.to_expr().over(window).name(op.name)
+        elif isinstance(op, ops.WindowFunction):
+            return op.func.to_expr().over(window)
+        elif isinstance(window, bl.WindowBuilder):
+            if table := an.find_first_base_table(self.op()):
+                return bind(table)
+            else:
+                return de.Deferred(bind)
         else:
-            op = ops.Window(self, window)
-
-        return op.to_expr()
+            return ops.WindowFunction(self, window).to_expr()
 
     def isnull(self) -> ir.BooleanValue:
         """Return whether this expression is NULL."""
@@ -374,9 +438,9 @@ class Value(Expr):
         ...              .else_('null or (not a and not b)')
         ...              .end())
         >>> case_expr
-        r0 := UnboundTable[t]
+        r0 := UnboundTable: t
           string_col string
-        SimpleCase(base=r0.string_col, cases=[List(values=['a', 'b'])], results=[List(values=['an a', 'a b'])], default='null or (not a and not b)')
+        SimpleCase(...)
         """
         import ibis.expr.builders as bl
 
@@ -406,8 +470,69 @@ class Value(Expr):
             builder = builder.when(case, result)
         return builder.else_(default).end()
 
-    def collect(self, where: ir.BooleanValue | None = None) -> ir.ArrayValue:
-        """Return an array of the elements of this expression."""
+    def collect(self, where: ir.BooleanValue | None = None) -> ir.ArrayScalar:
+        """Aggregate this expression's elements into an array.
+
+        This function is called `array_agg`, `list_agg`, or `list` in other systems.
+
+        Parameters
+        ----------
+        where
+            Filter to apply before aggregation
+
+        Returns
+        -------
+        ArrayScalar
+            Collected array
+
+        Examples
+        --------
+        Basic collect usage
+
+        >>> import ibis
+        >>> ibis.options.interactive = True
+        >>> t = ibis.memtable({"key": list("aaabb"), "value": [1, 2, 3, 4, 5]})
+        >>> t
+        ┏━━━━━━━━┳━━━━━━━┓
+        ┃ key    ┃ value ┃
+        ┡━━━━━━━━╇━━━━━━━┩
+        │ string │ int64 │
+        ├────────┼───────┤
+        │ a      │     1 │
+        │ a      │     2 │
+        │ a      │     3 │
+        │ b      │     4 │
+        │ b      │     5 │
+        └────────┴───────┘
+        >>> t.value.collect()
+        [1, 2, 3, 4, 5]
+        >>> type(t.value.collect())
+        <class 'ibis.expr.types.arrays.ArrayScalar'>
+
+        Collect elements per group
+
+        >>> t.group_by("key").agg(v=lambda t: t.value.collect())
+        ┏━━━━━━━━┳━━━━━━━━━━━━━━━━━━━━━━┓
+        ┃ key    ┃ v                    ┃
+        ┡━━━━━━━━╇━━━━━━━━━━━━━━━━━━━━━━┩
+        │ string │ array<int64>         │
+        ├────────┼──────────────────────┤
+        │ a      │ [1, 2, ... +1]       │
+        │ b      │ [4, 5]               │
+        └────────┴──────────────────────┘
+
+        Collect elements per group using a filter
+
+        >>> t.group_by("key").agg(v=lambda t: t.value.collect(where=t.value > 1))
+        ┏━━━━━━━━┳━━━━━━━━━━━━━━━━━━━━━━┓
+        ┃ key    ┃ v                    ┃
+        ┡━━━━━━━━╇━━━━━━━━━━━━━━━━━━━━━━┩
+        │ string │ array<int64>         │
+        ├────────┼──────────────────────┤
+        │ a      │ [2, 3]               │
+        │ b      │ [4, 5]               │
+        └────────┴──────────────────────┘
+        """
         return ops.ArrayCollect(self, where=where).to_expr()
 
     def identical_to(self, other: Value) -> ir.BooleanValue:
@@ -480,8 +605,22 @@ class Value(Expr):
         """Sort an expression descending."""
         return ops.SortKey(self, ascending=False).to_expr()
 
-    def to_projection(self) -> ir.Table:
-        """Promote this value expression to a projection."""
+    def as_table(self) -> ir.Table:
+        """Promote the expression to a table.
+
+        Returns
+        -------
+        Table
+            A table expression
+
+        Examples
+        --------
+        >>> t = ibis.table(dict(a="str"), name="t")
+        >>> expr = t.a.length().name("len").as_table()
+        >>> expected = t.select(len=t.a.length())
+        >>> expr.equals(expected)
+        True
+        """
         from ibis.expr.analysis import find_immediate_parent_tables
 
         roots = find_immediate_parent_tables(self.op())
@@ -492,7 +631,17 @@ class Value(Expr):
                 'to a projection'
             )
         table = roots[0].to_expr()
-        return table.projection([self])
+        return table.select(self)
+
+    def to_pandas(self, **kwargs) -> pd.Series:
+        """Convert a column expression to a pandas Series or scalar object.
+
+        Parameters
+        ----------
+        kwargs
+            Same as keyword arguments to [`execute`][ibis.expr.types.core.Expr.execute]
+        """
+        return self.execute(**kwargs)
 
 
 @public
@@ -505,6 +654,31 @@ class Scalar(Value):
         return console.render(repr(self.execute()), options=options)
 
     def as_table(self) -> ir.Table:
+        """Promote the scalar expression to a table.
+
+        Returns
+        -------
+        Table
+            A table expression
+
+        Examples
+        --------
+        Promote an aggregation to a table
+
+        >>> import ibis
+        >>> import ibis.expr.types as ir
+        >>> t = ibis.table(dict(a="str"), name="t")
+        >>> expr = t.a.length().sum().name("len").as_table()
+        >>> isinstance(expr, ir.Table)
+        True
+
+        Promote a literal value to a table
+
+        >>> import ibis.expr.types as ir
+        >>> lit = ibis.literal(1).name("a").as_table()
+        >>> isinstance(lit, ir.Table)
+        True
+        """
         from ibis.expr.analysis import (
             find_first_base_table,
             is_scalar_reduction,
@@ -527,7 +701,7 @@ class Scalar(Value):
 
 
 @public
-class Column(Value, JupyterMixin):
+class Column(Value, _FixedTextJupyterMixin):
     # Higher than numpy & dask objects
     __array_priority__ = 20
 
@@ -536,9 +710,6 @@ class Column(Value, JupyterMixin):
     def __array__(self, dtype=None):
         return self.execute().__array__(dtype)
 
-    def as_table(self):
-        return self.to_projection()
-
     def __rich_console__(self, console, options):
         named = self.name(self.op().name)
         projection = named.as_table()
@@ -613,6 +784,18 @@ class Column(Value, JupyterMixin):
         return ops.ArgMin(self, key=key, where=where).to_expr()
 
     def nunique(self, where: ir.BooleanValue | None = None) -> ir.IntegerScalar:
+        """Compute the number of distinct rows in an expression.
+
+        Parameters
+        ----------
+        where
+            Filter expression
+
+        Returns
+        -------
+        IntegerScalar
+            Number of distinct elements in an expression
+        """
         return ops.CountDistinct(self, where).to_expr()
 
     def topk(
@@ -661,6 +844,11 @@ class Column(Value, JupyterMixin):
             .limit(k)
         )
 
+    @util.deprecated(
+        instead="Reach out at https://github.com/ibis-project/ibis if you'd like this API to remain.",
+        as_of="5.0",
+        removed_in="6.0",
+    )
     def summary(
         self,
         exact_nunique: bool = False,
@@ -741,53 +929,130 @@ class Column(Value, JupyterMixin):
         """
         return ops.Count(self, where).to_expr()
 
-    def value_counts(self, metric_name: str = "count") -> ir.Table:
+    def value_counts(self) -> ir.Table:
         """Compute a frequency table.
 
-        Parameters
-        ----------
-        metric_name
-            Output column name of the `count()` metric
-
         Returns
         -------
         Table
             Frequency table expression
+
+        Examples
+        --------
+        >>> import ibis
+        >>> ibis.options.interactive = True
+        >>> t = ibis.memtable({"chars": char} for char in "aabcddd")
+        >>> t
+        ┏━━━━━━━━┓
+        ┃ chars  ┃
+        ┡━━━━━━━━┩
+        │ string │
+        ├────────┤
+        │ a      │
+        │ a      │
+        │ b      │
+        │ c      │
+        │ d      │
+        │ d      │
+        │ d      │
+        └────────┘
+        >>> t.chars.value_counts()
+        ┏━━━━━━━━┳━━━━━━━━━━━━━┓
+        ┃ chars  ┃ chars_count ┃
+        ┡━━━━━━━━╇━━━━━━━━━━━━━┩
+        │ string │ int64       │
+        ├────────┼─────────────┤
+        │ a      │           2 │
+        │ b      │           1 │
+        │ c      │           1 │
+        │ d      │           3 │
+        └────────┴─────────────┘
         """
         from ibis.expr.analysis import find_first_base_table
 
+        name = self.get_name()
         return (
             find_first_base_table(self.op())
             .to_expr()
             .select(self)
-            .group_by(self.get_name())
-            .agg(**{metric_name: lambda t: t.count()})
+            .group_by(name)
+            .agg(**{f"{name}_count": lambda t: t.count()})
         )
 
     def first(self) -> Column:
+        """Return the first value of a column.
+
+        Equivalent to SQL's `FIRST_VALUE` window function.
+        """
         return ops.FirstValue(self).to_expr()
 
     def last(self) -> Column:
+        """Return the last value of a column.
+
+        Equivalent to SQL's `LAST_VALUE` window function.
+        """
         return ops.LastValue(self).to_expr()
 
-    def rank(self) -> Column:
+    def rank(self) -> ir.IntegerColumn:
+        """Compute position of first element within each equal-value group in sorted order.
+
+        Equivalent to SQL's `RANK()` window function.
+
+        Examples
+        --------
+        values   ranks
+        1        0
+        1        0
+        2        2
+        2        2
+        2        2
+        3        5
+
+        Returns
+        -------
+        Int64Column
+            The min rank
+        """
         return ops.MinRank(self).to_expr()
 
-    def dense_rank(self) -> Column:
+    def dense_rank(self) -> ir.IntegerColumn:
+        """Position of first element within each group of equal values.
+
+        Values are returned in sorted order and duplicate values are ignored.
+
+        Equivalent to SQL's `DENSE_RANK()`.
+
+        Examples
+        --------
+        values   ranks
+        1        0
+        1        0
+        2        1
+        2        1
+        2        1
+        3        2
+
+        Returns
+        -------
+        IntegerColumn
+            The rank
+        """
         return ops.DenseRank(self).to_expr()
 
     def percent_rank(self) -> Column:
+        """Return the relative rank of the values in the column."""
         return ops.PercentRank(self).to_expr()
 
     def cume_dist(self) -> Column:
-        import ibis.expr.operations as ops
-
+        """Return the cumulative distribution over a window."""
         return ops.CumeDist(self).to_expr()
 
     def cummin(self) -> Column:
+        """Return the cumulative min over a window."""
         return ops.CumulativeMin(self).to_expr()
 
     def cummax(self) -> Column:
+        """Return the cumulative max over a window."""
         return ops.CumulativeMax(self).to_expr()
 
     def lag(
@@ -795,6 +1060,15 @@ class Column(Value, JupyterMixin):
         offset: int | ir.IntegerValue | None = None,
         default: Value | None = None,
     ) -> Column:
+        """Return the row located at `offset` rows **before** the current row.
+
+        Parameters
+        ----------
+        offset
+            Index of row to select
+        default
+            Value used if no row exists at `offset`
+        """
         return ops.Lag(self, offset, default).to_expr()
 
     def lead(
@@ -802,9 +1076,25 @@ class Column(Value, JupyterMixin):
         offset: int | ir.IntegerValue | None = None,
         default: Value | None = None,
     ) -> Column:
+        """Return the row located at `offset` rows **after** the current row.
+
+        Parameters
+        ----------
+        offset
+            Index of row to select
+        default
+            Value used if no row exists at `offset`
+        """
         return ops.Lead(self, offset, default).to_expr()
 
     def ntile(self, buckets: int | ir.IntegerValue) -> ir.IntegerColumn:
+        """Return the integer number of a partitioning of the column values.
+
+        Parameters
+        ----------
+        buckets
+            Number of buckets to partition into
+        """
         return ops.NTile(self, buckets).to_expr()
 
     def nth(self, n: int | ir.IntegerValue) -> Column:
