@@ -1,11 +1,13 @@
 from __future__ import annotations
 
 import collections
-from typing import TYPE_CHECKING, Literal, Sequence
+import functools
+from typing import TYPE_CHECKING, Iterable, Literal, Sequence
 
 from public import public
 
 import ibis.expr.operations as ops
+from ibis import util
 from ibis.common.exceptions import IbisTypeError
 from ibis.expr.types.core import _binop
 from ibis.expr.types.generic import Column, Scalar, Value
@@ -18,6 +20,7 @@ if TYPE_CHECKING:
 class NumericValue(Value):
     @staticmethod
     def __negate_op__():
+        # TODO(kszucs): do we need this?
         return ops.Negate
 
     def negate(self) -> NumericValue:
@@ -466,6 +469,7 @@ class NumericColumn(Column, NumericValue):
         return ops.Mean(self, where=where).to_expr()
 
     def cummean(self) -> NumericColumn:
+        """Return the cumulative mean of the input."""
         return ops.CumulativeMean(self).to_expr()
 
     def sum(
@@ -487,6 +491,7 @@ class NumericColumn(Column, NumericValue):
         return ops.Sum(self, where=where).to_expr()
 
     def cumsum(self) -> NumericColumn:
+        """Return the cumulative sum of the input."""
         return ops.CumulativeSum(self).to_expr()
 
     def bucket(
@@ -496,7 +501,7 @@ class NumericColumn(Column, NumericValue):
         close_extreme: bool = True,
         include_under: bool = False,
         include_over: bool = False,
-    ) -> ir.CategoryColumn:
+    ) -> ir.IntegerColumn:
         """Compute a discrete binning of a numeric array.
 
         Parameters
@@ -520,7 +525,7 @@ class NumericColumn(Column, NumericValue):
 
         Returns
         -------
-        CategoryColumn
+        IntegerColumn
             A categorical column expression
         """
         return ops.Bucket(
@@ -579,6 +584,11 @@ class NumericColumn(Column, NumericValue):
 
         return ((self - base) / binwidth).floor()
 
+    @util.deprecated(
+        instead="Reach out at https://github.com/ibis-project/ibis if you'd like this API to remain.",
+        as_of="5.0",
+        removed_in="6.0",
+    )
     def summary(
         self,
         exact_nunique: bool = False,
@@ -743,6 +753,47 @@ class IntegerValue(NumericValue):
         else:
             return node.to_expr()
 
+    def label(self, labels: Iterable[str], nulls: str | None = None) -> ir.StringValue:
+        """Label a set of integer values with strings.
+
+        Parameters
+        ----------
+        labels
+            An iterable of string labels. Each integer value in `self` will be mapped to
+            a value in `labels`.
+        nulls
+            String label to use for `NULL` values
+
+        Returns
+        -------
+        StringValue
+            `self` labeled with `labels`
+
+        Examples
+        --------
+        >>> import ibis
+        >>> ibis.options.interactive = True
+        >>> t = ibis.memtable({"a": [0, 1, 0, 2]})
+        >>> t.select(t.a, labeled=t.a.label(["a", "b", "c"]))
+        ┏━━━━━━━┳━━━━━━━━━┓
+        ┃ a     ┃ labeled ┃
+        ┡━━━━━━━╇━━━━━━━━━┩
+        │ int64 │ string  │
+        ├───────┼─────────┤
+        │     0 │ a       │
+        │     1 │ b       │
+        │     0 │ a       │
+        │     2 │ c       │
+        └───────┴─────────┘
+        """
+        return (
+            functools.reduce(
+                lambda stmt, inputs: stmt.when(*inputs), enumerate(labels), self.case()
+            )
+            .else_(nulls)
+            .end()
+        )
+
 
 @public
 class IntegerScalar(NumericScalar, IntegerValue):
