@@ -1,6 +1,7 @@
 import datetime
 import pickle
 import re
+from typing import List
 
 import numpy as np
 import pandas as pd
@@ -12,6 +13,8 @@ import ibis.common.exceptions as com
 import ibis.expr.analysis as an
 import ibis.expr.datatypes as dt
 import ibis.expr.operations as ops
+import ibis.expr.schema as sch
+import ibis.expr.selectors as s
 import ibis.expr.types as ir
 from ibis import _
 from ibis import literal as L
@@ -86,13 +89,21 @@ def test_getitem_column_select(table):
         assert isinstance(col, Column)
 
 
+def test_table_tab_completion():
+    table = ibis.table({"a": "int", "b": "int", "for": "int", "with spaces": "int"})
+    # Only valid python identifiers in getattr tab completion
+    attrs = set(dir(table))
+    assert {"a", "b"}.issubset(attrs)
+    assert {"for", "with spaces"}.isdisjoint(attrs)
+    # All columns in getitem tab completion
+    items = set(table._ipython_key_completions_())
+    assert items.issuperset(table.columns)
+
+
 def test_getitem_attribute(table):
     result = table.a
     assert_equal(result, table['a'])
 
-    assert 'a' in dir(table)
-    assert 'a' in table._ipython_key_completions_()
-
     # Project and add a name that conflicts with a Table built-in
     # attribute
     view = table[[table, table['a'].name('schema')]]
@@ -361,9 +372,9 @@ def test_filter_fusion_distinct_table_objects(con):
     assert_equal(expr, expr4)
 
 
-def test_column_relabel(table):
+def test_column_relabel():
     table = api.table({"x": "int32", "y": "string", "z": "double"})
-    sol = api.schema({"x_1": "int32", "y_1": "string", "z": "double"})
+    sol = sch.schema({"x_1": "int32", "y_1": "string", "z": "double"})
 
     # Using a mapping
     res = table.relabel({"x": "x_1", "y": "y_1"}).schema()
@@ -378,6 +389,23 @@ def test_column_relabel(table):
         table.relabel({"missing": "oops"})
 
 
+def test_relabel_snake_case():
+    cases = [
+        ("cola", "cola"),
+        ("ColB", "col_b"),
+        ("colC", "col_c"),
+        ("col-d", "col_d"),
+        ("col_e", "col_e"),
+        (" Column F ", "column_f"),
+        ("Column G-with-hyphens", "column_g_with_hyphens"),
+        ("Col H notCamelCase", "col_h_notcamelcase"),
+    ]
+    t = ibis.table({c: "int" for c, _ in cases})
+    res = t.relabel("snake_case")
+    sol = t.relabel(dict(cases))
+    assert_equal(res, sol)
+
+
 def test_limit(table):
     limited = table.limit(10, offset=5)
     assert limited.op().n == 10
@@ -454,6 +482,36 @@ def test_order_by_scalar(table, key, expected):
     assert result.op().sort_keys == (ops.SortKey(expected),)
 
 
+@pytest.mark.parametrize(
+    "key",
+    [
+        "bogus",
+        ("bogus", False),
+        ibis.desc("bogus"),
+        1000,
+        (1000, False),
+        _.bogus,
+        _.bogus.desc(),
+    ],
+)
+@pytest.mark.parametrize(
+    "expr_func",
+    [
+        param(lambda t: t, id="table"),
+        param(lambda t: t.select("a", "b"), id="selection"),
+        param(lambda t: t.group_by("a").agg(new=_.b.sum()), id="aggregation"),
+    ],
+)
+def test_order_by_nonexistent_column_errors(table, expr_func, key):
+    # `order_by` is implemented on a few different operations, we check them
+    # all in turn here.
+    expr = expr_func(table)
+    # Depending on the expression, this can raise a few different errors.
+    # For now just check that an exception is raised.
+    with pytest.raises(Exception):
+        expr.order_by(key)
+
+
 def test_slice(table):
     expr = table[:5]
     expr2 = table[:5:1]
@@ -568,7 +626,8 @@ def test_groupby_alias(table):
 
 
 def test_summary_expand_list(table):
-    summ = table.f.summary()
+    with pytest.warns(FutureWarning, match="is deprecated"):
+        summ = table.f.summary()
 
     metric = table.g.group_concat().name('bar')
     result = table.aggregate([metric, summ])
@@ -580,40 +639,45 @@ def test_summary_prefix_suffix(table):
     def get_names(exprs):
         return [e.get_name() for e in exprs]
 
-    assert get_names(table.g.summary(prefix="string_")) == [
-        'string_count',
-        'string_nulls',
-        'string_uniques',
-    ]
-    assert get_names(table.g.summary(suffix="_string")) == [
-        'count_string',
-        'nulls_string',
-        'uniques_string',
-    ]
-    assert get_names(table.g.summary(prefix="pre_", suffix="_post")) == [
-        'pre_count_post',
-        'pre_nulls_post',
-        'pre_uniques_post',
-    ]
+    with pytest.warns(FutureWarning, match="is deprecated"):
+        assert get_names(table.g.summary(prefix="string_")) == [
+            'string_count',
+            'string_nulls',
+            'string_uniques',
+        ]
+    with pytest.warns(FutureWarning, match="is deprecated"):
+        assert get_names(table.g.summary(suffix="_string")) == [
+            'count_string',
+            'nulls_string',
+            'uniques_string',
+        ]
+    with pytest.warns(FutureWarning, match="is deprecated"):
+        assert get_names(table.g.summary(prefix="pre_", suffix="_post")) == [
+            'pre_count_post',
+            'pre_nulls_post',
+            'pre_uniques_post',
+        ]
 
-    assert get_names(table.f.summary(prefix="float_")) == [
-        "float_count",
-        "float_nulls",
-        "float_min",
-        "float_max",
-        "float_sum",
-        "float_mean",
-        "float_approx_nunique",
-    ]
-    assert get_names(table.f.summary(suffix="_numeric")) == [
-        "count_numeric",
-        "nulls_numeric",
-        "min_numeric",
-        "max_numeric",
-        "sum_numeric",
-        "mean_numeric",
-        "approx_nunique_numeric",
-    ]
+    with pytest.warns(FutureWarning, match="is deprecated"):
+        assert get_names(table.f.summary(prefix="float_")) == [
+            "float_count",
+            "float_nulls",
+            "float_min",
+            "float_max",
+            "float_sum",
+            "float_mean",
+            "float_approx_nunique",
+        ]
+    with pytest.warns(FutureWarning, match="is deprecated"):
+        assert get_names(table.f.summary(suffix="_numeric")) == [
+            "count_numeric",
+            "nulls_numeric",
+            "min_numeric",
+            "max_numeric",
+            "sum_numeric",
+            "mean_numeric",
+            "approx_nunique_numeric",
+        ]
 
 
 def test_filter_aggregate_pushdown_predicate(table):
@@ -725,7 +789,7 @@ def test_group_by_column_select_api(table):
 def test_value_counts_convenience(table):
     # #152
     result = table.g.value_counts()
-    expected = table.select('g').group_by('g').aggregate(count=lambda t: t.count())
+    expected = table.select('g').group_by('g').aggregate(g_count=lambda t: t.count())
 
     assert_equal(result, expected)
 
@@ -843,7 +907,7 @@ def test_equijoin_schema_merge():
     pred = table1['key1'] == table2['key2']
     join_types = ['inner_join', 'left_join', 'outer_join']
 
-    ex_schema = api.Schema(
+    ex_schema = sch.schema(
         ['key1', 'value1', 'key2', 'stuff'],
         ['string', 'double', 'string', 'int32'],
     )
@@ -907,7 +971,7 @@ def test_self_join(table):
 
     # Try aggregating on top of joined
     aggregated = joined.aggregate([metric], by=[left['g']])
-    ex_schema = api.Schema(['g', 'metric'], ['string', 'double'])
+    ex_schema = api.Schema({'g': 'string', 'metric': 'double'})
     assert_equal(aggregated.schema(), ex_schema)
 
 
@@ -980,8 +1044,9 @@ def test_cross_join(table):
     scalar_aggs = table.aggregate(metrics)
 
     joined = table.cross_join(scalar_aggs)
-    agg_schema = api.Schema(['sum_a', 'mean_b'], ['int64', 'double'])
-    ex_schema = table.schema().append(agg_schema)
+    agg_schema = api.Schema({'sum_a': 'int64', 'mean_b': 'double'})
+    with pytest.warns(FutureWarning):
+        ex_schema = table.schema().merge(agg_schema)
     assert_equal(joined.schema(), ex_schema)
 
 
@@ -1439,6 +1504,26 @@ def test_unbound_table_name():
     assert match is not None
 
 
+class MyTable:
+    a: int
+    b: str
+    c: List[float]
+
+
+def test_unbound_table_using_class_definition():
+    expected_schema = ibis.schema({'a': 'int64', 'b': 'string', 'c': 'array<double>'})
+
+    t1 = ibis.table(MyTable)
+    t2 = ibis.table(MyTable, name="MyNamedTable")
+
+    cases = {t1: "MyTable", t2: "MyNamedTable"}
+    for t, name in cases.items():
+        assert isinstance(t, ir.TableExpr)
+        assert isinstance(t.op(), ops.UnboundTable)
+        assert t.schema() == expected_schema
+        assert t.get_name() == name
+
+
 def test_mutate_chain():
     one = ibis.table([('a', 'string'), ('b', 'string')], name='t')
     two = one.mutate(b=lambda t: t.b.fillna('Short Term'))
@@ -1473,10 +1558,7 @@ def test_multiple_db_different_backends():
     backend2_table = con2.table('alltypes')
 
     expr = backend1_table.union(backend2_table)
-    with pytest.raises(
-        ValueError,
-        match=re.compile("multiple backends", flags=re.IGNORECASE),
-    ):
+    with pytest.raises(com.IbisError, match="Multiple backends"):
         expr.compile()
 
 
@@ -1560,13 +1642,13 @@ def test_drop():
     res = t.drop("a", "b")
     assert res.equals(t.select("c", "d"))
 
-    with pytest.raises(KeyError, match="Fields not in table"):
-        t.drop("missing")
-
-    with pytest.warns(FutureWarning, match="a sequence of fields"):
-        res = t.drop(["a", "b"])
     assert res.equals(t.select("c", "d"))
 
+    assert res.equals(t.drop(s.matches("a|b")))
+
+    with pytest.raises(KeyError):
+        t.drop("e")
+
 
 def test_python_table_ambiguous():
     with pytest.raises(NotImplementedError):
@@ -1626,3 +1708,104 @@ def test_filter_with_literal(value, api):
     int_val = ibis.literal(int(value))
     with pytest.raises((NotImplementedError, com.IbisTypeError)):
         api(t, int_val)
+
+
+def test_cast():
+    t = ibis.table(dict(a="int", b="string", c="float"), name="t")
+
+    assert t.cast({"a": "string"}).equals(t.mutate(a=t.a.cast("string")))
+
+    with pytest.raises(
+        com.IbisError, match="fields that are not in the table: .+'d'.+"
+    ):
+        t.cast({"d": "array<int>"}).equals(t.select())
+
+    assert t.cast(ibis.schema({"a": "string", "b": "int"})).equals(
+        t.mutate(a=t.a.cast("string"), b=t.b.cast("int"))
+    )
+    assert t.cast([("a", "string"), ("b", "float")]).equals(
+        t.mutate(a=t.a.cast("string"), b=t.b.cast("float"))
+    )
+
+
+def test_pivot_longer():
+    diamonds = ibis.table(
+        {
+            'carat': 'float64',
+            'cut': 'string',
+            'color': 'string',
+            'clarity': 'string',
+            'depth': 'float64',
+            'table': 'float64',
+            'price': 'int64',
+            'x': 'float64',
+            'y': 'float64',
+            'z': 'float64',
+        },
+        name="diamonds",
+    )
+    res = diamonds.pivot_longer(s.c("x", "y", "z"), names_to="pos", values_to="xyz")
+    assert res.schema().names == (
+        "carat",
+        "cut",
+        "color",
+        "clarity",
+        "depth",
+        "table",
+        "price",
+        "pos",
+        "xyz",
+    )
+
+
+def test_pivot_longer_strip_prefix():
+    t = ibis.table(
+        dict(artist="string", track="string", wk1="int", wk2="int", wk3="int")
+    )
+    expr = t.pivot_longer(
+        s.startswith("wk"),
+        names_to="week",
+        names_pattern=r"wk(.+)",
+        names_transform=int,
+        values_to="rank",
+        values_transform=_.cast("int"),
+    )
+    schema = ibis.schema(dict(artist="string", track="string", week="int8", rank="int"))
+    assert expr.schema() == schema
+
+
+def test_pivot_longer_pluck_regex():
+    t = ibis.table(
+        dict(artist="string", track="string", x_wk1="int", x_wk2="int", x_wk3="int")
+    )
+    expr = t.pivot_longer(
+        s.matches("^.+wk.$"),
+        names_to=["other_var", "week"],
+        names_pattern=r"(.)_wk(\d)",
+        names_transform=dict(other_var=str.upper, week=int),
+        values_to="rank",
+        values_transform=_.cast("int"),
+    )
+    schema = ibis.schema(
+        dict(
+            artist="string", track="string", other_var="string", week="int8", rank="int"
+        )
+    )
+    assert expr.schema() == schema
+
+
+def test_pivot_longer_no_match():
+    t = ibis.table(
+        dict(artist="string", track="string", x_wk1="int", x_wk2="int", x_wk3="int")
+    )
+    with pytest.raises(
+        com.IbisInputError, match="Selector returned no columns to pivot on"
+    ):
+        t.pivot_longer(
+            s.matches("foo"),
+            names_to=["other_var", "week"],
+            names_pattern=r"(.)_wk(\d)",
+            names_transform=dict(other_var=str.upper, week=int),
+            values_to="rank",
+            values_transform=_.cast("int"),
+        )
