@@ -20,6 +20,7 @@ import ibis.expr.operations as ops
 import ibis.expr.rules as rlz
 import ibis.expr.types as ir
 from ibis import _, literal
+from ibis.common.collections import frozendict
 from ibis.common.exceptions import IbisTypeError
 from ibis.expr import api
 from ibis.tests.util import assert_equal
@@ -39,6 +40,12 @@ def test_null():
     assert expr2.type().equals(dt.null)
 
 
+def test_literal_mixed_type_fails():
+    data = [1, 'a']
+    with pytest.raises(TypeError):
+        ibis.literal(data)
+
+
 @pytest.mark.parametrize(
     ['value', 'expected_type'],
     [
@@ -263,17 +270,6 @@ def test_literal_array():
     assert expr.type().equals(dt.Array(dt.null))
 
 
-def test_mixed_arity(table):
-    what = ["bar", table.g, "foo"]
-    expr = api.sequence(what)
-
-    assert isinstance(expr, tuple)
-    assert isinstance(expr[1], ops.TableColumn)
-
-    # it works!
-    repr(expr)
-
-
 @pytest.mark.parametrize('container', [list, tuple, set, frozenset])
 def test_isin_notin_list(table, container):
     values = container([1, 2, 3, 4])
@@ -1292,27 +1288,6 @@ def test_valid_negate(type):
     assert -expr is not None
 
 
-@pytest.mark.parametrize(
-    ('kind', 'begin', 'end'),
-    [
-        ('preceding', None, None),
-        ('preceding', 1, None),
-        ('preceding', -1, 1),
-        ('preceding', 1, -1),
-        ('preceding', -1, -1),
-        ('following', None, None),
-        ('following', None, 1),
-        ('following', -1, 1),
-        ('following', 1, -1),
-        ('following', -1, -1),
-    ],
-)
-def test_window_unbounded_invalid(kind, begin, end):
-    kwargs = {kind: (begin, end)}
-    with pytest.raises(com.IbisInputError):
-        ibis.window(**kwargs)
-
-
 @pytest.mark.parametrize(
     ('left', 'right', 'expected'),
     [
@@ -1534,24 +1509,12 @@ def test_non_null_with_null_precedence(expr_fn, expected_type):
     assert expr.type() == expected_type
 
 
-@pytest.mark.parametrize(
-    ("name", "expected"),
-    [
-        ("names", ("a", "b", "c")),
-        ("types", (dt.int8, dt.string, dt.Array(dt.Array(dt.float64)))),
-        (
-            "fields",
-            ibis.util.frozendict(
-                a=dt.int8,
-                b=dt.string,
-                c=dt.Array(dt.Array(dt.float64)),
-            ),
-        ),
-    ],
-)
-def test_struct_names_types_fields(name, expected):
+def test_struct_names_types_fields():
     s = ibis.struct(dict(a=1, b="2", c=[[1.0], [], [None]]))
-    assert getattr(s, name) == expected
+    assert s.names == ("a", "b", "c")
+    assert s.types == (dt.int8, dt.string, dt.Array(dt.Array(dt.float64)))
+    assert s.fields == dict(a=dt.int8, b=dt.string, c=dt.Array(dt.Array(dt.float64)))
+    assert isinstance(s.fields, frozendict)
 
 
 @pytest.mark.parametrize(
@@ -1576,6 +1539,24 @@ def test_array_length_scalar():
     assert isinstance(expr.op(), ops.ArrayLength)
 
 
+def double_int(x):
+    return x * 2
+
+
+def double_float(x):
+    return x * 2.0
+
+
+def test_array_map():
+    arr = ibis.array([1, 2, 3])
+
+    result_int = arr.map(double_int)
+    result_float = arr.map(double_float)
+
+    assert result_int.type() == dt.Array(dt.int16)
+    assert result_float.type() == dt.Array(dt.float64)
+
+
 @pytest.mark.parametrize(
     ("func", "expected_type"),
     [
@@ -1654,3 +1635,32 @@ def test_rowid_only_physical_tables():
     table[table.rowid(), table.x].filter(_.x > 10)  # works
     with pytest.raises(com.IbisTypeError, match="only valid for physical tables"):
         table.filter(table.x > 0).rowid()
+
+
+def test_where_output_shape():
+    # GH-5191
+    t = ibis.table(dict(a="int64", b="string"), name="t")
+    expr = ibis.literal(True).ifelse(t.a, -t.a)
+    assert isinstance(expr, ir.IntegerColumn)
+    assert isinstance(expr, ir.ColumnExpr)
+
+
+def test_quantile_shape():
+    t = ibis.table([("a", "float64")])
+
+    b1 = t.a.quantile(0.25).name("br2")
+    assert isinstance(b1, ir.Scalar)
+
+    projs = [b1]
+    expr = t.select(projs)
+    (b1,) = expr.op().selections
+
+    assert b1.output_shape.is_columnar()
+
+
+def test_sequence():
+    with pytest.warns(FutureWarning):
+        exprs = ibis.sequence([3, 2])
+
+    for expr in exprs:
+        assert isinstance(expr, ir.ScalarExpr)
