@@ -1,8 +1,9 @@
-SELECT t0.*
-FROM (
-  SELECT *
-  FROM star1
+WITH t0 AS (
+  SELECT t2.*
+  FROM star1 t2
   LIMIT 100
-) t0
+)
+SELECT t0.*
+FROM t0
   INNER JOIN star2 t1
     ON t0.`foo_id` = t1.`foo_id`
\ No newline at end of file
