@@ -1,2 +1,2 @@
 SELECT count(1) AS `count`
-FROM star1
\ No newline at end of file
+FROM star1 t0
\ No newline at end of file
