@@ -2,6 +2,7 @@ import pytest
 from pytest import param
 
 import ibis
+from ibis import _
 from ibis.backends.base.sql.compiler import Compiler
 from ibis.tests.sql.conftest import get_query, to_sql
 from ibis.tests.util import assert_decompile_roundtrip
@@ -48,7 +49,7 @@ def test_select_sql(alltypes, star1, expr_fn, snapshot):
 def test_nameless_table(snapshot):
     # Generate a unique table name when we haven't passed on
     nameless = ibis.table([('key', 'string')])
-    assert to_sql(nameless) == f'SELECT *\nFROM {nameless.op().name}'
+    assert to_sql(nameless) == f'SELECT t0.*\nFROM {nameless.op().name} t0'
 
     expr = ibis.table([('key', 'string')], name='baz')
     snapshot.assert_match(to_sql(expr), "out.sql")
@@ -221,16 +222,6 @@ def test_aggregate_count_joined(con, snapshot):
     assert_decompile_roundtrip(expr, snapshot)
 
 
-def test_no_aliases_needed():
-    table = ibis.table([('key1', 'string'), ('key2', 'string'), ('value', 'double')])
-
-    expr = table.aggregate([table['value'].sum().name('total')], by=['key1', 'key2'])
-
-    query = get_query(expr)
-    context = query.context
-    assert not context.need_aliases()
-
-
 def test_fuse_projections(snapshot):
     table = ibis.table(
         [('foo', 'int32'), ('bar', 'int64'), ('value', 'double')],
@@ -823,3 +814,33 @@ def test_sort_then_group_by_propagates_keys(snapshot):
     # b IS in the output's order by clause
     result = t.order_by("b").b.value_counts()
     snapshot.assert_match(to_sql(result), "result2.sql")
+
+
+def test_incorrect_predicate_pushdown(snapshot):
+    t = ibis.table({"x": int}, name="t")
+    result = t.mutate(x=_.x + 1).filter(_.x > 1)
+    snapshot.assert_match(to_sql(result), "result.sql")
+
+
+def test_incorrect_predicate_pushdown_with_literal(snapshot):
+    t = ibis.table(dict(a="int"), name="t")
+    expr = t.mutate(a=ibis.literal(1)).filter(lambda t: t.a > 1)
+    snapshot.assert_match(to_sql(expr), "result.sql")
+
+
+def test_complex_union(snapshot):
+    def compute(t):
+        return (
+            t.select("diag", "status")
+            .mutate(diag=_.diag + 1)
+            .mutate(diag=_.diag.cast("int32"))
+        )
+
+    # schema subset from ibis.examples.Aids2
+    schema = ibis.schema(dict(diag="int64", status="string"))
+
+    t1 = compute(ibis.table(schema, name="aids2_one"))
+    t2 = compute(ibis.table(schema, name="aids2_two"))
+
+    u = ibis.union(t1, t2)
+    snapshot.assert_match(to_sql(u), "result.sql")
