@@ -37,27 +37,6 @@ def test_flatten_invalid_input(case):
         list(flat)
 
 
-def test_dotdict():
-    d = util.DotDict({"a": 1, "b": 2, "c": 3})
-    assert d["a"] == d.a == 1
-    assert d["b"] == d.b == 2
-
-    d.b = 3
-    assert d.b == 3
-    assert d["b"] == 3
-
-    del d.c
-    assert not hasattr(d, "c")
-    assert "c" not in d
-
-    assert repr(d) == "DotDict({'a': 1, 'b': 3})"
-
-    with pytest.raises(KeyError):
-        assert d['x']
-    with pytest.raises(AttributeError):
-        assert d.x
-
-
 def test_import_object():
     import collections
 
@@ -69,3 +48,6 @@ def test_import_object():
 
     with pytest.raises(ImportError):
         util.import_object("collections.this_attribute_doesnt_exist")
+
+
+# TODO(kszucs): add tests for promote_list and promote_tuple
